@@ -53,12 +53,11 @@ alljoyn_busattachment g_bus;
 static void printAboutData(alljoyn_aboutdata aboutData, const char* language, int tabNum) {
     size_t count = alljoyn_aboutdata_getfields(aboutData, NULL, 0);
     size_t i = 0;
-    int j = 0;
     size_t k = 0;
     const char** fields = (const char**) malloc(sizeof(char*) * count);
     alljoyn_aboutdata_getfields(aboutData, fields, count);
     for (i = 0; i < count; ++i) {
-        for (j = 0; j < tabNum; ++j) {
+        for (int j = 0; j < tabNum; ++j) {
             printf("\t");
         }
         printf("Key: %s", fields[i]);
@@ -341,10 +340,8 @@ static void destroy_my_alljoyn_aboutlistener(my_about_listener* listener)
     }
 }
 
-int CDECL_CALL main(int argc, char** argv)
+int CDECL_CALL main(void)
 {
-    QCC_UNUSED(argc);
-    QCC_UNUSED(argv);
     /* Install SIGINT handler so Ctrl + C deallocates memory properly */
     signal(SIGINT, sig_int_handler);
 
