@@ -257,6 +257,21 @@ QStatus AJ_CALL alljoyn_busattachment_enablepeersecurity(alljoyn_busattachment b
                                                            (isShared == QCC_TRUE ? true : false));
 }
 
+QStatus AJ_CALL alljoyn_busattachment_enablepeersecuritywithpermissionconfigurationlistener(alljoyn_busattachment bus,
+                                                                                            const char* authMechanisms,
+                                                                                            alljoyn_authlistener authListener,
+                                                                                            const char* keyStoreFileName,
+                                                                                            QCC_BOOL isShared,
+                                                                                            alljoyn_permissionconfigurationlistener permissionConfigurationListener)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return ((ajn::BusAttachmentC*)bus)->EnablePeerSecurity(authMechanisms,
+                                                           (ajn::AuthListener*)authListener,
+                                                           keyStoreFileName,
+                                                           (isShared == QCC_TRUE ? true : false),
+                                                           (ajn::PermissionConfigurationListener*)permissionConfigurationListener);
+}
+
 QCC_BOOL AJ_CALL alljoyn_busattachment_ispeersecurityenabled(alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -439,6 +454,18 @@ QStatus AJ_CALL alljoyn_busattachment_setlinktimeoutasync(alljoyn_busattachment
                                                             (void*)new ajn::SetLinkTimeoutContext(callback, context));
 }
 
+QStatus AJ_CALL alljoyn_busattachment_secureconnection(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return ((ajn::BusAttachmentC*)bus)->SecureConnection(name, forceAuth);
+}
+
+QStatus AJ_CALL alljoyn_busattachment_secureconnectionasync(alljoyn_busattachment bus, const char* name, QCC_BOOL forceAuth)
+{
+    QCC_DbgTrace(("%s", __FUNCTION__));
+    return ((ajn::BusAttachmentC*)bus)->SecureConnectionAsync(name, forceAuth);
+}
+
 QStatus AJ_CALL alljoyn_busattachment_namehasowner(alljoyn_busattachment bus, const char* name, QCC_BOOL* hasOwner)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
@@ -530,10 +557,6 @@ QStatus AJ_CALL alljoyn_busattachment_ping(alljoyn_busattachment bus, const char
     return ((ajn::BusAttachmentC*)bus)->Ping(name, timeout);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 void AJ_CALL alljoyn_busattachment_registeraboutlistener(alljoyn_busattachment bus,
                                                          alljoyn_aboutlistener aboutListener)
 {
@@ -541,10 +564,6 @@ void AJ_CALL alljoyn_busattachment_registeraboutlistener(alljoyn_busattachment b
     return ((ajn::BusAttachmentC*)bus)->RegisterAboutListener(*(ajn::AboutListener*)aboutListener);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 void AJ_CALL alljoyn_busattachment_unregisteraboutlistener(alljoyn_busattachment bus,
                                                            alljoyn_aboutlistener aboutListener)
 {
@@ -552,20 +571,12 @@ void AJ_CALL alljoyn_busattachment_unregisteraboutlistener(alljoyn_busattachment
     return ((ajn::BusAttachmentC*)bus)->UnregisterAboutListener(*(ajn::AboutListener*)aboutListener);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 void AJ_CALL alljoyn_busattachment_unregisterallaboutlisteners(alljoyn_busattachment bus)
 {
     QCC_DbgTrace(("%s", __FUNCTION__));
     return ((ajn::BusAttachmentC*)bus)->UnregisterAllAboutListeners();
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 QStatus AJ_CALL alljoyn_busattachment_whoimplements_interfaces(alljoyn_busattachment bus,
                                                                const char** implementsInterfaces,
                                                                size_t numberInterfaces)
@@ -574,10 +585,6 @@ QStatus AJ_CALL alljoyn_busattachment_whoimplements_interfaces(alljoyn_busattach
     return ((ajn::BusAttachmentC*)bus)->WhoImplements(implementsInterfaces, numberInterfaces);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 QStatus AJ_CALL alljoyn_busattachment_whoimplements_interface(alljoyn_busattachment bus,
                                                               const char* implementsInterface)
 {
@@ -585,10 +592,6 @@ QStatus AJ_CALL alljoyn_busattachment_whoimplements_interface(alljoyn_busattachm
     return ((ajn::BusAttachmentC*)bus)->WhoImplements(implementsInterface);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfaces(alljoyn_busattachment bus,
                                                                      const char** implementsInterfaces,
                                                                      size_t numberInterfaces)
@@ -597,10 +600,6 @@ QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interfaces(alljoyn_bus
     return ((ajn::BusAttachmentC*)bus)->CancelWhoImplements(implementsInterfaces, numberInterfaces);
 }
 
-/**
- * This function is experimental, and as such has not been fully tested.
- * Please help make it more robust by contributing fixes if you find problems.
- */
 QStatus AJ_CALL alljoyn_busattachment_cancelwhoimplements_interface(alljoyn_busattachment bus,
                                                                     const char* implementsInterface)
 {
