@@ -61,25 +61,45 @@ extern const char* WellKnownName;     /**< Well known bus name */
 /** Interface definitions for org.alljoyn.Bus */
 namespace Bus {
 
-extern const char* ErrorName;                     /**< Standard AllJoyn error name */
-extern const char* ObjectPath;                    /**< Object path */
-extern const char* InterfaceName;                 /**< Interface name */
-extern const char* WellKnownName;                 /**< Well known bus name */
-extern const char* Secure;                        /**< Secure interface annotation */
+extern const char* ErrorName;     /**< Standard AllJoyn error name */
+extern const char* ObjectPath;    /**< Object path */
+extern const char* InterfaceName; /**< Interface name */
+extern const char* WellKnownName; /**< Well known bus name */
+extern const char* Secure;        /**< Secure interface annotation */
+
+/** Interface definitions for org.alljoyn.Bus.Application */
+namespace Application {
+extern const char* InterfaceName; /**< Interface name */
+}
 
 /** Interface definitions for org.alljoyn.Bus.Peer.* */
 namespace Peer {
-extern const char* ObjectPath;                         /**< Object path */
+extern const char* ObjectPath;    /**< Object path */
 namespace HeaderCompression {
-extern const char* InterfaceName;                      /**<Interface name */
+extern const char* InterfaceName; /**< Interface name */
 }
 namespace Authentication {
-extern const char* InterfaceName;                      /**<Interface name */
+extern const char* InterfaceName; /**< Interface name */
 }
 namespace Session {
-extern const char* InterfaceName;                      /**<Interface name */
+extern const char* InterfaceName; /**< Interface name */
 }
 }
+
+/** Interface definitions for org.alljoyn.Bus.Security */
+namespace Security {
+extern const char* ObjectPath; /**< Object path */
+namespace Application {
+extern const char* InterfaceName; /**< Interface name */
+}
+namespace ClaimableApplication {
+extern const char* InterfaceName; /**< Interface name */
+}
+namespace ManagedApplication {
+extern const char* InterfaceName; /**< Interface name */
+}
+}
+
 }
 
 /** Interface definitions for org.alljoyn.Daemon */
@@ -100,7 +120,6 @@ QStatus CreateInterfaces(BusAttachment& bus);          /**< Create the org.alljo
 }
 
 namespace allseen {
-
 /** Interface definitions for org.allseen.Introspectable */
 namespace Introspectable {
 
@@ -108,9 +127,11 @@ extern const char* IntrospectDocType;                 /**< Type of extended (wit
 extern const char* InterfaceName;                 /**< Interface name */
 
 }
-}
+
 
 }
+}
+
 
 /**
  * @anchor BindSessionPortReplyAnchor
@@ -121,7 +142,7 @@ extern const char* InterfaceName;                 /**< Interface name */
  * Create a named session for other bus nodes to join.
  *
  * In params:
- *  inPort       - Session Port number to bind to or SESSION_PORT_ANY to have router allocate an available port number.
+ *  inPort       - Session Port number to bind to (see below for reserved values and range) or SESSION_PORT_ANY to have router allocate an available port number.
  *  isMultipoint - true iff session supports more than two participants.
  *  opts         - Session options
  *
@@ -131,6 +152,9 @@ extern const char* InterfaceName;                 /**< Interface name */
  */
 // @{
 /* org.alljoyn.Bus.BindSessionPort */
+/* The router session port range is from 1 to 999 */
+#define ALLJOYN_SESSIONPORT_SESSIONLESS      100 /**< BindSessionPort port reserved for the SessionlessObj to connect */
+#define ALLJOYN_SESSIONPORT_PERMISSION_MGMT  101 /**< BindSessionPort port reserved for the PermissionMgmt object */
 #define ALLJOYN_BINDSESSIONPORT_REPLY_SUCCESS         1   /**< BindSessionPort reply: Success */
 #define ALLJOYN_BINDSESSIONPORT_REPLY_ALREADY_EXISTS  2   /**< BindSessionPort reply: SessionPort already exists */
 #define ALLJOYN_BINDSESSIONPORT_REPLY_FAILED          3   /**< BindSessionPort reply: Failed */
