@@ -25,6 +25,8 @@
 #error Only include BusAttachment.h in C++ code.
 #endif
 
+#include <set>
+
 #include <qcc/platform.h>
 
 #include <qcc/String.h>
@@ -41,6 +43,10 @@
 #include <alljoyn/SessionPortListener.h>
 #include <alljoyn/Status.h>
 #include <alljoyn/Translator.h>
+#include <alljoyn/ApplicationStateListener.h>
+#include <alljoyn/PermissionPolicy.h>
+#include <alljoyn/PermissionConfigurator.h>
+#include <alljoyn/PermissionConfigurationListener.h>
 
 namespace ajn {
 
@@ -153,6 +159,41 @@ class BusAttachment : public MessageReceiver {
         virtual void GetNameOwnerCB(QStatus status, const char* uniqueName, void* context) = 0;
     };
 
+    /**
+     * Pure virtual base class implemented by classes that wish to call AddMatchAsync().
+     */
+    class AddMatchAsyncCB {
+      public:
+        /** Destructor */
+        virtual ~AddMatchAsyncCB() { }
+
+        /**
+         * Called when AddMatchAsyncCB() completes.
+         *
+         * @param status        ER_OK if successful
+         * @param context       User defined context which will be passed as-is to callback.
+         */
+        virtual void AddMatchCB(QStatus status, void* context) = 0;
+    };
+
+
+    /**
+     * Pure virtual base class implemented by classes that wish to call RemoveMatchAsync().
+     */
+    class RemoveMatchAsyncCB {
+      public:
+        /** Destructor */
+        virtual ~RemoveMatchAsyncCB() { }
+
+        /**
+         * Called when RemoveMatchAsyncCB() completes.
+         *
+         * @param status        ER_OK if successful
+         * @param context       User defined context which will be passed as-is to callback.
+         */
+        virtual void RemoveMatchCB(QStatus status, void* context) = 0;
+    };
+
     /**
      * Construct a BusAttachment.
      *
@@ -492,7 +533,7 @@ class BusAttachment : public MessageReceiver {
      * will be ignored and the bundled router connectSpec will be used.  Use
      * Disconnect() instead which will use correct connectSpec.
      *
-     * @param connectSpec  The transport connection spec used to connect.
+     * @param disconnectConnectSpec  The transport connection spec used to connect.
      *
      * @return
      *          - #ER_OK if successful
@@ -500,7 +541,7 @@ class BusAttachment : public MessageReceiver {
      *          - #ER_BUS_NOT_CONNECTED if the %BusAttachment is not connected to the bus
      *          - Other error status codes indicating a failure
      */
-    QCC_DEPRECATED(QStatus Disconnect(const char* connectSpec));
+    QCC_DEPRECATED(QStatus Disconnect(const char* disconnectConnectSpec));
 
     /**
      * %Disconnect the %BusAttachment from the remote bus.
@@ -728,36 +769,36 @@ class BusAttachment : public MessageReceiver {
      * own key store implementation it must have already called RegisterKeyStoreListener() before
      * calling this function.
      *
-     * Once peer security has been enabled it is not possible to change the authMechanism set without
-     * clearing it first (setting authMechanism to NULL). This is true regardless of whether the BusAttachment
-     * has been disconnected or not.
+     * This method can be called multiple times with different auth mechanisms.
      *
-     * @param authMechanisms   The authentication mechanism(s) to use for peer-to-peer authentication.
-     *                         If this parameter is NULL peer-to-peer authentication is disabled.  This is a
-     *                         space separated list of any of the following values:
-     *                         ALLJOYN_SRP_LOGON, ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK,
-     *                         ALLJOYN_ECDHE_ECDSA, GSSAPI.
+     * @param authMechanisms       The authentication mechanism(s) to use for peer-to-peer authentication.
+     *                             If this parameter is NULL peer-to-peer authentication is disabled.  This is a
+     *                             space separated list of any of the following values:
+     *                             ALLJOYN_SRP_LOGON, ALLJOYN_SRP_KEYX, ALLJOYN_ECDHE_NULL, ALLJOYN_ECDHE_PSK,
+     *                             ALLJOYN_ECDHE_ECDSA, GSSAPI.
      *
-     * @param listener         Passes password and other authentication related requests to the application.
+     * @param authListener         Passes password and other authentication related requests to the application.
      *
-     * @param keyStoreFileName Optional parameter to specify the filename of the default key store. The
-     *                         default value is the applicationName parameter of BusAttachment().
-     *                         Note that this parameter is only meaningful when using the default
-     *                         key store implementation.
+     * @param keyStoreFileName     Optional parameter to specify the filename of the default key store. The
+     *                             default value is the applicationName parameter of BusAttachment().
+     *                             Note that this parameter is only meaningful when using the default
+     *                             key store implementation.
      *
-     * @param isShared         optional parameter that indicates if the key store is shared between multiple
-     *                         applications. It is generally harmless to set this to true even when the
-     *                         key store is not shared but it adds some unnecessary calls to the key store
-     *                         listener to load and store the key store in this case.
+     * @param isShared             optional parameter that indicates if the key store is shared between multiple
+     *                             applications. It is generally harmless to set this to true even when the
+     *                             key store is not shared but it adds some unnecessary calls to the key store
+     *                             listener to load and store the key store in this case.
+     * @param permissionConfigurationListener   Passes security 2.0 callbacks to the application.
      *
      * @return
      *      - #ER_OK if peer security was enabled.
      *      - #ER_BUS_BUS_NOT_STARTED BusAttachment::Start has not be called
      */
     QStatus EnablePeerSecurity(const char* authMechanisms,
-                               AuthListener* listener,
+                               AuthListener* authListener,
                                const char* keyStoreFileName = NULL,
-                               bool isShared = false);
+                               bool isShared = false,
+                               PermissionConfigurationListener* permissionConfigurationListener = NULL);
 
     /**
      * Check is peer security has been enabled for this bus attachment.
@@ -766,7 +807,6 @@ class BusAttachment : public MessageReceiver {
      */
     bool IsPeerSecurityEnabled();
 
-
     /**
      * Register an object that will receive bus event notifications.
      *
@@ -926,6 +966,24 @@ class BusAttachment : public MessageReceiver {
      */
     QStatus AddMatch(const char* rule);
 
+    /**
+     * Add a DBus match rule asynchronously.
+     *
+     * This method is a shortcut/helper that issues an org.freedesktop.DBus.AddMatch method call to the local router.
+     *
+     * @param[in]  rule     Match rule to be added (see DBus specification for format of this string).
+     * @param[in]  callback The object to be called when AddMatchAsync completes.
+     * @param[in]  context  User-defined context that will be returned to the reply handler
+     *
+     * @return
+     *      - #ER_OK if the AddMatch request was successful.
+     *      - #ER_BUS_NOT_CONNECTED if a connection has not been made with a local bus.
+     *      - Other error status codes indicating a failure.
+     */
+    QStatus AddMatchAsync(const char* rule,
+                          AddMatchAsyncCB* callback,
+                          void* context = NULL);
+
     /**
      * Remove a DBus match rule.
      * This method is a shortcut/helper that issues an org.freedesktop.DBus.RemoveMatch method call to the local router.
@@ -939,6 +997,23 @@ class BusAttachment : public MessageReceiver {
      */
     QStatus RemoveMatch(const char* rule);
 
+    /**
+     * Remove a DBus match rule.
+     * This method is a shortcut/helper that issues an org.freedesktop.DBus.RemoveMatch method call to the local router.
+     *
+     * @param[in]  rule     Match rule to be removed (see DBus specification for format of this string).
+     * @param[in]  callback The object to be called when RemoveMatchAsync completes.
+     * @param[in]  context  User-defined context that will be returned to the reply handler
+     *
+     * @return
+     *      - #ER_OK if the RemoveMatch request was successful.
+     *      - #ER_BUS_NOT_CONNECTED if a connection has not been made with a local bus.
+     *      - Other error status codes indicating a failure.
+     */
+    QStatus RemoveMatchAsync(const char* rule,
+                             RemoveMatchAsyncCB* callback,
+                             void* context = NULL);
+
     /**
      * Add a DBus match rule.
      *
@@ -1405,6 +1480,50 @@ class BusAttachment : public MessageReceiver {
                                 BusAttachment::SetLinkTimeoutAsyncCB* callback,
                                 void* context = NULL);
 
+    /**
+     * Explicitly secure the connection to the remote peer. Peer-to-peer
+     * connections can only be secured if EnablePeerSecurity() was previously called on the bus
+     * attachment. If the peer-to-peer connection is already secure this
+     * function does nothing. Note that peer-to-peer connections are automatically secured when a
+     * method call requiring encryption is sent.
+     *
+     * This call causes messages to be sent on the bus, therefore it cannot be called within AllJoyn
+     * callbacks (method/signal/reply handlers or ObjectRegistered callbacks, etc.)
+     *
+     * @param[in]  name       The unique name of the remote peer or NULL to secure the connections to all peers
+     *                        this BusAttachment is in session with.
+     * @param[in]  forceAuth  If true, forces re-authentication even if the peer connection is already
+     *                        authenticated.
+     *
+     * @return
+     *          - #ER_OK if the connection was secured or an error status indicating that the
+     *            connection could not be secured.
+     *          - #ER_BUS_NO_AUTHENTICATION_MECHANISM if BusAttachment::EnablePeerSecurity() has not been called.
+     *          - #ER_AUTH_FAIL if the attempt(s) to authenticate the peer failed.
+     *          - Other error status codes indicating a failure.
+     */
+    QStatus SecureConnection(const char* name, bool forceAuth = false);
+
+    /**
+     * Asynchronously secure the connection to the remote peer. Peer-to-peer connections can only
+     * be secured if EnablePeerSecurity() was previously called on the bus attachment.
+     * If the peer-to-peer connection is already secure this function does nothing.
+     * Note that peer-to-peer connections are automatically secured when a
+     * method call requiring encryption is sent.
+     *
+     * Notification of success or failure is via the AuthListener passed to EnablePeerSecurity().
+     *
+     * @param[in]  name      The unique name of the remote peer or NULL to secure the connections to all peers
+     *                       this BusAttachment is in session with.
+     * @param[in] forceAuth  If true, forces re-authentication even if the peer connection is already
+     *                       authenticated.
+     * @return
+     *          - #ER_OK if securing could begin.
+     *          - #ER_BUS_NO_AUTHENTICATION_MECHANISM if BusAttachment::EnablePeerSecurity() has not been called.
+     *          - Other error status codes indicating a failure.
+     */
+    QStatus SecureConnectionAsync(const char* name, bool forceAuth = false);
+
     /**
      * Determine whether a given well-known name exists on the bus.
      * This method is a shortcut/helper that issues an org.freedesktop.DBus.NameHasOwner method call to the router
@@ -1534,9 +1653,9 @@ class BusAttachment : public MessageReceiver {
 
     /**
      * Set a Translator for all BusObjects and InterfaceDescriptions. This Translator is used for
-     * descriptions appearing in introspection. Note that any Translators set on a specific
-     * InterfaceDescription or BusObject will be used for those specific elements - this Translator
-     * is used only for BusObjects and InterfaceDescriptions that do not have Translators set for them.
+     * descriptions appearing in introspection, and localizable text in about data. Note that any Translators
+     * set on a specific InterfaceDescription or BusObject will be used for those specific elements - this
+     * Translator is used only for BusObjects and InterfaceDescriptions that do not have Translators set for them.
      *
      * @param[in]  translator       The Translator instance
      */
@@ -1549,6 +1668,13 @@ class BusAttachment : public MessageReceiver {
      */
     Translator* GetDescriptionTranslator();
 
+    /**
+     * Get the permission configurator for the bus attachment.
+     * @return the permission configurator
+     */
+
+    PermissionConfigurator& GetPermissionConfigurator();
+
     /**
      * Registers a handler to receive the org.alljoyn.about Announce signal.
      *
@@ -1562,7 +1688,7 @@ class BusAttachment : public MessageReceiver {
     void RegisterAboutListener(AboutListener& aboutListener);
 
     /**
-     * Unregisters the AnnounceHandler from receiving the org.alljoyn.about Announce signal.
+     * Unregisters the handler from receiving the org.alljoyn.about Announce signal.
      *
      * @param[in] aboutListener reference to AboutListener to unregister
      */
@@ -1747,6 +1873,50 @@ class BusAttachment : public MessageReceiver {
      */
     QStatus CancelWhoImplementsNonBlocking(const char* iface);
 
+    /**
+     * Registers a handler to receive the org.alljoyn.Bus.Application
+     * State signal.
+     *
+     * @param[in] applicationStateListener reference to an ApplicationStateListener
+     */
+    void RegisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
+
+    /**
+     * Unregisters the ApplicationStateListener from receiving the
+     * org.alljoyn.Bus.Application State signal.
+     *
+     * @param[in] applicationStateListener reference to an
+     *                                     ApplicationStateListener to
+     *                                     unregister
+     */
+    void UnregisterApplicationStateListener(ApplicationStateListener& applicationStateListener);
+
+    /**
+     * This is a helper function that will add the match rule responsible for
+     * receiving the org.alljoyn.Bus.Application State signal.
+     *
+     * The ApplicationStateListener should be registered before calling this method.
+     *
+     * This will only call AddMatch for the State signal.
+     *
+     * @return
+     *    - #ER_OK on success
+     *    - An error status otherwise
+     */
+    QStatus AddApplicationStateRule();
+
+    /**
+     * This is a helper function that will remove the match rule responsible for
+     * receiving the org.alljoyn.Bus.Application State signal.
+     *
+     * This will only call RemoveMatch for the State signal.
+     *
+     * @return
+     *    - #ER_OK on success
+     *    - An error status otherwise
+     */
+    QStatus RemoveApplicationStateRule();
+
     /// @cond ALLJOYN_DEV
     /**
      * @internal
@@ -1870,14 +2040,9 @@ class BusAttachment : public MessageReceiver {
     QStatus LeaveSession(const SessionId& sessionId, const char*method, SessionSideMask bitset);
 
     /**
-     * Clear session listeners for a particular session
+     * Clear session bookkeeping for a particular session
      */
-    void ClearSessionListener(SessionId sessionId, SessionSideMask bitset);
-
-    /**
-     * Remove references to session
-     */
-    void ClearSessionSet(SessionId sessionId, SessionSideMask bitset);
+    void ClearSession(SessionId sessionId, SessionSideMask bitset);
 
     /**
      * Register signal handlers for BusListener
@@ -1889,6 +2054,11 @@ class BusAttachment : public MessageReceiver {
      */
     void UnregisterSignalHandlers();
 
+    /**
+     * Common logic for securing a connection. Called by SecureConnection and SecureConnectionAsync.
+     */
+    QStatus SecureConnectionInternal(const char* name, bool forceAuth, bool async);
+
     qcc::String connectSpec;  /**< The connect spec used to connect to the bus */
     bool isStarted;           /**< Indicates if the bus has been started */
     bool isStopping;          /**< Indicates Stop has been called */
@@ -1935,7 +2105,7 @@ class BusAttachment : public MessageReceiver {
         BusAttachment* bus;
     };
 
-    Translator* translator;       /**< Global translator for descriptions */
+    Translator* translator;       /**< Global translator for localizable text */
 
     JoinObj joinObj;          /**< MUST BE LAST MEMBER. Ensure all threads are joined before BusAttachment destruction */
 
