@@ -51,8 +51,12 @@ static const uint8_t PROP_ANNOTATE_EMIT_CHANGED_SIGNAL_INVALIDATES = 2; /**< Emi
 // @}
 /** @name Method/Signal Annotation flags */
 // @{
-static const uint8_t MEMBER_ANNOTATE_NO_REPLY   = 1; /**< No reply annotate flag */
-static const uint8_t MEMBER_ANNOTATE_DEPRECATED = 2; /**< Deprecated annotate flag */
+static const uint8_t MEMBER_ANNOTATE_NO_REPLY         = 1; /**< No reply annotate flag */
+static const uint8_t MEMBER_ANNOTATE_DEPRECATED       = 2; /**< Deprecated annotate flag */
+static const uint8_t MEMBER_ANNOTATE_SESSIONCAST      = 4; /**< Sessioncast annotate flag */
+static const uint8_t MEMBER_ANNOTATE_SESSIONLESS      = 8; /**< Sessionless annotate flag */
+static const uint8_t MEMBER_ANNOTATE_UNICAST          = 16; /**< Unicast annotate flag */
+static const uint8_t MEMBER_ANNOTATE_GLOBAL_BROADCAST = 32; /**< Global broadcast annotate flag */
 // @}
 
 /**
@@ -117,7 +121,10 @@ class InterfaceDescription {
         qcc::String accessPerms;                    /**< Required permissions to invoke this call */
         qcc::String description;                    /**< Introspection description for this member */
         ArgumentDescriptions* argumentDescriptions; /**< Introspection descriptions for arguments to this member */
+        bool isSessioncastSignal;                   /**< True if this is described as a sessioncast signal */
         bool isSessionlessSignal;                   /**< True if this is described as a sessionless signal */
+        bool isUnicastSignal;                       /**< True if this is described as a unicast signal */
+        bool isGlobalBroadcastSignal;               /**< True if this is described as a global broadcast signal */
 
         /**
          * %Member constructor.
@@ -166,11 +173,11 @@ class InterfaceDescription {
         /**
          * Get this member's annotation value
          *
-         * @param        name   name of the annotation to look for
-         * @param[out]   value  The value of the annotation, if found
-         * @return    true iff annotations[name] == value
+         * @param        annotationName   name of the annotation to look for
+         * @param[out]   value            The value of the annotation, if found
+         * @return                        true iff annotations[name] == value
          */
-        bool GetAnnotation(const qcc::String& name, qcc::String& value) const;
+        bool GetAnnotation(const qcc::String& annotationName, qcc::String& value) const;
 
         /**
          * Equality. Two members are defined to be equal if their members are
@@ -232,11 +239,11 @@ class InterfaceDescription {
 
         /**
          * Get this property's annotation value
-         * @param name   name of the annotation to look for
-         * @param[out]   value  The value of the annotation, if found
-         * @return    true iff annotations[name] == value
+         * @param annotationName   name of the annotation to look for
+         * @param[out]             value  The value of the annotation, if found
+         * @return                 true iff annotations[name] == value
          */
-        bool GetAnnotation(const qcc::String& name, qcc::String& value) const;
+        bool GetAnnotation(const qcc::String& annotationName, qcc::String& value) const;
 
         /** Equality */
         bool operator==(const Property& o) const;
@@ -299,7 +306,7 @@ class InterfaceDescription {
     /**
      * Add a method call member to the interface.
      *
-     * @param name        Name of method call member.
+     * @param methodName  Name of method call member.
      * @param inputSig    Signature of input parameters or NULL for none.
      * @param outSig      Signature of output parameters or NULL for none.
      * @param argNames    Comma separated list of input and then output arg names used in annotation XML.
@@ -310,9 +317,9 @@ class InterfaceDescription {
      *      - #ER_OK if successful
      *      - #ER_BUS_MEMBER_ALREADY_EXISTS if member already exists
      */
-    QStatus AddMethod(const char* name, const char* inputSig, const char* outSig, const char* argNames, uint8_t annotation = 0, const char* accessPerms = 0)
+    QStatus AddMethod(const char* methodName, const char* inputSig, const char* outSig, const char* argNames, uint8_t annotation = 0, const char* accessPerms = 0)
     {
-        return AddMember(MESSAGE_METHOD_CALL, name, inputSig, outSig, argNames, annotation, accessPerms);
+        return AddMember(MESSAGE_METHOD_CALL, methodName, inputSig, outSig, argNames, annotation, accessPerms);
     }
 
     /**
@@ -344,21 +351,21 @@ class InterfaceDescription {
     /**
      * Lookup a member method description by name
      *
-     * @param name  Name of the method to lookup
+     * @param methodName  Name of the method to lookup
      * @return
      *      - Pointer to member.
      *      - NULL if does not exist.
      */
-    const Member* GetMethod(const char* name) const
+    const Member* GetMethod(const char* methodName) const
     {
-        const Member* method = GetMember(name);
+        const Member* method = GetMember(methodName);
         return (method && method->memberType == MESSAGE_METHOD_CALL) ? method : NULL;
     }
 
     /**
      * Add a signal member to the interface.
      *
-     * @param name        Name of method call member.
+     * @param signalName  Name of signal member.
      * @param sig         Signature of parameters or NULL for none.
      * @param argNames    Comma separated list of arg names used in annotation XML.
      * @param annotation  Annotation flags.
@@ -368,22 +375,40 @@ class InterfaceDescription {
      *      - #ER_OK if successful
      *      - #ER_BUS_MEMBER_ALREADY_EXISTS if member already exists
      */
-    QStatus AddSignal(const char* name, const char* sig, const char* argNames, uint8_t annotation = 0, const char* accessPerms = 0)
+    QStatus AddSignal(const char* signalName, const char* sig, const char* argNames, uint8_t annotation, const char* accessPerms = 0)
     {
-        return AddMember(MESSAGE_SIGNAL, name, sig, NULL, argNames, annotation, accessPerms);
+        return AddMember(MESSAGE_SIGNAL, signalName, sig, NULL, argNames, annotation, accessPerms);
+    }
+
+    /**
+     * Add a signal member to the interface.
+     *
+     * @deprecated Use annotation flags to specify signal type.
+     *
+     * @param signalName  Name of signal member.
+     * @param sig         Signature of parameters or NULL for none.
+     * @param argNames    Comma separated list of arg names used in annotation XML.
+     *
+     * @return
+     *      - #ER_OK if successful
+     *      - #ER_BUS_MEMBER_ALREADY_EXISTS if member already exists
+     */
+    QCC_DEPRECATED(QStatus AddSignal(const char* signalName, const char* sig, const char* argNames))
+    {
+        return AddMember(MESSAGE_SIGNAL, signalName, sig, NULL, argNames, 0, 0);
     }
 
     /**
      * Lookup a member signal description by name
      *
-     * @param name  Name of the signal to lookup
+     * @param signalName  Name of the signal to lookup
      * @return
      *      - Pointer to member.
      *      - NULL if does not exist.
      */
-    const Member* GetSignal(const char* name) const
+    const Member* GetSignal(const char* signalName) const
     {
-        const Member* method = GetMember(name);
+        const Member* method = GetMember(signalName);
         return (method && method->memberType == MESSAGE_SIGNAL) ? method : NULL;
     }
 
@@ -445,10 +470,10 @@ class InterfaceDescription {
     /**
      * Check for existence of a property.
      *
-     * @param name       Name of the property to lookup
+     * @param propertyName       Name of the property to lookup
      * @return true if the property exists.
      */
-    bool HasProperty(const char* name) const { return GetProperty(name) != NULL; }
+    bool HasProperty(const char* propertyName) const { return GetProperty(propertyName) != NULL; }
 
     /**
      * Check for existence of any properties
@@ -562,7 +587,7 @@ class InterfaceDescription {
     void SetDescriptionLanguage(const char* language);
 
     /**
-     * Get the language tag for the instrospection descriptions of this InterfaceDescription
+     * Get the language tag for the introspection descriptions of this InterfaceDescription
      *
      * @return The langauge tag
      */
@@ -585,14 +610,30 @@ class InterfaceDescription {
      *
      * @param member The name of the member
      * @param description The introspection description
+     * @return
+     *      - #ER_OK if successful.
+     *      - #ER_BUS_INTERFACE_ACTIVATED If the interface has already been activated
+     *      - #ER_BUS_INTERFACE_NO_SUCH_MEMBER If the member was not found
+     */
+    QStatus SetMemberDescription(const char* member, const char* description);
+
+    /**
+     * Set the introspection description for "member" of this InterfaceDescription
+     *
+     * @deprecated The isSessionlessSignal argument is deprecated. Use annotation
+     * flags with AddSignal() instead.
+     *
+     * @param member The name of the member
+     * @param description The introspection description
      * @param isSessionlessSignal True to document this member as a sessionless signal
      * @return
      *      - #ER_OK if successful.
      *      - #ER_BUS_INTERFACE_ACTIVATED If the interface has already been activated
      *      - #ER_BUS_INTERFACE_NO_SUCH_MEMBER If the member was not found
      */
-    QStatus SetMemberDescription(const char* member,
-                                 const char* description, bool isSessionlessSignal = false);
+    QCC_DEPRECATED(QStatus SetMemberDescription(const char* member,
+                                                const char* description,
+                                                bool isSessionlessSignal));
 
     /**
      * Set the introspection description for the argument "arg of "member" of this InterfaceDescription
