@@ -529,14 +529,14 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
      * @param endpoint  Bus-to-bus endpoint to add.
      * @param ER_OK if successful
      */
-    QStatus AddBusToBusEndpoint(RemoteEndpoint& endpoint);
+    virtual QStatus AddBusToBusEndpoint(RemoteEndpoint& endpoint);
 
     /**
      * Remove an existing Bus-to-bus endpoint.
      *
      * @param endpoint  Bus-to-bus endpoint to add.
      */
-    void RemoveBusToBusEndpoint(RemoteEndpoint& endpoint);
+    virtual void RemoveBusToBusEndpoint(RemoteEndpoint& endpoint);
 
     /**
      * Respond to a remote daemon request to attach a session through this daemon.
@@ -666,7 +666,7 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
         qcc::ThreadReturn STDCALL Run(void* arg);
 
       private:
-        static int jstCount;
+        static volatile int32_t jstCount;
         qcc::ThreadReturn STDCALL RunAttach();
         /*
          * This must be called with the locks as it looks through the various advertisement maps.
@@ -791,9 +791,7 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
      * @return  ER_OK if successful.
      */
     virtual QStatus AddSessionRoute(SessionId id, BusEndpoint& srcEp, RemoteEndpoint* srcB2bEp, BusEndpoint& destEp,
-                                    RemoteEndpoint& destB2bEp) {
-        return router.AddSessionRoute(id, srcEp, srcB2bEp, destEp, destB2bEp);
-    }
+                                    RemoteEndpoint& destB2bEp);
 
     /// @endcond
 
@@ -864,6 +862,7 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
         std::vector<qcc::String> memberNames;
         bool isInitializing;
         bool isRawReady;
+        bool isActive;                   /* This is used to mark multipoint sessions as inactive once the session host leaves. */
         bool IsSelfJoin() const {
             return find(memberNames.begin(), memberNames.end(), sessionHost) != memberNames.end();
         }
@@ -894,7 +893,8 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
             opts(),
             fd(qcc::INVALID_SOCKET_FD),
             isInitializing(false),
-            isRawReady(false) { }
+            isRawReady(false),
+            isActive(true) { }
     };
 
     typedef std::multimap<std::pair<qcc::String, SessionId>, SessionMapEntry> SessionMapType;
@@ -931,7 +931,7 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
      */
     void SessionMapErase(SessionMapEntry& sme);
 
-    const qcc::GUID128& guid;                                  /**< Global GUID of this daemon */
+    const qcc::GUID128& daemonGuid;                            /**< Global GUID of this daemon */
 
     const InterfaceDescription::Member* detachSessionSignal;   /**< org.alljoyn.Daemon.DetachSession signal member */
 
@@ -1178,7 +1178,7 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
     }
 
     /**
-     * Helper method used to shutdown a remote endpoint while preserving it's open file descriptor.
+     * Helper method used to shutdown a remote endpoint while preserving its open file descriptor.
      * This is used for converting a RemoteEndpoint into a raw streaming socket.
      *
      * @param b2bEp    Bus to bus endpoint being shutdown.
@@ -1276,6 +1276,24 @@ class AllJoynObj : public BusObject, public NameListener, public TransportListen
      */
     bool IsMemberOfSession(BusEndpoint endpoint, uint32_t sessionId);
 
+    /* Add a session ref to the VirtualEndpoint
+     * @param  vep Virtual endpoint to which a ref needs to be added.
+     * @param  id Id of the session
+     * @param  b2bEp B2b endpoint of the session
+     * @return ER_OK if successful
+     */
+    QStatus AddSessionRef(VirtualEndpoint vep, SessionId id, RemoteEndpoint b2bEp);
+
+    /* Remove a session ref to the virtualendpoint with the specified name
+     * @param  vepName Name of virtual endpoint to which a ref needs to be decremented.
+     * @param  id Id of the session
+     */
+    void RemoveSessionRef(qcc::String vepName, SessionId id);
+
+    /* Clean up session references from endpoints for a particular session.
+     * @param  sme: The session being torn down
+     */
+    void CleanupSessionEndpoints(SessionMapEntry sme);
 };
 
 }
