@@ -49,7 +49,7 @@
 
 #include "ns/IpNameService.h"
 
-#define WORKAROUND_1298 1 /**< Implement workaround for ASACORE-1298 */
+#define WORKAROUND_1298 0 /**< Implement workaround for ASACORE-1298 */
 
 namespace ajn {
 
@@ -353,14 +353,28 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
 
     MessagePump* m_messagePumps[N_PUMPS];                          /**< array of MessagePumps (with possiblly running pump threads) */
 
-    std::list<std::pair<qcc::String, qcc::SocketFd> > m_listenFds; /**< File descriptors the transport is listening on */
-    qcc::Mutex m_listenFdsLock;                                    /**< Mutex that protects m_listenFds */
+    class ListenFdEntry {
+      public:
+        ListenFdEntry(qcc::String normSpec, qcc::SocketFd sockFd) : m_normSpec(normSpec), m_sockFd(sockFd), m_remove(false) { }
+        qcc::String m_normSpec;  /**< The normalized listen spec for this entry */
+        qcc::SocketFd m_sockFd;  /**< The socket associated with this entry */
+        bool m_remove;           /**< If true, a request to stop listening on this liste spec has been received */
+    };
+
+    std::list<ListenFdEntry> m_listenFds; /**< Listen specs that the transport should be listening on */
+    qcc::Mutex m_listenFdsLock;             /**< Mutex that protects m_listenFds */
 
     std::list<qcc::String> m_listenSpecs;                          /**< Listen specs clients have requested us to listen on */
     qcc::Mutex m_listenSpecsLock;                                  /**< Mutex that protects m_listenSpecs */
 
     std::list<qcc::String> m_discovering;                          /**< Name prefixes the transport is looking for */
-    std::list<qcc::String> m_advertising;                          /**< Names the transport is advertising */
+
+    struct AdvEntry {
+        qcc::String name;                                          /** < The advertised name */
+        bool quietly;                                              /** < Whether the name is being advertised actively or quietly */
+        AdvEntry(qcc::String name, bool quietly) : name(name), quietly(quietly) { }
+    };
+    std::list<AdvEntry> m_advertising;                          /**< Names the transport is advertising */
     std::list<qcc::String> m_listening;                            /**< ListenSpecs on which the transport is listening */
     qcc::Mutex m_discoLock;                                        /**< Mutex that protects discovery and advertisement lists */
 
@@ -622,7 +636,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @warning This maximum is enforced on incoming connections from untrusted clients over udp.
      * This is to limit the amount of resources being used by untrusted clients over udp.
      */
-    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_UDP_DEFAULT = 0;
+    static const uint32_t ALLJOYN_MAX_REMOTE_CLIENTS_UDP_DEFAULT = 16;
 
     /**
      * @brief The default value for the router advertisement prefix that untrusted thin clients
@@ -659,11 +673,6 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
         DISABLE_DISCOVERY  /**< A request to cancel a discovery has been received */
     };
 
-    enum AdvertiseOp {
-        ENABLE_ADVERTISEMENT,  /**< A request to start advertising has been received */
-        DISABLE_ADVERTISEMENT  /**< A request to cancel advertising has been received */
-    };
-
     enum ListenOp {
         START_LISTEN,  /**< A request to start listening has been received */
         STOP_LISTEN    /**< A request to stop listening has been received */
@@ -687,13 +696,30 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
     bool NewDiscoveryOp(DiscoveryOp op, qcc::String namePrefix, bool& isFirst);
 
     /**
-     * @brief Add or remove an advertisement indication.
+     * @brief Add an advertisement indication.
      *
      * Called when the transport has received a new advertisement operation.
-     * This will either be an EnableAdvertisement() or DisbleAdvertisement()
-     * discriminated by the AdvertiseOp enum.
+     * This will be an EnableAdvertisement().
      *
-     * We want to keep a list of names that are currently being advertised.  The
+     * We want to keep a list of names and the type(quietly/actively)
+     * that are currently being advertised.  The
+     * presence of a non-zero size of this list indicates that at least one
+     * advertisement is in-process and the Name Service should be kept alive to
+     * respond to WHO_HAS queries.
+     *
+     * @return true if the list of advertisements was empty before the
+     *              operation.
+     */
+    bool NewAdvertiseOp(qcc::String name, bool quietly);
+
+    /**
+     * @brief Cancel an existing advertisement indication.
+     *
+     * Called when the transport has received a cancel advertisement operation.
+     * This will be an DisableAdvertisement().
+     *
+     * We want to keep a list of names and the type(quietly/actively)
+     * that are currently being advertised.  The
      * presence of a non-zero size of this list indicates that at least one
      * advertisement is in-process and the Name Service should be kept alive to
      * respond to WHO_HAS queries.
@@ -701,7 +727,9 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
      * @return true if the list of advertisements is empty as a result of the
      *              operation.
      */
-    bool NewAdvertiseOp(AdvertiseOp op, qcc::String name, bool& isFirst);
+    bool CancelAdvertiseOp(qcc::String name, bool& quietly);
+
+
 
     /**
      * @brief Add or remove a listen operation.
@@ -728,6 +756,9 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
     void QueueDisableAdvertisement(const qcc::String& advertiseName, TransportMask transports);
     void QueueUpdateRouterAdvertisementAndDynamicScore();
 
+    void ScheduleEndpointDeletedHook();
+    void EndpointDeletedHook();
+
     void RunListenMachine(ListenRequest& listenRequest);
 
     void StartListenInstance(ListenRequest& listenRequest);
@@ -739,12 +770,13 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
     void HandleNetworkEventInstance(ListenRequest& listenRequest);
     void UpdateDynamicScoreInstance(ListenRequest& listenRequest);
 
-    void UntrustedClientExit();
-    QStatus UntrustedClientStart();
+    void EnableDiscoveryListen();
+
     bool m_isAdvertising;
     bool m_isDiscovering;
     bool m_isListening;
     bool m_isNsEnabled;
+    volatile int32_t m_connecting;
 
     enum ReloadState {
         STATE_RELOADING = 0,    /**< The set of listen FDs has changed and needs to be reloaded by the main thread */
@@ -773,7 +805,7 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
 
     std::list<ListenRequest> m_pendingDiscoveries; /**< A list of discovery requests that came in while no interfaces were yet IFF_UP */
 
-    int32_t m_nsReleaseCount; /**< the number of times we have released the name service singleton */
+    volatile int32_t m_nsReleaseCount; /**< the number of times we have released the name service singleton */
 
     bool m_wildcardIfaceProcessed; /**< flag to avoid processing network events if we have a wildcard interface request */
 
@@ -891,7 +923,8 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
             CONNECT_CB,
             DISCONNECT_CB,
             RECV_CB,
-            SEND_CB
+            SEND_CB,
+            ENDPOINT_DELETED
         };
 
         Command m_command;
@@ -938,9 +971,9 @@ class UDPTransport : public Transport, public _RemoteEndpoint::EndpointListener,
     void ConnectCb(ArdpHandle* handle, ArdpConnRecord* conn, bool passive, uint8_t* buf, uint16_t len, QStatus status);
     void DoConnectCb(ArdpHandle* handle, ArdpConnRecord* conn, uint32_t connId, bool passive, uint8_t* buf, uint16_t len, QStatus status);
     void DisconnectCb(ArdpHandle* handle, ArdpConnRecord* conn, QStatus status);
-    void RecvCb(ArdpHandle* handle, ArdpConnRecord* conn, ArdpRcvBuf* rcv, QStatus status);
-    void SendCb(ArdpHandle* handle, ArdpConnRecord* conn, uint8_t* buf, uint32_t len, QStatus status);
-    void SendWindowCb(ArdpHandle* handle, ArdpConnRecord* conn, uint16_t window, QStatus status);
+    void RecvCb(ArdpHandle* ardpHandle, ArdpConnRecord* conn, ArdpRcvBuf* rcv, QStatus status);
+    void SendCb(ArdpHandle* ardpHandle, ArdpConnRecord* conn, uint8_t* buf, uint32_t len, QStatus status);
+    void SendWindowCb(ArdpHandle* ardpHandle, ArdpConnRecord* conn, uint16_t window, QStatus status);
     void ExitEndpoint(uint32_t connId);
 };
 
