@@ -45,12 +45,14 @@ class IpNameServiceListener {
     virtual ~IpNameServiceListener() { }
     virtual bool QueryHandler(TransportMask transport, MDNSPacket query, const qcc::IPEndpoint& src, const qcc::IPEndpoint& dst) {
         QCC_UNUSED(transport);
+        QCC_UNUSED(query);
         QCC_UNUSED(src);
         QCC_UNUSED(dst);
         return false;
     }
     virtual bool ResponseHandler(TransportMask transport, MDNSPacket response, uint16_t recvPort) {
         QCC_UNUSED(transport);
+        QCC_UNUSED(response);
         QCC_UNUSED(recvPort);
         return false;
     }
@@ -431,7 +433,7 @@ class IpNameService {
      *     advertisement be canceled.
      * @param wkn The well-known name to stop advertising.
      */
-    QStatus CancelAdvertiseName(TransportMask transportMask, const qcc::String& wkn, TransportMask completeTransportMask);
+    QStatus CancelAdvertiseName(TransportMask transportMask, const qcc::String& wkn, bool quietly, TransportMask completeTransportMask);
 
     /**
      * @brief Handle the suspending event of the process. Release exclusive socket file descriptor and port.
@@ -544,7 +546,7 @@ class IpNameService {
 
     bool m_constructed;          /**< State variable indicating the singleton has been constructed */
     bool m_destroyed;            /**< State variable indicating the singleton has been destroyed */
-    int32_t m_refCount;          /**< The number of transports that have registered as users of the singleton */
+    volatile int32_t m_refCount; /**< The number of transports that have registered as users of the singleton */
     IpNameServiceImpl* m_pimpl;  /**< A pointer to the private implementation of the name service */
 };
 
