@@ -537,7 +537,11 @@ class IpNameServiceImpl : public qcc::Thread {
 
     static uint32_t ComputeStaticScore(uint32_t powerSource, uint32_t mobility, uint32_t availability, uint32_t nodeConnection);
 
-    static uint32_t ComputeDynamicScore(uint32_t availableTcpConnections, uint32_t maximumTcpConnections, uint32_t availableUdpConnections, uint32_t maximumUdpConnections, uint32_t availableTcpRemoteClients, uint32_t maximumTcpRemoteClients);
+    static uint32_t ComputeDynamicScore(
+        uint32_t availableTcpConnections, uint32_t maximumTcpConnections,
+        uint32_t availableUdpConnections, uint32_t maximumUdpConnections,
+        uint32_t availableTcpRemoteClients, uint32_t maximumTcpRemoteClients,
+        uint32_t availableUdpRemoteClients, uint32_t maximumUdpRemoteClients);
 
     static qcc::String ToLowerCase(const qcc::String& str);
 
@@ -790,7 +794,7 @@ class IpNameServiceImpl : public qcc::Thread {
      *
      * @return Status of the operation.  Returns ER_OK on success.
      */
-    QStatus CancelAdvertiseName(TransportMask transportMask, const qcc::String& wkn, TransportMask completeTransportMask);
+    QStatus CancelAdvertiseName(TransportMask transportMask, const qcc::String& wkn, bool quietly, TransportMask completeTransportMask);
 
     /**
      * @internal
@@ -827,7 +831,7 @@ class IpNameServiceImpl : public qcc::Thread {
      *
      * @return Status of the operation.  Returns ER_OK on success.
      */
-    QStatus CancelAdvertiseName(TransportMask transportMask, std::vector<qcc::String>& wkn, TransportMask completeTransportMask);
+    QStatus CancelAdvertiseName(TransportMask transportMask, std::vector<qcc::String>& wkn, bool quietly, TransportMask completeTransportMask);
 
     /**
      * @brief Returns a count of the number of names currently being advertised
