@@ -59,6 +59,8 @@
 #include "BusController.h"
 #include "ConfigDB.h"
 
+#include "ConfigHelper.h"
+
 #if !defined(ROUTER_LIB)
 
 #if defined(QCC_OS_LINUX)
@@ -102,7 +104,10 @@ static const char defaultConfig[] =
     "  <limit name=\"max_completed_connections\">32</limit>"
     "  <limit name=\"max_remote_clients_tcp\">0</limit>"
     "  <limit name=\"max_remote_clients_udp\">0</limit>"
-    "  <flag name=\"restrict_untrusted_clients\">true</flag>"
+    "  <property name=\"router_power_source\">Battery powered and chargeable</property>"
+    "  <property name=\"router_mobility\">Intermediate mobility</property>"
+    "  <property name=\"router_availability\">3-6 hr</property>"
+    "  <property name=\"router_node_connection\">Wireless</property>"
     "</busconfig>";
 #else
 static const char defaultConfig[] =
@@ -111,28 +116,23 @@ static const char defaultConfig[] =
     "  <limit name=\"max_incomplete_connections\">16</limit>"
     "  <limit name=\"max_completed_connections\">32</limit>"
     "  <limit name=\"max_remote_clients_tcp\">16</limit>"
-    "  <limit name=\"max_remote_clients_udp\">0</limit>"
-    "  <property name=\"router_power_source\">Battery powered and chargeable</property>"
-    "  <property name=\"router_mobility\">Intermediate mobility</property>"
-    "  <property name=\"router_availability\">3-6 hr</property>"
-    "  <property name=\"router_node_connection\">Wireless</property>"
-    "  <flag name=\"restrict_untrusted_clients\">false</flag>"
+    "  <limit name=\"max_remote_clients_udp\">16</limit>"
     "</busconfig>";
 #endif
 
 /*
- * Router Power Source
+ * Options for router_power_source
  *  Always AC powered
  *  Battery powered and chargeable
  *  Battery powered and not chargeable
  *
- * Router Mobility
+ * Options for router_mobility
  *  Always Stationary
  *  Low mobility
  *  Intermediate mobility
  *  High mobility
  *
- * Router Availability
+ * Options for router_availability
  *  0-3 hr
  *  3-6 hr
  *  6-9 hr
@@ -142,11 +142,10 @@ static const char defaultConfig[] =
  *  18-21 hr
  *  21-24 hr
  *
- * Router Node Connection
+ * Options for router_node_connection
  *  Access Point
  *  Wired
  *  Wireless
- *
  */
 
 static const char internalConfig[] =
@@ -204,6 +203,7 @@ class OptParse {
         printAddressFd(-1), printPidFd(-1),
         internal(false),
         configService(false),
+        custom(false),
         verbosity(LOG_WARNING) {
     }
 
@@ -247,9 +247,21 @@ class OptParse {
     bool GetInternalConfig() const {
         return internal;
     }
+    bool GetCustomConfig() const {
+        return custom;
+    }
     bool GetServiceConfig() const {
         return configService;
     }
+    qcc::String GetCustomConfigPrettyXml() {
+        configHelper.Pretty();
+        return configHelper.Generate();
+    }
+
+    qcc::String GetCustomConfigXml() {
+        configHelper.Normal();
+        return configHelper.Generate();
+    }
 
   private:
     int argc;
@@ -269,6 +281,10 @@ class OptParse {
     int printPidFd;
     bool internal;
     bool configService;
+
+    bool custom;
+    ConfigHelper configHelper;
+
     int verbosity;
 
     void PrintUsage();
@@ -280,7 +296,7 @@ void OptParse::PrintUsage() {
 
     fprintf(
         stderr,
-        "%s [--session | --system | --internal | --config-file=FILE"
+        "%s [--session | --system | --internal | --config-file=FILE | --config"
 #if defined(QCC_OS_ANDROID) && defined(ROUTER_LIB)
         " | --config-service"
 #endif
@@ -298,6 +314,17 @@ void OptParse::PrintUsage() {
         "        Use the standard configuration for the system message bus.\n\n"
         "    --internal\n"
         "        Use a basic internally defined message bus for AllJoyn.\n\n"
+        "    --custom\n"
+        "        begin building your own custom configuration using\n"
+        "        --flag name value\n"
+        "        --limit name value\n"
+        "        --property name value\n"
+        "        --listen transport spec\n"
+        "        --listen transport DEL\n"
+        "        --clear\n"
+        "        --defaults\n"
+        "        --end\n\n"
+        "        as in \"--custom --listen tcp iface=*,port=9954 --end\n\n"
 #if defined(QCC_OS_ANDROID) && defined(ROUTER_LIB)
         "    --config-service\n"
         "        Use a configuration passed from the calling service.\n\n"
@@ -336,7 +363,6 @@ void OptParse::PrintUsage() {
         "");
 }
 
-
 OptParse::ParseResultCode OptParse::ParseResult()
 {
     ParseResultCode result = PR_OK;
@@ -354,28 +380,39 @@ OptParse::ParseResultCode OptParse::ParseResult()
             result = PR_EXIT_NO_ERROR;
             goto exit;
         } else if (arg.compare("--session") == 0) {
-            if (!configFile.empty() || internal) {
+            if (!configFile.empty() || internal || custom) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
             }
             configFile = "/etc/dbus-1/session.conf";
         } else if (arg.compare("--system") == 0) {
-            if (!configFile.empty() || internal) {
+            if (!configFile.empty() || internal || custom) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
             }
             configFile = "/etc/dbus-1/system.conf";
         } else if (arg.compare("--internal") == 0) {
-            if (!configFile.empty()) {
+            if (!configFile.empty() || custom) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
             }
             internal = true;
-        } else if (arg.compare("--config-file") == 0) {
+        } else if (arg.compare("--custom") == 0) {
             if (!configFile.empty() || internal) {
                 result = PR_OPTION_CONFLICT;
                 goto exit;
             }
+            i = configHelper.ParseArgs(i, argc, argv);
+            if (0 == strcmp("--end", argv[i])) {
+                ++i;
+                printf("%s\n", GetCustomConfigPrettyXml().c_str());
+            }
+            custom = true;
+        } else if (arg.compare("--config-file") == 0) {
+            if (!configFile.empty() || internal || custom) {
+                result = PR_OPTION_CONFLICT;
+                goto exit;
+            }
             ++i;
             if (i == argc) {
                 result = PR_MISSING_OPTION;
@@ -647,10 +684,13 @@ int daemon(OptParse& opts) {
 //
 #if defined(ROUTER_LIB)
 int DaemonMain(int argc, char** argv, char* serviceConfig)
+{
 #else
 int CDECL_CALL main(int argc, char** argv, char** env)
-#endif
 {
+    QCC_UNUSED(env);
+#endif
+
     if (AllJoynInit() != ER_OK) {
         return DAEMON_EXIT_STARTUP_ERROR;
     }
@@ -699,14 +739,18 @@ int CDECL_CALL main(int argc, char** argv, char** env)
         loggerSettings->SetFile(stderr);
     }
 
-    configStr = defaultConfig;
+    if (opts.GetCustomConfig()) {
+        configStr = opts.GetCustomConfigXml();
+    } else {
+        configStr = defaultConfig;
 #if defined(QCC_OS_ANDROID) && defined(ROUTER_LIB)
-    configStr.append(opts.GetServiceConfig() ? serviceConfig : internalConfig);
+        configStr.append(opts.GetServiceConfig() ? serviceConfig : internalConfig);
 #else
-    if (opts.GetInternalConfig()) {
-        configStr.append(internalConfig);
-    }
+        if (opts.GetInternalConfig()) {
+            configStr.append(internalConfig);
+        }
 #endif
+    }
 
     config = new ConfigDB(configStr, opts.GetConfigFile());
     if (!config->LoadConfig()) {
