@@ -64,8 +64,6 @@ static const char daemonConfig[] =
     "  <limit name=\"max_incomplete_connections\">16</limit>"
     "  <limit name=\"max_completed_connections\">32</limit>"
     "  <limit name=\"max_untrusted_clients\">16</limit>"
-    "  <flag name=\"restrict_untrusted_clients\">false</flag>"
-    "  <property name=\"ns_interfaces\">*</property>"
     "</busconfig>";
 
 /** Static top level message bus object */
