@@ -19,11 +19,6 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-/*
- * Compile this file only for Windows version 10 or greater.
- */
- #if (_WIN32_WINNT > 0x0603)
-
 #include <qcc/platform.h>
 #include <qcc/IPAddress.h>
 #include <qcc/Socket.h>
@@ -31,9 +26,9 @@
 #include <qcc/String.h>
 #include <qcc/StringUtil.h>
 #include <qcc/Windows/NamedPipeStream.h>
+#include <qcc/Windows/NamedPipeWrapper.h>
 #include <aclapi.h>
 #include <sddl.h>
-#include <MSAJTransport.h>
 
 #include <alljoyn/BusAttachment.h>
 #include <alljoyn/Session.h>
@@ -683,7 +678,7 @@ void NamedPipeDaemonTransport::ManageAuthenticatingEndpoints()
     list<NamedPipeDaemonEndpoint>::iterator authIterator = authenticatingEndpointList.begin();
 
     while (authIterator != authenticatingEndpointList.end()) {
-        if ((*authIterator)->AuthSucceeded()) {
+        if ((*authIterator)->IsValid() && (*authIterator)->AuthSucceeded()) {
             QCC_DbgHLPrintf(("ManageAuthenticatingEndpoints: moving endpoint to active list"));
             list<NamedPipeDaemonEndpoint>::iterator movedIterator = authIterator;
             authIterator++;
@@ -691,8 +686,8 @@ void NamedPipeDaemonTransport::ManageAuthenticatingEndpoints()
             NamedPipeDaemonEndpoint movedEndpoint = *movedIterator;
             authenticatingEndpointList.erase(movedIterator);
             endpointList.push_back(RemoteEndpoint::cast(movedEndpoint));
-        } else if ((*authIterator)->AuthFailed() || (*authIterator)->AuthStopped()) {
-            QCC_DbgHLPrintf(("ManageAuthenticatingEndpoints: removing failed or stopped endpoint"));
+        } else if (!(*authIterator)->IsValid() || (*authIterator)->AuthFailed() || (*authIterator)->AuthStopped()) {
+            QCC_DbgHLPrintf(("ManageAuthenticatingEndpoints: removing invalidated, failed or stopped endpoint"));
             list<NamedPipeDaemonEndpoint>::iterator erasedIterator = authIterator;
             authIterator++;
             (*erasedIterator)->Join();
@@ -718,7 +713,7 @@ void* NamedPipeAcceptThread::Run(_In_ void* arg)
          * Creating a new instance of the named pipe.
          */
         static const uint32_t BUFSIZE = 128 * 1024;
-        HANDLE pipeHandle = AllJoynCreateBus(BUFSIZE, BUFSIZE, nullptr);
+        HANDLE pipeHandle = qcc::NamedPipeWrapper::AllJoynCreateBus(BUFSIZE, BUFSIZE, nullptr);
 
         if (pipeHandle == INVALID_HANDLE_VALUE) {
             status = ER_OS_ERROR;
@@ -732,7 +727,7 @@ void* NamedPipeAcceptThread::Run(_In_ void* arg)
          * when this transport should stop its execution.
          */
         QCC_DbgHLPrintf(("NamedPipeAcceptThread: Waiting for connection on pipeHandle = 0x%p", pipeHandle));
-        DWORD acceptResult = AllJoynAcceptBusConnection(pipeHandle, stopEvent.GetHandle());
+        DWORD acceptResult = qcc::NamedPipeWrapper::AllJoynAcceptBusConnection(pipeHandle, stopEvent.GetHandle());
 
         if (acceptResult == ERROR_SUCCESS) {
             QCC_DbgHLPrintf(("NamedPipeAcceptThread: Accepted client connection on pipeHandle 0x%p", pipeHandle));
@@ -746,7 +741,7 @@ void* NamedPipeAcceptThread::Run(_In_ void* arg)
                 status = ER_OS_ERROR;
             }
 
-            BOOL success = AllJoynCloseBusHandle(pipeHandle);
+            BOOL success = qcc::NamedPipeWrapper::AllJoynCloseBusHandle(pipeHandle);
             assert(success);
         }
 
@@ -762,6 +757,27 @@ void* NamedPipeAcceptThread::Run(_In_ void* arg)
     return (void*) status;
 }
 
-} // namespace ajn
+void NamedPipeDaemonTransport::EndpointExit(RemoteEndpoint& ep)
+{
+    /*
+     * This is a callback driven from the remote endpoint thread exit function.
+     * NamedPipeDaemonEndpoint inherits from class RemoteEndpoint and so when
+     * either of the threads (transmit or receive) of one of our endpoints exits
+     * for some reason, we get called back here.
+     */
+    QCC_DbgTrace(("NamedPipeDaemonTransport::EndpointExit()"));
 
-#endif // #if (_WIN32_WINNT > 0x0603)
+    /* Remove the dead endpoint from the live endpoint list */
+    endpointListLock.Lock(MUTEX_CONTEXT);
+    for (list<NamedPipeDaemonEndpoint>::iterator i = authenticatingEndpointList.begin(); i != authenticatingEndpointList.end(); i++) {
+        if (ep.iden(*i)) {
+            authenticatingEndpointList.erase(i);
+            break;
+        }
+    }
+    endpointListLock.Unlock(MUTEX_CONTEXT);
+
+    DaemonTransport::EndpointExit(ep);
+}
+
+} // namespace ajn
