@@ -35,6 +35,7 @@
 
 #include <alljoyn/BusAttachment.h>
 #include <alljoyn/InterfaceDescription.h>
+#include <alljoyn/PermissionConfigurator.h>
 
 #include "AuthManager.h"
 #include "ObserverManager.h"
@@ -43,6 +44,7 @@
 #include "PeerState.h"
 #include "Transport.h"
 #include "TransportList.h"
+#include "PermissionManager.h"
 
 #include <alljoyn/Status.h>
 #include <set>
@@ -51,8 +53,11 @@ namespace ajn {
 
 class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncCB {
     friend class BusAttachment;
-  public:
+    friend class BusObject;
 
+    struct Session;
+
+  public:
     /**
      * Get a reference to the internal store object.
      *
@@ -201,6 +206,27 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
      */
     const qcc::String& GetListenAddresses() const { return listenAddresses; }
 
+    /**
+     * Inform PermissionConfigurationListener of incoming request to reset
+     * application state.
+     *
+     * @return  ER_OK if successful.
+     */
+    QStatus CallFactoryResetCallback();
+
+    /**
+     * Inform PermissionConfigurationListener of a security policy change.
+     */
+    void CallPolicyChangedCallback();
+
+    /**
+     * Set the PermissionConfigurationListener.
+     *
+     * @param listener   PermissionConfigurationListener to use.
+     * @return  ER_OK if successful.
+     */
+    QStatus SetPermissionConfigurationListener(PermissionConfigurationListener* listener);
+
     /**
      * Inform BusListeners of incoming JoinSession attempt.
      *
@@ -273,11 +299,35 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
      */
     void SetLinkTimeoutAsyncCB(Message& message, void* context);
 
+    /**
+     * AddMatchAsync method_reply handler
+     */
+    void AddMatchAsyncCB(Message& message, void* context);
+
+    /**
+     * RemoveMatchAsync method_reply handler
+     */
+    void RemoveMatchAsyncCB(Message& message, void* context);
+
     /**
      * PingAsync method_reply handler
      */
     void PingAsyncCB(Message& message, void* context);
 
+    /**
+     * Get a reference to the internal permission manager
+     *
+     * @return A reference to the bus's permmision manager
+     */
+    PermissionManager& GetPermissionManager() { return permissionManager; }
+
+    /**
+     * Get a reference to the permission configurator
+     *
+     * @return A reference to the bus's permmision configurator
+     */
+    PermissionConfigurator& GetPermissionConfigurator() { return permissionConfigurator; }
+
     /**
      * GetNameOwnerAsync method_reply handler
      */
@@ -308,13 +358,47 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
      *
      * @return set with all hosted session ids
      */
-    std::set<SessionId> GetHostedSessions() const {
-        sessionSetLock[SESSION_SIDE_HOST].Lock(MUTEX_CONTEXT);
-        std::set<SessionId> copy = sessionSet[SESSION_SIDE_HOST];
-        sessionSetLock[SESSION_SIDE_HOST].Unlock(MUTEX_CONTEXT);
+    std::vector<Session> GetHostedSessions() const {
+        sessionsLock[SESSION_SIDE_HOST].Lock(MUTEX_CONTEXT);
+        std::vector<Session> copy;
+        for (SessionMap::const_iterator it = sessions[SESSION_SIDE_HOST].begin(); it != sessions[SESSION_SIDE_HOST].end(); ++it) {
+            copy.push_back(it->second);
+        }
+        sessionsLock[SESSION_SIDE_HOST].Unlock(MUTEX_CONTEXT);
         return copy;
     }
 
+    /**
+     * Get a particular session description
+     *
+     * @param[in]  sessionId the session id
+     * @param[out] session_out   a reference to a Section struct to be filled in
+     *
+     * @return ER_OK if the session was found and the Session struct is filled in
+     *         ER_FAIL otherwise
+     */
+    QStatus GetSession(SessionId sessionId, Session& session_out) const {
+        bool found = false;
+        for (size_t side = 0; !found && side < ArraySize(sessions); ++side) {
+            sessionsLock[side].Lock(MUTEX_CONTEXT);
+            SessionMap::const_iterator it = sessions[side].find(sessionId);
+            if (it != sessions[side].end()) {
+                found = true;
+                session_out = it->second;
+            }
+            sessionsLock[side].Unlock(MUTEX_CONTEXT);
+        }
+        return found ? ER_OK : ER_FAIL;
+    }
+
+    /**
+     * Retrieve bus names for all peers that are in session (either hosted
+     * or joined) with this BusAttachment.
+     *
+     * @param[out] busNames a reference to a set<string> to be filled in
+     */
+    void GetConnectedPeers(std::set<std::string>& busNames);
+
     /**
      * Indicate whether bus is currently connected.
      *
@@ -369,7 +453,7 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
     /**
      * Disconnect a remote bus address connection.
      *
-     * @param[in] connectSpec The transport connection spec used to connect.
+     * @param[in] disconnectConnectSpec The transport connection spec used to connect.
      *
      * @return
      *     - #ER_OK if successful
@@ -377,7 +461,7 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
      *     - #ER_BUS_NOT_CONNECTED if the %BusAttachment is not connected to the bus
      *     - Other error status codes indicating a failure
      */
-    virtual QStatus TransportDisconnect(const char* connectSpec);
+    virtual QStatus TransportDisconnect(const char* disconnectConnectSpec);
 
     /** @copydoc _LocalEndpoint::RegisterSignalHandler() */
     virtual QStatus RegisterSignalHandler(MessageReceiver* receiver,
@@ -430,13 +514,13 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
      * @internal
      * Connect to an AllJoyn router at a specific connectSpec destination.
      *
-     * @param[in] connectSpec The transport connection spec to try.
+     * @param[in] requestedConnectSpec The transport connection spec to try.
      *
      * @return
      *     - #ER_OK if successful.
      *     - An error status otherwise
      */
-    QStatus TransportConnect(const char* connectSpec);
+    QStatus TransportConnect(const char* requestedConnectSpec);
 
     qcc::String application;              /* Name of the that owns the BusAttachment application */
     BusAttachment& bus;                   /* Reference back to the bus attachment that owns this state */
@@ -452,7 +536,7 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
     KeyStore keyStore;                    /* The key store for the bus attachment */
     AuthManager authManager;              /* The authentication manager for the bus attachment */
     qcc::GUID128 globalGuid;              /* Global GUID for this BusAttachment */
-    int32_t msgSerial;                    /* Serial number is updated for every message sent by this bus */
+    volatile int32_t msgSerial;           /* Serial number is updated for every message sent by this bus */
     Router* router;                       /* Message bus router */
     PeerStateTable peerStateTable;        /* Table that maintains state information about remote peers */
     LocalEndpoint localEndpoint;          /* The local endpoint */
@@ -460,21 +544,29 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
     bool allowRemoteMessages;             /* true iff endpoints of this attachment can receive messages from remote devices */
     qcc::String listenAddresses;          /* The set of bus addresses that this bus can listen on. (empty for clients) */
     qcc::Mutex stopLock;                  /* Protects BusAttachement::Stop from being reentered */
-    int32_t stopCount;                    /* Number of caller's blocked in BusAttachment::Stop() */
+    volatile int32_t stopCount;           /* Number of caller's blocked in BusAttachment::Stop() */
 
     typedef qcc::ManagedObj<SessionPortListener*> ProtectedSessionPortListener;
     typedef std::map<SessionPort, ProtectedSessionPortListener> SessionPortListenerMap;
     SessionPortListenerMap sessionPortListeners;  /* Lookup SessionPortListener by session port */
-    qcc::Mutex sessionPortListenersLock;       /* Lock protecting sessionPortListeners maps */
+    qcc::Mutex sessionPortListenersLock;       /* Lock protecting sessionPortListeners map */
 
     typedef qcc::ManagedObj<SessionListener*> ProtectedSessionListener;
-    typedef std::map<SessionId, ProtectedSessionListener> SessionListenerMap;
-    SessionListenerMap sessionListeners[SESSION_SIDE_NUM];   /* Lookup SessionListener by session id (index 0 for hoster, index 1 for joiner)*/
-    mutable qcc::Mutex sessionListenersLock[SESSION_SIDE_NUM];       /* Lock protecting sessionListeners maps */
+    struct Session {
+        bool host;
+        bool multipoint;
+        SessionId id;
+        ProtectedSessionListener listener;
+        std::set<std::string> otherParticipants;
+    };
+
+    typedef std::map<SessionId, Session> SessionMap;
+    SessionMap sessions[SESSION_SIDE_NUM]; /* look up session description by session id (index 0 for hosted sessions, index 1 for joined sessions) */
+    mutable qcc::Mutex sessionsLock[SESSION_SIDE_NUM];
 
     typedef qcc::ManagedObj<AboutListener*> ProtectedAboutListener;
     typedef std::set<ProtectedAboutListener> AboutListenerSet;
-    AboutListenerSet aboutListeners; /* About Signals are recieved out of Sessions so a set is all that is needed */
+    AboutListenerSet aboutListeners; /* About Signals are received out of Sessions so a set is all that is needed */
 
     qcc::Mutex aboutListenersLock;   /* Lock protecting the aboutListeners set */
 
@@ -484,18 +576,22 @@ class BusAttachment::Internal : public MessageReceiver, public JoinSessionAsyncC
         SessionOpts opts;
     };
 
-
-    std::set<SessionId> sessionSet[SESSION_SIDE_NUM];
-    mutable qcc::Mutex sessionSetLock[SESSION_SIDE_NUM];
-
     std::map<qcc::Thread*, JoinContext> joinThreads;  /* List of threads waiting to join */
     qcc::Mutex joinLock;                              /* Mutex that protects joinThreads */
     KeyStoreKeyEventListener ksKeyEventListener;
+    PermissionManager permissionManager;
+    PermissionConfigurator permissionConfigurator;
 
-    std::set<SessionId> hostedSessions;    /* session IDs for all sessions hosted by this bus attachment */
-    qcc::Mutex hostedSessionsLock;         /* Mutex that protects hostedSessions */
+    typedef qcc::ManagedObj<ApplicationStateListener*> ProtectedApplicationStateListener;
+    typedef std::set<ProtectedApplicationStateListener> ApplicationStateListenerSet;
+    ApplicationStateListenerSet applicationStateListeners; /* State Signals are received outside Sessions so a set container is all that is needed */
 
+    qcc::Mutex applicationStateListenersLock;   /* Lock protecting the applicationStateListeners set */
     ObserverManager* observerManager;      /* The observer manager for the bus attachment */
+
+    typedef qcc::ManagedObj<PermissionConfigurationListener*> ProtectedPermissionConfigurationListener;
+    ProtectedPermissionConfigurationListener* permissionConfigurationListener;
+    qcc::Mutex permissionConfigurationListenerLock;   /* Lock protecting permissionConfigurationListener */
 };
 
 }
