@@ -24,6 +24,7 @@
 
 #include <assert.h>
 
+#include <algorithm>
 #include <map>
 #include <vector>
 #include <set>
@@ -62,6 +63,13 @@ typedef struct {
 } MethodContext;
 #pragma pack(pop, MethodContext)
 
+static bool operator==(const MethodContext& a, const MethodContext& b)
+{
+    return (a.member == b.member) &&
+           (a.handler == b.handler) &&
+           (a.context == b.context);
+}
+
 struct BusObject::Components {
     /** The interfaces this object implements */
     vector<pair<const InterfaceDescription*, bool> > ifaces;
@@ -74,9 +82,58 @@ struct BusObject::Components {
     qcc::Mutex counterLock;
 
     /** counter to prevent this BusObject being deleted if it is being used by another thread. */
-    int32_t inUseCounter;
+    volatile int32_t inUseCounter;
 };
 
+/**
+ * Callback to allow the signal caller to authorize the signal.
+ */
+class SignalAuthorizationCallback {
+  public:
+    SignalAuthorizationCallback(BusAttachment& bus, qcc::String ifcName, vector<qcc::String> propNames) : bus(bus), ifcName(ifcName), propNames(propNames)
+    {
+    }
+
+    ~SignalAuthorizationCallback()
+    {
+        propNames.clear();
+    }
+
+    /**
+     * Authorize the signal
+     * @return ER_OK when signal is authorized; otherwise, a failure code
+     */
+    QStatus Authorize(Message& msg)
+    {
+        if (propNames.size() == 0) {
+            return ER_PERMISSION_DENIED;
+        }
+
+        PeerState peerState = bus.GetInternal().GetPeerStateTable()->GetPeerState(msg->GetDestination());
+        for (vector<qcc::String>::iterator it = propNames.begin(); it != propNames.end(); it++) {
+            QStatus status = bus.GetInternal().GetPermissionManager().AuthorizeGetProperty(msg->GetObjectPath(), ifcName.c_str(), (*it).c_str(), peerState);
+            if (ER_OK != status) {
+                return status;
+            }
+        }
+        return ER_OK;
+    }
+
+  private:
+    /**
+     * Assignment not allowed
+     */
+    SignalAuthorizationCallback& operator=(const SignalAuthorizationCallback& other);
+
+    /**
+     * Copy constructor not allowed
+     */
+    SignalAuthorizationCallback(const SignalAuthorizationCallback& other);
+
+    BusAttachment& bus;
+    qcc::String ifcName;
+    vector<qcc::String> propNames;
+};
 
 static inline bool SecurityApplies(const BusObject* obj, const InterfaceDescription* ifc)
 {
@@ -133,7 +190,7 @@ qcc::String BusObject::GenerateIntrospection(bool deep, size_t indent) const
     return BusObject::GenerateIntrospection(NULL, deep, indent);
 }
 
-qcc::String BusObject::GenerateIntrospection(const char* languageTag, bool deep, size_t indent) const
+qcc::String BusObject::GenerateIntrospection(const char* requestedLanguageTag, bool deep, size_t indent) const
 {
     qcc::String in(indent, ' ');
     qcc::String xml;
@@ -146,8 +203,8 @@ qcc::String BusObject::GenerateIntrospection(const char* languageTag, bool deep,
         xml += in + "<node name=\"" + child->GetName() + "\"";
 
         const char* nodeDesc = NULL;
-        if (languageTag) {
-            nodeDesc = child->GetDescription(languageTag, buffer);
+        if (requestedLanguageTag) {
+            nodeDesc = child->GetDescription(requestedLanguageTag, buffer);
         }
 
         if (deep || nodeDesc) {
@@ -156,7 +213,7 @@ qcc::String BusObject::GenerateIntrospection(const char* languageTag, bool deep,
                 xml += in + "  <description>" + XmlElement::EscapeXml(nodeDesc) + "</description>";
             }
             if (deep) {
-                xml += child->GenerateIntrospection(languageTag, deep, indent + 2);
+                xml += child->GenerateIntrospection(requestedLanguageTag, deep, indent + 2);
             }
 
             xml += "\n" + in + "</node>\n";
@@ -179,7 +236,7 @@ qcc::String BusObject::GenerateIntrospection(const char* languageTag, bool deep,
                 (strcmp((itIf->first)->GetName(), org::freedesktop::DBus::Properties::InterfaceName) == 0)) {
                 ++itIf;
             } else {
-                xml += (itIf->first)->Introspect(indent, languageTag, bus ? bus->GetDescriptionTranslator() : NULL);
+                xml += (itIf->first)->Introspect(indent, requestedLanguageTag, bus ? bus->GetDescriptionTranslator() : NULL);
                 ++itIf;
             }
         }
@@ -244,6 +301,9 @@ void BusObject::EmitPropChanged(const char* ifcName, const char* propName, MsgAr
     qcc::String emitsChanged;
     if (ifc && ifc->GetPropertyAnnotation(propName, org::freedesktop::DBus::AnnotateEmitsChanged, emitsChanged)) {
         QCC_DbgPrintf(("emitsChanged = %s", emitsChanged.c_str()));
+        if (SecurityApplies(this, ifc)) {
+            flags |= ALLJOYN_FLAG_ENCRYPTED;
+        }
         if (emitsChanged == "true") {
             const InterfaceDescription* bus_ifc = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
             const InterfaceDescription::Member* propChanged = (bus_ifc ? bus_ifc->GetMember("PropertiesChanged") : NULL);
@@ -255,7 +315,11 @@ void BusObject::EmitPropChanged(const char* ifcName, const char* propName, MsgAr
                 MsgArg str("{sv}", propName, &val);
                 args[1].Set("a{sv}", 1, &str);
                 args[2].Set("as", 0, NULL);
-                Signal(NULL, id, *propChanged, args, ArraySize(args), 0, flags);
+                vector<qcc::String> vNames;
+                vNames.push_back(propName);
+                SignalAuthorizationCallback signalAuth(*bus, ifcName, vNames);
+                vNames.clear();
+                SignalInternal(NULL, id, *propChanged, args, ArraySize(args), 0, flags, NULL, &signalAuth);
             }
         } else if (emitsChanged == "invalidates") {
             const InterfaceDescription* bus_ifc = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
@@ -266,7 +330,11 @@ void BusObject::EmitPropChanged(const char* ifcName, const char* propName, MsgAr
                 args[0].Set("s", ifcName);
                 args[1].Set("a{sv}", 0, NULL);
                 args[2].Set("as", 1, &propName);
-                Signal(NULL, id, *propChanged, args, ArraySize(args), 0, flags);
+                vector<qcc::String> vNames;
+                vNames.push_back(propName);
+                SignalAuthorizationCallback signalAuth(*bus, ifcName, vNames);
+                vNames.clear();
+                SignalInternal(NULL, id, *propChanged, args, ArraySize(args), 0, flags, NULL, &signalAuth);
             }
         }
     }
@@ -287,6 +355,10 @@ QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames,
         size_t updatedPropNum = 0;
         size_t invalidatedPropNum = 0;
 
+        if (SecurityApplies(this, ifc)) {
+            flags |= ALLJOYN_FLAG_ENCRYPTED;
+        }
+        vector<qcc::String> vNames;
         for (size_t i = 0; i < numProps; ++i) {
             const char* propName = propNames[i];
             const InterfaceDescription::Property* prop = ifc->GetProperty(propName);
@@ -310,10 +382,12 @@ QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames,
                     updatedProp[updatedPropNum].Set("{sv}", propName, val);
                     updatedProp[updatedPropNum].SetOwnershipFlags(MsgArg::OwnsArgs, true /*deep*/);
                     updatedPropNum++;
+                    vNames.push_back(propName);
                 } else if (emitsChanged == "invalidates") {
                     /* only emit that it's invalidated */
                     invalidatedProp[invalidatedPropNum] = propName;
                     invalidatedPropNum++;
+                    vNames.push_back(propName);
                 }
             }
         }
@@ -327,9 +401,11 @@ QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames,
             args[0].Set("s", ifcName);
             args[1].Set("a{sv}", updatedPropNum, updatedProp);
             args[2].Set("as", invalidatedPropNum, invalidatedProp);
+            SignalAuthorizationCallback signalAuth(*bus, ifcName, vNames);
             /* send the signal */
-            status = Signal(NULL, id, *propChanged, args, ArraySize(args), 0, flags);
+            status = SignalInternal(NULL, id, *propChanged, args, ArraySize(args), 0, flags, NULL, &signalAuth);
         }
+        vNames.clear();
 
         delete[] updatedProp;
         delete[] invalidatedProp;
@@ -337,7 +413,6 @@ QStatus BusObject::EmitPropChanged(const char* ifcName, const char** propNames,
     return status;
 }
 
-
 void BusObject::SetProp(const InterfaceDescription::Member* member, Message& msg)
 {
     QCC_UNUSED(member);
@@ -401,18 +476,36 @@ void BusObject::GetAllProps(const InterfaceDescription::Member* member, Message&
             size_t numProps = ifc->GetProperties();
             props = new const InterfaceDescription::Property*[numProps];
             ifc->GetProperties(props, numProps);
+            bool* allowed = new bool[numProps];
+            for (size_t i = 0; i < numProps; i++) {
+                allowed[i] = true;
+            }
             size_t readable = 0;
             /* Count readable properties */
-            for (size_t i = 0; i < numProps; i++) {
-                if (props[i]->access & PROP_ACCESS_READ) {
-                    readable++;
+            if (msg->IsEncrypted()) {
+                PeerState peerState = bus->GetInternal().GetPeerStateTable()->GetPeerState(msg->GetSender());
+                for (size_t i = 0; i < numProps; i++) {
+                    if (props[i]->access & PROP_ACCESS_READ) {
+                        if (ER_OK == bus->GetInternal().GetPermissionManager().AuthorizeGetProperty(msg->GetObjectPath(), ifc->GetName(), props[i]->name.c_str(), peerState)) {
+                            readable++;
+                        } else {
+                            /* mark the property as not allowed because of permission denied */
+                            allowed[i] = false;
+                        }
+                    }
+                }
+            } else {
+                for (size_t i = 0; i < numProps; i++) {
+                    if (props[i]->access & PROP_ACCESS_READ) {
+                        readable++;
+                    }
                 }
             }
             MsgArg* dict = new MsgArg[readable];
             MsgArg* entry = dict;
             /* Get readable properties */
             for (size_t i = 0; i < numProps; i++) {
-                if (props[i]->access & PROP_ACCESS_READ) {
+                if ((props[i]->access & PROP_ACCESS_READ) && allowed[i]) {
                     MsgArg* val = new MsgArg();
                     status = Get(iface->v_string.str, props[i]->name.c_str(), *val);
                     if (status != ER_OK) {
@@ -426,6 +519,7 @@ void BusObject::GetAllProps(const InterfaceDescription::Member* member, Message&
             }
             vals.Set("a{sv}", readable, dict);
             vals.SetOwnershipFlags(MsgArg::OwnsArgs, false);
+            delete [] allowed;
         }
     } else {
         status = ER_BUS_UNKNOWN_INTERFACE;
@@ -471,7 +565,9 @@ QStatus BusObject::AddMethodHandler(const InterfaceDescription::Member* member,
         QCC_LogError(status, ("Cannot add method handler to an object that is already registered"));
     } else if (ImplementsInterface(member->iface->GetName())) {
         MethodContext ctx = { member, handler, handlerContext };
-        components->methodContexts.push_back(ctx);
+        if (find(components->methodContexts.begin(), components->methodContexts.end(), ctx) == components->methodContexts.end()) {
+            components->methodContexts.push_back(ctx);
+        }
     } else {
         status = ER_BUS_NO_SUCH_INTERFACE;
         QCC_LogError(status, ("Cannot add method handler for unknown interface"));
@@ -562,11 +658,17 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
     /* Add the standard DBus interfaces */
     const InterfaceDescription* introspectable = bus->GetInterface(org::freedesktop::DBus::Introspectable::InterfaceName);
     assert(introspectable);
-    components->ifaces.push_back(make_pair(introspectable, false));
+    pair<const InterfaceDescription*, bool> comp = make_pair(introspectable, false);
+    if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
+        components->ifaces.push_back(comp);
+    }
 
     const InterfaceDescription* allseenIntrospectable = bus->GetInterface(org::allseen::Introspectable::InterfaceName);
     assert(allseenIntrospectable);
-    components->ifaces.push_back(make_pair(allseenIntrospectable, false));
+    comp = make_pair(allseenIntrospectable, false);
+    if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
+        components->ifaces.push_back(comp);
+    }
 
     /* Add the standard method handlers */
     const MethodEntry methodEntries[] = {
@@ -585,7 +687,10 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
             /* Add the org::freedesktop::DBus::Properties interface to this list of interfaces implemented by this obj */
             const InterfaceDescription* propIntf = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
             assert(propIntf);
-            components->ifaces.push_back(make_pair(propIntf, false));
+            comp = make_pair(propIntf, false);
+            if (find(components->ifaces.begin(), components->ifaces.end(), comp) == components->ifaces.end()) {
+                components->ifaces.push_back(comp);
+            }
 
             /* Attach the handlers */
             const MethodEntry propHandlerList[] = {
@@ -605,14 +710,15 @@ QStatus BusObject::DoRegistration(BusAttachment& busAttachment)
     return status;
 }
 
-QStatus BusObject::Signal(const char* destination,
-                          SessionId sessionId,
-                          const InterfaceDescription::Member& signalMember,
-                          const MsgArg* args,
-                          size_t numArgs,
-                          uint16_t timeToLive,
-                          uint8_t flags,
-                          Message* outMsg)
+QStatus BusObject::SignalInternal(const char* destination,
+                                  SessionId sessionId,
+                                  const InterfaceDescription::Member& signalMember,
+                                  const MsgArg* args,
+                                  size_t numArgs,
+                                  uint16_t timeToLive,
+                                  uint8_t flags,
+                                  Message* outMsg,
+                                  SignalAuthorizationCallback* authorizationCallback)
 {
     /* Protect against calling Signal before object is registered */
     if (!bus) {
@@ -629,38 +735,134 @@ QStatus BusObject::Signal(const char* destination,
         return ER_BUS_SECURITY_NOT_ENABLED;
     }
 
-    std::set<SessionId> ids;
-    if (sessionId != SESSION_ID_ALL_HOSTED) {
-        ids.insert(sessionId);
-    } else {
-        ids = bus->GetInternal().GetHostedSessions();
+    if (signalMember.isSessioncastSignal ||
+        signalMember.isSessionlessSignal ||
+        signalMember.isUnicastSignal ||
+        signalMember.isGlobalBroadcastSignal) {
+        // Enforce signal type, since signal type was explicitly set.
+        if ((destination == NULL) && (sessionId != 0) && !signalMember.isSessioncastSignal) {
+            QCC_LogError(ER_INVALID_SIGNAL_EMISSION_TYPE, ("Attempt to send a sessioncast signal when %s is not sessioncast", signalMember.name.c_str()));
+            return ER_INVALID_SIGNAL_EMISSION_TYPE;
+        }
+        if ((flags & ALLJOYN_FLAG_SESSIONLESS) && !signalMember.isSessionlessSignal) {
+            QCC_LogError(ER_INVALID_SIGNAL_EMISSION_TYPE, ("Attempt to send a sessionless signal when %s is not sessionless", signalMember.name.c_str()));
+            return ER_INVALID_SIGNAL_EMISSION_TYPE;
+        }
+        if ((destination != NULL) && !signalMember.isUnicastSignal) {
+            QCC_LogError(ER_INVALID_SIGNAL_EMISSION_TYPE, ("Attempt to send a unicast signal when %s is not unicast", signalMember.name.c_str()));
+            return ER_INVALID_SIGNAL_EMISSION_TYPE;
+        }
+        if ((flags & ALLJOYN_FLAG_GLOBAL_BROADCAST) && !signalMember.isGlobalBroadcastSignal) {
+            QCC_LogError(ER_INVALID_SIGNAL_EMISSION_TYPE, ("Attempt to send a global broadcast signal when %s is not global broadcast", signalMember.name.c_str()));
+            return ER_INVALID_SIGNAL_EMISSION_TYPE;
+        }
     }
 
-    QStatus status = ER_OK;
+    /* For sessioncast signals (sessionId != 0 && destination == NULL), we need
+     * to perform some tricks here:
+     * - if sessionId == SESSION_ID_ALL_HOSTED, we want to emit sessioncast
+     *   signals for each hosted session
+     * - for individual sessioncast signals in point-to-point sessions, we fill
+     *   in the destination (i.e. the other end of the session) explicitly so
+     *   that Security 2.0 can perform better permission checking. This
+     *   preserves the non-Sec2.0 behavior that a sessioncast signal in a
+     *   point-to-point session is functionally equivalent to a unicast signal
+     *   over the same session.
+     */
 
-    for (std::set<SessionId>::iterator it = ids.begin(); it != ids.end(); ++it) {
+    /* declare these in the method scope to keep the strings inside alive until the end of the method */
+    std::vector<BusAttachment::Internal::Session> sessions;
+    BusAttachment::Internal::Session session;
+
+    std::vector<std::pair<SessionId, const char*> > emissionParams;
+    if (sessionId == SESSION_ID_ALL_HOSTED) {
+        sessions = bus->GetInternal().GetHostedSessions();
+        std::vector<BusAttachment::Internal::Session>::iterator it;
+        for (it = sessions.begin(); it != sessions.end(); ++it) {
+            if (it->multipoint) {
+                emissionParams.push_back(std::make_pair(it->id, (const char*) NULL));
+            } else {
+                emissionParams.push_back(std::make_pair(it->id, it->otherParticipants.begin()->c_str()));
+            }
+        }
+    } else if (sessionId != 0 && destination == NULL) {
+        QStatus status = bus->GetInternal().GetSession(sessionId, session);
+        if (status == ER_OK && !session.multipoint) {
+            emissionParams.push_back(std::make_pair(sessionId, session.otherParticipants.begin()->c_str()));
+        } else {
+            emissionParams.push_back(std::make_pair(sessionId, (const char*) NULL));
+        }
+    } else {
+        emissionParams.push_back(std::make_pair(sessionId, destination));
+    }
+
+    if (emissionParams.empty()) {
+        return ER_OK;
+    }
+
+    QStatus status = ER_FAIL;
+
+    std::vector<std::pair<SessionId, const char*> >::iterator it;
+    for (it = emissionParams.begin(); it != emissionParams.end(); ++it) {
         Message msg(*bus);
-        status = msg->SignalMsg(signalMember.signature,
-                                destination,
-                                *it,
-                                path,
-                                signalMember.iface->GetName(),
-                                signalMember.name,
-                                args,
-                                numArgs,
-                                flags,
-                                timeToLive);
-        if (status == ER_OK) {
-            BusEndpoint bep = BusEndpoint::cast(bus->GetInternal().GetLocalEndpoint());
-            QStatus status = bus->GetInternal().GetRouter().PushMessage(msg, bep);
-            if ((status == ER_OK) && outMsg) {
+        QStatus aStatus = msg->SignalMsg(signalMember.signature,
+                                         it->second,
+                                         it->first,
+                                         path,
+                                         signalMember.iface->GetName(),
+                                         signalMember.name,
+                                         args,
+                                         numArgs,
+                                         flags,
+                                         timeToLive);
+        if (aStatus == ER_OK) {
+            if (msg->IsEncrypted()) {
+                if (authorizationCallback != NULL) {
+                    aStatus = authorizationCallback->Authorize(msg);
+                    /* mark the message so the Message::EncryptMessage method does
+                     * not need to authorize the message again.
+                     */
+                    msg->authorizationChecked = true;
+                } else if (!bus->GetInternal().GetRouter().IsDaemon()) {
+                    /* do an earlier permission authorization to make sure this
+                     * signal is allowed to send to the router for delivery.
+                     */
+                    PeerState peerState = bus->GetInternal().GetPeerStateTable()->GetPeerState(msg->GetDestination());
+                    aStatus = bus->GetInternal().GetPermissionManager().AuthorizeMessage(true, msg, peerState);
+                    /* mark the message so the Message::EncryptMessage method does
+                     * not need to authorize the message again.
+                     */
+                    msg->authorizationChecked = true;
+                }
+            }
+            if (aStatus == ER_OK) {
+                BusEndpoint bep = BusEndpoint::cast(bus->GetInternal().GetLocalEndpoint());
+                aStatus = bus->GetInternal().GetRouter().PushMessage(msg, bep);
+            }
+            if ((aStatus == ER_OK) && outMsg) {
                 *outMsg = msg;
             }
         }
+        if (status != ER_OK) {
+            /* Once status is ER_OK, it will return ER_OK */
+            status = aStatus;
+        }
     }
     return status;
 }
 
+QStatus BusObject::Signal(const char* destination,
+                          SessionId sessionId,
+                          const InterfaceDescription::Member& signalMember,
+                          const MsgArg* args,
+                          size_t numArgs,
+                          uint16_t timeToLive,
+                          uint8_t flags,
+                          Message* outMsg)
+{
+    return SignalInternal(destination, sessionId, signalMember, args, numArgs, timeToLive, flags, outMsg, NULL);
+}
+
 QStatus BusObject::CancelSessionlessMessage(uint32_t serialNum)
 {
     if (!bus) {
@@ -700,7 +902,7 @@ QStatus BusObject::CancelSessionlessMessage(uint32_t serialNum)
     return status;
 }
 
-QStatus BusObject::MethodReply(const Message& msg, const MsgArg* args, size_t numArgs)
+QStatus BusObject::MethodReply(const Message& msg, const MsgArg* args, size_t numArgs, Message* replyMsg)
 {
     QStatus status;
 
@@ -723,11 +925,14 @@ QStatus BusObject::MethodReply(const Message& msg, const MsgArg* args, size_t nu
             BusEndpoint bep = BusEndpoint::cast(bus->GetInternal().GetLocalEndpoint());
             status = bus->GetInternal().GetRouter().PushMessage(reply, bep);
         }
+        if (NULL != replyMsg) {
+            *replyMsg = reply;
+        }
     }
     return status;
 }
 
-QStatus BusObject::MethodReply(const Message& msg, const char* errorName, const char* errorMessage)
+QStatus BusObject::MethodReply(const Message& msg, const char* errorName, const char* errorMessage, Message* replyMsg)
 {
     QStatus status;
 
@@ -751,11 +956,14 @@ QStatus BusObject::MethodReply(const Message& msg, const char* errorName, const
             BusEndpoint bep = BusEndpoint::cast(bus->GetInternal().GetLocalEndpoint());
             status = bus->GetInternal().GetRouter().PushMessage(error, bep);
         }
+        if (NULL != replyMsg) {
+            *replyMsg = error;
+        }
     }
     return status;
 }
 
-QStatus BusObject::MethodReply(const Message& msg, QStatus status)
+QStatus BusObject::MethodReply(const Message& msg, QStatus status, Message* replyMsg)
 {
     /* Protect against calling before object is registered */
     if (!bus) {
@@ -768,7 +976,8 @@ QStatus BusObject::MethodReply(const Message& msg, QStatus status)
     }
 
     if (status == ER_OK) {
-        return MethodReply(msg);
+        /* Casts needed to unambiguously call the correct overload of MethodReply. */
+        return MethodReply(msg, (const MsgArg*)NULL, (size_t)0, replyMsg);
     } else {
         if (msg->GetType() != MESSAGE_METHOD_CALL) {
             return ER_BUS_NO_CALL_FOR_REPLY;
@@ -777,6 +986,9 @@ QStatus BusObject::MethodReply(const Message& msg, QStatus status)
             QStatus result = error->ErrorMsg(msg, status);
             assert(ER_OK == result);
             QCC_UNUSED(result);
+            if (NULL != replyMsg) {
+                *replyMsg = error;
+            }
             BusEndpoint bep = BusEndpoint::cast(bus->GetInternal().GetLocalEndpoint());
             return bus->GetInternal().GetRouter().PushMessage(error, bep);
         }
@@ -916,7 +1128,9 @@ const char* BusObject::GetDescription(const char* toLanguage, qcc::String& buffe
     }
 
     if (myTranslator) {
-        const char* ret = myTranslator->Translate(languageTag.c_str(), toLanguage, description.c_str(), buffer);
+        qcc::String bestLanguage;
+        myTranslator->GetBestLanguage(toLanguage, languageTag, bestLanguage);
+        const char* ret = myTranslator->Translate(languageTag.c_str(), bestLanguage.c_str(), description.c_str(), buffer);
         if (ret) {
             return ret;
         }
@@ -975,7 +1189,7 @@ void BusObject::GetDescriptionLanguages(const InterfaceDescription::Member* memb
     bool hasDescription = false;
     bool someoneHasNoTranslator = false;
 
-    //First merge this objects languages...
+    // First merge this object's languages...
     if (!languageTag.empty()) {
         langs.insert(languageTag);
         hasDescription = true;
@@ -1039,9 +1253,9 @@ void BusObject::GetDescriptionLanguages(const InterfaceDescription::Member* memb
     }
 }
 
-void BusObject::SetDescriptionTranslator(Translator* translator)
+void BusObject::SetDescriptionTranslator(Translator* newTranslator)
 {
-    this->translator = translator;
+    this->translator = newTranslator;
 }
 
 size_t BusObject::GetAnnouncedInterfaceNames(const char** interfaces, size_t numInterfaces)
