@@ -38,6 +38,7 @@
 
 #include "PeerState.h"
 #include "KeyStore.h"
+#include "BusUtil.h"
 
 #include <alljoyn/Status.h>
 
@@ -54,11 +55,24 @@ namespace ajn {
  */
 static const uint16_t LowStoreVersion = 0x0102;
 
+/*
+ * key store version (or any other older) that did not account for guid in secret
+ * In the key store version x103 or order, there are some occurrences where the
+ * GUID string used in the encryption key for the keystore can be empty
+ * because the use of the GetGuid().
+ */
+static const uint16_t MixupGuidKeyStoreVersion = 0x0103;
+
+/**
+ * the key store version where the key is a composite key instead of just
+ * the GUID.
+ */
+static const uint16_t CompositeKeyKeyStoreVersion = 0x0104;
+
 /*
  * Current key store version we will write
  */
-static const uint16_t KeyStoreVersion = 0x0103;
-
+static const uint16_t KeyStoreVersion = 0x0104;
 
 QStatus KeyStoreListener::PutKeys(KeyStore& keyStore, const qcc::String& source, const qcc::String& password)
 {
@@ -86,8 +100,14 @@ KeyStore::KeyStore(const qcc::String& application) :
     keyStoreKey(NULL),
     shared(false),
     stored(NULL),
+    storedRefCount(0),
     loaded(NULL),
-    keyEventListener(NULL)
+    loadedRefCount(0),
+    keyEventListener(NULL),
+    useDefaultListener(false),
+    guidSetEvent(NULL),
+    guidSet(false),
+    guidSetRefCount(0)
 {
 }
 
@@ -113,20 +133,45 @@ KeyStore::~KeyStore()
         lock.Lock(MUTEX_CONTEXT);
     }
     lock.Unlock(MUTEX_CONTEXT);
+    /* Unblock thread that might be waiting for a guid set to complete */
+    if (guidSetEvent) {
+        guidSetEvent->SetEvent();
+        while (guidSetEvent) {
+            qcc::Sleep(1);
+        }
+    }
     delete defaultListener;
     delete listener;
     delete keyStoreKey;
     delete keys;
 }
 
-QStatus KeyStore::SetListener(KeyStoreListener& listener)
+QStatus KeyStore::SetListener(KeyStoreListener& keyStoreListener)
 {
+    /**
+     * only allow to set new listener when
+     *    listener was not previously set
+     *    listener was set with the default listener
+     */
+
+    lock.Lock(MUTEX_CONTEXT);
+    bool setIt = false;
     if (this->listener != NULL) {
-        return ER_BUS_LISTENER_ALREADY_SET;
+        if (useDefaultListener) {
+            setIt = true;
+        }
     } else {
-        this->listener = new ProtectedKeyStoreListener(&listener);
+        setIt = true;
+    }
+    if (setIt) {
+        delete this->listener;
+        this->listener = new ProtectedKeyStoreListener(&keyStoreListener);
+        useDefaultListener = false;
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_OK;
     }
+    lock.Unlock(MUTEX_CONTEXT);
+    return ER_BUS_LISTENER_ALREADY_SET;
 }
 
 /**
@@ -134,45 +179,61 @@ QStatus KeyStore::SetListener(KeyStoreListener& listener)
  *
  * @param listener  The listener that will listen to key event.
  */
-QStatus KeyStore::SetKeyEventListener(KeyStoreKeyEventListener* listener) {
-    keyEventListener = listener;
+QStatus KeyStore::SetKeyEventListener(KeyStoreKeyEventListener* eventListener) {
+    keyEventListener = eventListener;
     return ER_OK;
 }
 
 QStatus KeyStore::SetDefaultListener()
 {
-    delete listener;
+    lock.Lock(MUTEX_CONTEXT);
+    delete this->listener;
     this->listener = new ProtectedKeyStoreListener(defaultListener);
+    useDefaultListener = true;
+    lock.Unlock(MUTEX_CONTEXT);
     return ER_OK;
 }
 
 QStatus KeyStore::Reset()
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState != UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
+        /* clear the keys first */
         QStatus status = Clear();
+        if (ER_OK != status) {
+            return status;
+        }
+        lock.Lock(MUTEX_CONTEXT);
         storeState = UNAVAILABLE;
         delete listener;
         listener = NULL;
         delete defaultListener;
         defaultListener = NULL;
         shared = false;
+        useDefaultListener = false;
+        lock.Unlock(MUTEX_CONTEXT);
         return status;
     } else {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_FAIL;
     }
 }
 
 QStatus KeyStore::Init(const char* fileName, bool isShared)
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
         if (listener == NULL) {
             defaultListener = KeyStoreListenerFactory::CreateInstance(application, fileName);
             listener = new ProtectedKeyStoreListener(defaultListener);
         }
         shared = isShared;
+        lock.Unlock(MUTEX_CONTEXT);
         return Load();
     } else {
-        return ER_FAIL;
+        lock.Unlock(MUTEX_CONTEXT);
+        return ER_KEY_STORE_ALREADY_INITIALIZED;
     }
 }
 
@@ -180,37 +241,45 @@ QStatus KeyStore::Store()
 {
     QStatus status = ER_OK;
 
+    lock.Lock(MUTEX_CONTEXT);
     /* Cannot store if never loaded */
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
     /* Don't store if not modified */
-    if (storeState == MODIFIED) {
+    if (storeState != MODIFIED) {
+        lock.Unlock(MUTEX_CONTEXT);
+        return ER_OK;
+    }
+    EraseExpiredKeys();
 
+    /* Reload to merge keystore changes before storing */
+    if (revision > 0) {
+        lock.Unlock(MUTEX_CONTEXT);
+        status = Reload();
         lock.Lock(MUTEX_CONTEXT);
-        EraseExpiredKeys();
-
-        /* Reload to merge keystore changes before storing */
-        if (revision > 0) {
-            lock.Unlock(MUTEX_CONTEXT);
-            status = Reload();
-            lock.Lock(MUTEX_CONTEXT);
-        }
-        if (status == ER_OK) {
+    }
+    if (status == ER_OK) {
+        if (storedRefCount == 0) {
             stored = new Event();
-            lock.Unlock(MUTEX_CONTEXT);
-            status = listener->StoreRequest(*this);
-            if (status == ER_OK) {
-                status = Event::Wait(*stored);
-            }
-            lock.Lock(MUTEX_CONTEXT);
+        }
+        storedRefCount++;
+        lock.Unlock(MUTEX_CONTEXT);
+        status = listener->StoreRequest(*this);
+        if (status == ER_OK) {
+            status = Event::Wait(*stored);
+        }
+        lock.Lock(MUTEX_CONTEXT);
+        storedRefCount--;
+        if (storedRefCount == 0) {
             delete stored;
             stored = NULL;
-            /* Done tracking deletions */
-            deletions.clear();
         }
-        lock.Unlock(MUTEX_CONTEXT);
+        /* Done tracking deletions */
+        deletions.clear();
     }
+    lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
@@ -220,19 +289,26 @@ QStatus KeyStore::Load()
     lock.Lock(MUTEX_CONTEXT);
     keys->clear();
     storeState = UNAVAILABLE;
-    loaded = new Event();
+    if (loadedRefCount == 0) {
+        loaded = new Event();
+    }
+    loadedRefCount++;
     lock.Unlock(MUTEX_CONTEXT);
     status = listener->LoadRequest(*this);
     if (status == ER_OK) {
         status = Event::Wait(*loaded);
     }
     lock.Lock(MUTEX_CONTEXT);
-    delete loaded;
-    loaded = NULL;
+    loadedRefCount--;
+    if (loadedRefCount == 0) {
+        delete loaded;
+        loaded = NULL;
+    }
     lock.Unlock(MUTEX_CONTEXT);
     return status;
 }
 
+/* private method assumes lock is already acquired by the caller. */
 size_t KeyStore::EraseExpiredKeys()
 {
     size_t count = 0;
@@ -242,7 +318,7 @@ size_t KeyStore::EraseExpiredKeys()
         KeyMap::iterator it = keys->begin();
         while (it != keys->end()) {
             KeyMap::iterator current = it++;
-            if (current->second.key.HasExpired()) {
+            if (current->second.keyBlob.HasExpired()) {
                 QCC_DbgPrintf(("Deleting expired key for GUID %s", current->first.ToString().c_str()));
                 bool affected = false;
                 if (keyEventListener) {
@@ -267,12 +343,12 @@ QStatus KeyStore::Pull(Source& source, const qcc::String& password)
 
 
     /* Don't load if already loaded */
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState != UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_OK;
     }
 
-    lock.Lock(MUTEX_CONTEXT);
-
     uint8_t guidBuf[qcc::GUID128::SIZE];
     size_t pulled;
     size_t len = 0;
@@ -292,19 +368,22 @@ QStatus KeyStore::Pull(Source& source, const qcc::String& password)
     if (status == ER_OK) {
         status = source.PullBytes(guidBuf, qcc::GUID128::SIZE, pulled);
         thisGuid.SetBytes(guidBuf);
+        MarkGuidSet();
     }
 
     /* This is the only chance to generate the key store key */
     if (!keyStoreKey) {
-        keyStoreKey = new KeyBlob;
+        keyStoreKey = new KeyStoreEncryptionKey();
     }
-    keyStoreKey->Derive(password + GetGuid(), Crypto_AES::AES128_SIZE, KeyBlob::AES);
+    keyStoreKey->Build(password, thisGuid.ToString());
+
 
     /* Allow for an uninitialized (empty) key store */
     if (status == ER_EOF) {
         keys->clear();
         storeState = MODIFIED;
         revision = 0;
+        MarkGuidSet();  /* make thisGuid the keystore guid */
         status = ER_OK;
         goto ExitPull;
     }
@@ -336,9 +415,33 @@ QStatus KeyStore::Pull(Source& source, const qcc::String& password)
             /*
              * Decrypt the key store.
              */
+            /**
+             * In the key store version 0x103 or older, the encryption secret was
+             * supposed to be the password + the key store guid.  However, the call
+             * the retrieve the guid may return empty (when the keystore is in
+             * loading mode) so it used just the password.
+             * However, the non-empty guid string is actually written to the listener
+             * so the decryption will fail because of key mismatch.
+             */
+            bool mayUseAlternateKey = (version <= MixupGuidKeyStoreVersion);
+            uint8_t* altData = NULL;
+            size_t altLen = len;
+            if (mayUseAlternateKey) {
+                altData = new uint8_t[altLen];
+                /* copy the data in case to decrypt with an alternate key.  The data needs to be copied since the call aes.Decrypt_CCM below overwrites the data buffer */
+                memcpy(altData, data, altLen);
+            }
             KeyBlob nonce((uint8_t*)&revision, sizeof(revision), KeyBlob::GENERIC);
             Crypto_AES aes(*keyStoreKey, Crypto_AES::CCM);
             status = aes.Decrypt_CCM(data, data, len, nonce, NULL, 0, 16);
+            if ((status != ER_OK) && mayUseAlternateKey) {
+                KeyBlob altKey;
+                altKey.Derive(password, Crypto_AES::AES128_SIZE, KeyBlob::AES);
+                Crypto_AES altAES(altKey, Crypto_AES::CCM);
+                status = altAES.Decrypt_CCM(altData, data, altLen, nonce, NULL, 0, 16);
+            }
+            delete [] altData;
+
             /*
              * Unpack the guid/key pairs from an intermediate string source.
              */
@@ -346,15 +449,20 @@ QStatus KeyStore::Pull(Source& source, const qcc::String& password)
             while (status == ER_OK) {
                 uint32_t rev;
                 status = strSource.PullBytes(&rev, sizeof(rev), pulled);
+                Key::KeyType keyType = Key::REMOTE;
+                if ((status == ER_OK) && (version >= CompositeKeyKeyStoreVersion)) {
+                    status = strSource.PullBytes(&keyType, sizeof(keyType), pulled);
+                }
                 if (status == ER_OK) {
                     status = strSource.PullBytes(guidBuf, qcc::GUID128::SIZE, pulled);
                 }
                 if (status == ER_OK) {
-                    qcc::GUID128 guid;
+                    qcc::GUID128 guid(0);
                     guid.SetBytes(guidBuf);
-                    KeyRecord& keyRec = (*keys)[guid];
+                    Key key(keyType, guid);
+                    KeyRecord& keyRec = (*keys)[key];
                     keyRec.revision = rev;
-                    status = keyRec.key.Load(strSource);
+                    status = keyRec.keyBlob.Load(strSource);
                     if (status == ER_OK) {
                         if (version > LowStoreVersion) {
                             status = strSource.PullBytes(&keyRec.accessRights, sizeof(keyRec.accessRights), pulled);
@@ -400,10 +508,11 @@ ExitPull:
 
 QStatus KeyStore::Clear()
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
-    lock.Lock(MUTEX_CONTEXT);
     keys->clear();
     storeState = MODIFIED;
     revision = 0;
@@ -413,6 +522,46 @@ QStatus KeyStore::Clear()
     return ER_OK;
 }
 
+static bool MatchesPrefix(const String& str, const String& prefixPattern)
+{
+    return !WildcardMatch(str, prefixPattern);
+}
+
+QStatus KeyStore::Clear(const qcc::String& tagPrefixPattern)
+{
+    lock.Lock(MUTEX_CONTEXT);
+    if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
+        return ER_BUS_KEYSTORE_NOT_LOADED;
+    }
+    bool dirty = true;
+    QStatus status = ER_OK;
+    while (dirty) {
+        dirty = false;
+        for (KeyMap::iterator it = keys->begin(); it != keys->end(); it++) {
+            if (it->second.keyBlob.GetTag().empty()) {
+                continue;  /* skip the untag */
+            }
+            if (MatchesPrefix(it->second.keyBlob.GetTag(), tagPrefixPattern)) {
+                status = DeleteKey(it->first);
+                if (ER_OK != status) {
+                    break;
+                }
+                dirty = true;
+                break;  /* redo the loop since the iterator is out-of-sync */
+            }
+        }
+        if (ER_OK != status) {
+            break;
+        }
+    }
+    lock.Unlock(MUTEX_CONTEXT);
+    if (ER_OK == status) {
+        listener->StoreRequest(*this);
+    }
+    return status;
+}
+
 QStatus KeyStore::Reload()
 {
     QCC_DbgHLPrintf(("KeyStore::Reload"));
@@ -420,17 +569,19 @@ QStatus KeyStore::Reload()
     /*
      * Cannot reload if the key store has never been loaded
      */
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
     /*
      * Reload is defined to be a no-op for non-shared key stores
      */
     if (!shared) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_OK;
     }
 
-    lock.Lock(MUTEX_CONTEXT);
     QStatus status;
     uint32_t currentRevision = revision;
     KeyMap* currentKeys = keys;
@@ -452,7 +603,7 @@ QStatus KeyStore::Reload()
         /*
          * Handle deletions
          */
-        std::set<qcc::GUID128>::iterator itDel;
+        std::set<Key>::iterator itDel;
         for (itDel = deletions.begin(); itDel != deletions.end(); ++itDel) {
             it = keys->find(*itDel);
             if ((it != keys->end()) && (it->second.revision <= currentRevision)) {
@@ -508,10 +659,13 @@ QStatus KeyStore::Push(Sink& sink)
     KeyMap::iterator it;
     for (it = keys->begin(); it != keys->end(); ++it) {
         strSink.PushBytes(&it->second.revision, sizeof(revision), pushed);
-        strSink.PushBytes(it->first.GetBytes(), qcc::GUID128::SIZE, pushed);
-        it->second.key.Store(strSink);
+        Key::KeyType keyType = it->first.GetType();
+        strSink.PushBytes(&keyType, sizeof(keyType), pushed);
+        strSink.PushBytes(it->first.GetGUID().GetBytes(), qcc::GUID128::SIZE, pushed);
+        it->second.keyBlob.Store(strSink);
+
         strSink.PushBytes(&it->second.accessRights, sizeof(it->second.accessRights), pushed);
-        QCC_DbgPrintf(("KeyStore::Push rev:%d GUID %s", it->second.revision, it->first.ToString().c_str()));
+        QCC_DbgPrintf(("KeyStore::Push rev:%d key %s", it->second.revision, it->first.ToString().c_str()));
     }
     size_t keysLen = strSink.GetString().size();
     /*
@@ -535,10 +689,12 @@ QStatus KeyStore::Push(Sink& sink)
      */
     if (status == ER_OK) {
         status = sink.PushBytes(thisGuid.GetBytes(), qcc::GUID128::SIZE, pushed);
+        MarkGuidSet(); /* now thisGuid is active */
     }
     if (status != ER_OK) {
         goto ExitPush;
     }
+
     if (keysLen > 0) {
         /*
          * Encrypt keys.
@@ -573,17 +729,18 @@ ExitPush:
     return status;
 }
 
-QStatus KeyStore::GetKey(const qcc::GUID128& guid, KeyBlob& key, uint8_t accessRights[4])
+QStatus KeyStore::GetKey(const Key& key, KeyBlob& keyBlob, uint8_t accessRights[4])
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
     QStatus status;
-    lock.Lock(MUTEX_CONTEXT);
-    QCC_DbgPrintf(("KeyStore::GetKey %s", guid.ToString().c_str()));
-    if (keys->find(guid) != keys->end()) {
-        KeyRecord& keyRec = (*keys)[guid];
-        key = keyRec.key;
+    QCC_DbgPrintf(("KeyStore::GetKey %s", key.ToString().c_str()));
+    if (keys->find(key) != keys->end()) {
+        KeyRecord& keyRec = (*keys)[key];
+        keyBlob = keyRec.keyBlob;
         memcpy(accessRights, &keyRec.accessRights, sizeof(uint8_t) * 4);
         QCC_DbgPrintf(("AccessRights %1x%1x%1x%1x", accessRights[0], accessRights[1], accessRights[2], accessRights[3]));
         status = ER_OK;
@@ -594,61 +751,77 @@ QStatus KeyStore::GetKey(const qcc::GUID128& guid, KeyBlob& key, uint8_t accessR
     return status;
 }
 
-bool KeyStore::HasKey(const qcc::GUID128& guid)
+bool KeyStore::HasKey(const Key& key)
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return false;
     }
-    bool hasKey;
-    lock.Lock(MUTEX_CONTEXT);
-    hasKey = keys->count(guid) != 0;
+    bool hasKey = keys->count(key) != 0;
     lock.Unlock(MUTEX_CONTEXT);
     return hasKey;
 }
 
-QStatus KeyStore::AddKey(const qcc::GUID128& guid, const KeyBlob& key, const uint8_t accessRights[4])
+QStatus KeyStore::AddKey(const Key& key, const KeyBlob& keyBlob, const uint8_t accessRights[4])
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
-    lock.Lock(MUTEX_CONTEXT);
-    QCC_DbgPrintf(("KeyStore::AddKey %s", guid.ToString().c_str()));
-    KeyRecord& keyRec = (*keys)[guid];
+    QCC_DbgPrintf(("KeyStore::AddKey %s", key.ToString().c_str()));
+    KeyRecord& keyRec = (*keys)[key];
     keyRec.revision = revision + 1;
-    keyRec.key = key;
+    keyRec.keyBlob = keyBlob;
     QCC_DbgPrintf(("AccessRights %1x%1x%1x%1x", accessRights[0], accessRights[1], accessRights[2], accessRights[3]));
     memcpy(&keyRec.accessRights, accessRights, sizeof(uint8_t) * 4);
     storeState = MODIFIED;
-    deletions.erase(guid);
+    deletions.erase(key);
     lock.Unlock(MUTEX_CONTEXT);
     return ER_OK;
 }
 
-QStatus KeyStore::DelKey(const qcc::GUID128& guid)
+/**
+ * This internal method deletes the key assuming the lock is already acquired.
+ */
+QStatus KeyStore::DeleteKey(const Key& key)
 {
     if (storeState == UNAVAILABLE) {
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
-    lock.Lock(MUTEX_CONTEXT);
-    QCC_DbgPrintf(("KeyStore::DelKey %s", guid.ToString().c_str()));
-    keys->erase(guid);
+    QCC_DbgPrintf(("KeyStore::DeleleKey %s", key.ToString().c_str()));
+    /* Use a local copy because erase might destroy the key */
+    Key keyCopy(key);
+    keys->erase(key);
     storeState = MODIFIED;
-    deletions.insert(guid);
+    deletions.insert(keyCopy);
+    return ER_OK;
+}
+
+QStatus KeyStore::DelKey(const Key& key)
+{
+    lock.Lock(MUTEX_CONTEXT);
+    QStatus status = DeleteKey(key);
     lock.Unlock(MUTEX_CONTEXT);
+    if (ER_OK != status) {
+        return status;
+    }
     listener->StoreRequest(*this);
     return ER_OK;
 }
 
-QStatus KeyStore::SetKeyExpiration(const qcc::GUID128& guid, const Timespec& expiration)
+QStatus KeyStore::SetKeyExpiration(const Key& key, const Timespec& expiration)
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
     QStatus status = ER_OK;
-    lock.Lock(MUTEX_CONTEXT);
-    QCC_DbgPrintf(("KeyStore::SetExpiration %s", guid.ToString().c_str()));
-    if (keys->count(guid) != 0) {
-        (*keys)[guid].key.SetExpiration(expiration);
+    QCC_DbgPrintf(("KeyStore::SetExpiration %s", key.ToString().c_str()));
+    if (keys->count(key) != 0) {
+        (*keys)[key].keyBlob.SetExpiration(expiration);
         storeState = MODIFIED;
     } else {
         status = ER_BUS_KEY_UNAVAILABLE;
@@ -660,20 +833,23 @@ QStatus KeyStore::SetKeyExpiration(const qcc::GUID128& guid, const Timespec& exp
     return status;
 }
 
-QStatus KeyStore::GetKeyExpiration(const qcc::GUID128& guid, Timespec& expiration)
+QStatus KeyStore::GetKeyExpiration(const Key& key, Timespec& expiration)
 {
+    lock.Lock(MUTEX_CONTEXT);
     if (storeState == UNAVAILABLE) {
+        lock.Unlock(MUTEX_CONTEXT);
         return ER_BUS_KEYSTORE_NOT_LOADED;
     }
+    lock.Unlock(MUTEX_CONTEXT);
     /*
      * For shared key stores we may need to do a reload before checking for key expiration.
      */
     QStatus status = Reload();
     if (status == ER_OK) {
         lock.Lock(MUTEX_CONTEXT);
-        QCC_DbgPrintf(("KeyStore::GetExpiration %s", guid.ToString().c_str()));
-        if (keys->count(guid) != 0) {
-            (*keys)[guid].key.GetExpiration(expiration);
+        QCC_DbgPrintf(("KeyStore::GetExpiration %s", key.ToString().c_str()));
+        if (keys->count(key) != 0) {
+            (*keys)[key].keyBlob.GetExpiration(expiration);
         } else {
             status = ER_BUS_KEY_UNAVAILABLE;
         }
@@ -682,15 +858,16 @@ QStatus KeyStore::GetKeyExpiration(const qcc::GUID128& guid, Timespec& expiratio
     return status;
 }
 
-QStatus KeyStore::SearchAssociatedKeys(const qcc::GUID128& guid, qcc::GUID128** list, size_t* numItems) {
+QStatus KeyStore::SearchAssociatedKeys(const Key& key, Key** list, size_t* numItems)
+{
     size_t count = 0;
     lock.Lock(MUTEX_CONTEXT);
     for (KeyMap::iterator it = keys->begin(); it != keys->end(); ++it) {
-        if ((it->second.key.GetAssociationMode() != KeyBlob::ASSOCIATE_MEMBER)
-            && (it->second.key.GetAssociationMode() != KeyBlob::ASSOCIATE_BOTH)) {
+        if ((it->second.keyBlob.GetAssociationMode() != KeyBlob::ASSOCIATE_MEMBER)
+            && (it->second.keyBlob.GetAssociationMode() != KeyBlob::ASSOCIATE_BOTH)) {
             continue;
         }
-        if (it->second.key.GetAssociation() == guid) {
+        if (it->second.keyBlob.GetAssociation() == key.GetGUID()) {
             count++;
         }
     }
@@ -700,25 +877,68 @@ QStatus KeyStore::SearchAssociatedKeys(const qcc::GUID128& guid, qcc::GUID128**
         return ER_OK;
     }
 
-    qcc::GUID128*guids = new qcc::GUID128[count];
+    Key* keyList = new Key[count];
     size_t idx = 0;
     for (KeyMap::iterator it = keys->begin(); it != keys->end(); ++it) {
-        if ((it->second.key.GetAssociationMode() != KeyBlob::ASSOCIATE_MEMBER)
-            && (it->second.key.GetAssociationMode() != KeyBlob::ASSOCIATE_BOTH)) {
+        if ((it->second.keyBlob.GetAssociationMode() != KeyBlob::ASSOCIATE_MEMBER)
+            && (it->second.keyBlob.GetAssociationMode() != KeyBlob::ASSOCIATE_BOTH)) {
             continue;
         }
-        if (it->second.key.GetAssociation() == guid) {
+        if (it->second.keyBlob.GetAssociation() == key.GetGUID()) {
             if (idx >= count) { /* bound check */
-                delete [] guids;
+                delete [] keyList;
                 lock.Unlock(MUTEX_CONTEXT);
                 return ER_FAIL;
             }
-            guids[idx++] = it->first;
+            keyList[idx++] = it->first;
         }
     }
     *numItems = count;
-    *list = guids;
+    *list = keyList;
     lock.Unlock(MUTEX_CONTEXT);
     return ER_OK;
 }
+
+QStatus KeyStore::WaitForGuidSet()
+{
+    if (guidSet) {
+        return ER_OK;
+    }
+    guidSetEventLock.Lock(MUTEX_CONTEXT);
+    if (guidSetRefCount == 0) {
+        guidSetEvent = new Event();
+    }
+    guidSetRefCount++;
+    guidSetEventLock.Unlock(MUTEX_CONTEXT);
+    Event::Wait(*guidSetEvent);
+    guidSetEventLock.Lock(MUTEX_CONTEXT);
+    guidSetRefCount--;
+    if (guidSetRefCount == 0) {
+        delete guidSetEvent;
+        guidSetEvent = NULL;
+    }
+    guidSetEventLock.Unlock(MUTEX_CONTEXT);
+    return ER_OK;
+}
+
+void KeyStore::MarkGuidSet()
+{
+    guidSet = true;
+    if (guidSetEvent) {
+        guidSetEvent->SetEvent();
+    }
+}
+
+void KeyStore::KeyStoreEncryptionKey::Build(const qcc::String& password, const qcc::String& guidString)
+{
+    SetPassword(password);
+    SetGuidString(guidString);
+    Build();
+}
+
+void KeyStore::KeyStoreEncryptionKey::Build()
+{
+    Derive(password + guidString, Crypto_AES::AES128_SIZE, KeyBlob::AES);
+}
+
 }
