@@ -52,10 +52,8 @@ using namespace qcc;
 
 namespace ajn {
 
-
 static const uint32_t LOCAL_ENDPOINT_CONCURRENCY = 4;
 
-
 class _LocalEndpoint::Dispatcher : public qcc::Timer, public qcc::AlarmListener {
   public:
     Dispatcher(_LocalEndpoint* endpoint, uint32_t concurrency = LOCAL_ENDPOINT_CONCURRENCY) :
@@ -83,7 +81,7 @@ class _LocalEndpoint::Dispatcher : public qcc::Timer, public qcc::AlarmListener
 
   private:
     _LocalEndpoint* endpoint;
-    static int32_t dispatcherCnt;
+    static volatile int32_t dispatcherCnt;
 
     Alarm pendingWork;
     bool needDeferredCallbacks;
@@ -92,7 +90,7 @@ class _LocalEndpoint::Dispatcher : public qcc::Timer, public qcc::AlarmListener
     qcc::Mutex workLock;
 };
 
-int32_t _LocalEndpoint::Dispatcher::dispatcherCnt = 0;
+volatile int32_t _LocalEndpoint::Dispatcher::dispatcherCnt = 0;
 
 LocalTransport::~LocalTransport()
 {
@@ -427,7 +425,9 @@ QStatus _LocalEndpoint::Dispatcher::DispatchMessage(Message& msg)
     uint32_t zero = 0;
     void* context = new Message(msg);
     qcc::AlarmListener* localEndpointListener = this;
-    Alarm alarm(zero, localEndpointListener, context, zero);
+    bool limitable = (endpoint->GetUniqueName() != msg->GetSender());
+    Alarm alarm(zero, localEndpointListener, context, zero, limitable);
+
     QStatus status = AddAlarm(alarm);
     if (status != ER_OK) {
         Message* temp = static_cast<Message*>(context);
@@ -704,7 +704,7 @@ QStatus _LocalEndpoint::RegisterBusObject(BusObject& object, bool isSecure)
             BusObject* parent = FindLocalObject(parentPath.c_str());
             if (!parent) {
                 parent = new BusObject(parentPath.c_str(), true);
-                QStatus status = DoRegisterBusObject(*parent, lastParent, true);
+                status = DoRegisterBusObject(*parent, lastParent, true);
                 if (ER_OK != status) {
                     delete parent;
                     QCC_LogError(status, ("Failed to register default object for path %s", parentPath.c_str()));
@@ -784,19 +784,9 @@ void _LocalEndpoint::UnregisterBusObject(BusObject& object)
     QCC_DbgPrintf(("UnregisterBusObject %s", object.GetPath()));
 
     /* Can't unregister while handlers are in flight. */
-    handlerThreadsLock.Lock(MUTEX_CONTEXT);
-    ActiveHandlers::const_iterator ahit = activeHandlers.find(&object);
-    if ((ahit != activeHandlers.end()) && (ahit->second.find(Thread::GetThread()) != ahit->second.end())) {
-        QCC_LogError(ER_DEADLOCK, ("Attempt to unregister BusObject from said BusObject's message handler -- BusObject not unregistered!"));
-        handlerThreadsLock.Unlock(MUTEX_CONTEXT);
+    if (!OkToUnregisterHandlerObj(&object)) {
         return;
     }
-    unregisteringObjects.insert(&object);
-    while (ahit != activeHandlers.end()) {
-        handlerThreadsDone.Wait(handlerThreadsLock);
-        ahit = activeHandlers.find(&object);
-    }
-    handlerThreadsLock.Unlock(MUTEX_CONTEXT);
 
     /* Remove members */
     methodTable.RemoveAll(&object);
@@ -831,11 +821,12 @@ void _LocalEndpoint::UnregisterBusObject(BusObject& object)
         object.InUseDecrement();
     }
     /* Delete the object if it was a default object */
+    bool deleteObj = false;
     vector<BusObject*>::iterator dit = defaultObjects.begin();
     while (dit != defaultObjects.end()) {
         if (*dit == &object) {
             defaultObjects.erase(dit);
-            delete &object;
+            deleteObj = true;
             break;
         } else {
             ++dit;
@@ -843,10 +834,10 @@ void _LocalEndpoint::UnregisterBusObject(BusObject& object)
     }
     objectsLock.Unlock(MUTEX_CONTEXT);
 
-    handlerThreadsLock.Lock(MUTEX_CONTEXT);
-    /* Yes object is deleted, but we never actually reference it from unregisteringObjects, so we are safe. */
-    unregisteringObjects.erase(&object);
-    handlerThreadsLock.Unlock(MUTEX_CONTEXT);
+    UnregisterComplete(&object);
+    if (deleteObj) {
+        delete &object;
+    }
 }
 
 BusObject* _LocalEndpoint::FindLocalObject(const char* objectPath) {
@@ -1041,6 +1032,33 @@ QStatus _LocalEndpoint::RegisterSignalHandler(MessageReceiver* receiver,
     return ER_OK;
 }
 
+bool _LocalEndpoint::OkToUnregisterHandlerObj(MessageReceiver* receiver)
+{
+    /* Can't unregister while handlers are in flight. */
+    handlerThreadsLock.Lock(MUTEX_CONTEXT);
+    ActiveHandlers::const_iterator ahit = activeHandlers.find(receiver);
+    if ((ahit != activeHandlers.end()) && (ahit->second.find(Thread::GetThread()) != ahit->second.end())) {
+        QCC_LogError(ER_DEADLOCK, ("Attempt to unregister MessageReceiver from said MessageReceiver's message handler -- MessageReceiver not unregistered!"));
+        handlerThreadsLock.Unlock(MUTEX_CONTEXT);
+        return false;
+    }
+    unregisteringObjects.insert(receiver);
+    while (ahit != activeHandlers.end()) {
+        handlerThreadsDone.Wait(handlerThreadsLock);
+        ahit = activeHandlers.find(receiver);
+    }
+    handlerThreadsLock.Unlock(MUTEX_CONTEXT);
+
+    return true;
+}
+
+void _LocalEndpoint::UnregisterComplete(MessageReceiver* receiver)
+{
+    handlerThreadsLock.Lock(MUTEX_CONTEXT);
+    unregisteringObjects.erase(receiver);
+    handlerThreadsLock.Unlock(MUTEX_CONTEXT);
+}
+
 QStatus _LocalEndpoint::UnregisterSignalHandler(MessageReceiver* receiver,
                                                 MessageReceiver::SignalHandler signalHandler,
                                                 const InterfaceDescription::Member* member,
@@ -1058,11 +1076,20 @@ QStatus _LocalEndpoint::UnregisterSignalHandler(MessageReceiver* receiver,
     if (!matchRule) {
         return ER_BAD_ARG_4;
     }
-    return signalTable.Remove(receiver, signalHandler, member, matchRule);
+
+    QStatus status = ER_DEADLOCK;
+    if (OkToUnregisterHandlerObj(receiver)) {
+        status =  signalTable.Remove(receiver, signalHandler, member, matchRule);
+        UnregisterComplete(receiver);
+    }
+    return status;
 }
 
 QStatus _LocalEndpoint::UnregisterAllHandlers(MessageReceiver* receiver)
 {
+    if (!OkToUnregisterHandlerObj(receiver)) {
+        return ER_DEADLOCK;
+    }
     /*
      * Remove all the signal handlers for this receiver.
      */
@@ -1093,6 +1120,8 @@ QStatus _LocalEndpoint::UnregisterAllHandlers(MessageReceiver* receiver)
         }
     }
     replyMapLock.Unlock(MUTEX_CONTEXT);
+
+    UnregisterComplete(receiver);
     return ER_OK;
 }
 
@@ -1238,6 +1267,11 @@ QStatus _LocalEndpoint::HandleMethodCall(Message& message)
             errMsg = QCC_StatusText(status);
             break;
 
+        case ER_PERMISSION_DENIED:
+            errStr = "org.alljoyn.Bus.Security.Error.PermissionDenied";
+            errMsg = message->Description();
+            break;
+
         default:
             errStr += "org.alljoyn.Bus.";
             errStr += QCC_StatusText(status);
