@@ -39,6 +39,7 @@
 
 #include "BusInternal.h"
 #include "BusUtil.h"
+#include "PermissionMgmtObj.h"
 
 #define QCC_MODULE "ALLJOYN"
 
@@ -319,14 +320,16 @@ _Message::_Message(BusAttachment& bus, const HeaderFields& hdrFields)
     this->hdrFields = hdrFields;
 }
 
-void _Message::Init(BusAttachment& bus)
+void _Message::Init(BusAttachment& busAttachment)
 {
-    this->bus = &bus;
+    this->bus = &busAttachment;
     endianSwap = false;
     _msgBuf = NULL;
     msgBuf = NULL;
     msgArgs = NULL;
     numMsgArgs = 0;
+    refMsgArgs = NULL;
+    numRefMsgArgs = 0;
     ttl = 0;
     handles = NULL;
     numHandles = 0;
@@ -338,6 +341,8 @@ void _Message::Init(BusAttachment& bus)
     countWrite = 0;
     msgHeader.msgType = MESSAGE_INVALID;
     msgHeader.endian = myEndian;
+    encryptionNotification = NULL;
+    authorizationChecked = false;
 }
 
 _Message::~_Message(void)
@@ -348,6 +353,7 @@ _Message::~_Message(void)
         qcc::Close(handles[--numHandles]);
     }
     delete [] handles;
+    delete [] refMsgArgs;
 }
 
 _Message::_Message(const _Message& other) :
@@ -355,6 +361,7 @@ _Message::_Message(const _Message& other) :
     endianSwap(other.endianSwap),
     msgHeader(other.msgHeader),
     numMsgArgs(other.numMsgArgs),
+    numRefMsgArgs(other.numRefMsgArgs),
     bufSize(other.bufSize),
     ttl(other.ttl),
     timestamp(other.timestamp),
@@ -368,7 +375,9 @@ _Message::_Message(const _Message& other) :
     countRead(other.countRead),
     writeState(other.writeState),
     countWrite(other.countWrite),
-    hdrFields(other.hdrFields)
+    hdrFields(other.hdrFields),
+    encryptionNotification(other.encryptionNotification),
+    authorizationChecked(other.authorizationChecked)
 {
     if (bufSize > 0) {
         assert(other.msgBuf != NULL);
@@ -398,6 +407,14 @@ _Message::_Message(const _Message& other) :
     } else {
         msgArgs = NULL;
     }
+    if (numRefMsgArgs > 0) {
+        refMsgArgs =  new MsgArg[numRefMsgArgs];
+        for (size_t i = 0; i < numRefMsgArgs; ++i) {
+            refMsgArgs[i] = other.refMsgArgs[i];
+        }
+    } else {
+        refMsgArgs = NULL;
+    }
     if (numHandles > 0) {
         handles = new qcc::SocketFd[numHandles];
         for (size_t i = 0; i < numHandles; ++i) {
@@ -420,6 +437,9 @@ QStatus _Message::ReMarshal(const char* senderName)
     delete [] msgArgs;
     msgArgs = NULL;
     numMsgArgs = 0;
+    delete [] refMsgArgs;
+    refMsgArgs = NULL;
+    numRefMsgArgs = 0;
 
     /*
      * We delete the current buffer after we have copied the body data
@@ -512,6 +532,9 @@ void _Message::ClearHeader()
         delete [] msgArgs;
         msgArgs = NULL;
         numMsgArgs = 0;
+        delete [] refMsgArgs;
+        refMsgArgs = NULL;
+        numRefMsgArgs = 0;
         ttl = 0;
         msgHeader.msgType = MESSAGE_INVALID;
         while (numHandles) {
@@ -524,4 +547,11 @@ void _Message::ClearHeader()
     }
 }
 
+void _Message::NotifyEncryptionComplete()
+{
+    if (NULL != encryptionNotification) {
+        encryptionNotification->EncryptionComplete();
+    }
+}
+
 }
