@@ -109,6 +109,8 @@ class CachedProps {
     bool isFullyCacheable;
     size_t numProperties;
     uint32_t lastMessageSerial;
+    bool enabled;
+
     bool IsCacheable(const char* propname);
     bool IsValidMessageSerial(uint32_t messageSerial);
 
@@ -116,11 +118,13 @@ class CachedProps {
     CachedProps() :
         lock(), values(), description(NULL),
         isFullyCacheable(false),
-        numProperties(0), lastMessageSerial(0) { }
+        numProperties(0), lastMessageSerial(0),
+        enabled(false) { }
 
     CachedProps(const InterfaceDescription*intf) :
         lock(), values(), description(intf),
-        isFullyCacheable(false), lastMessageSerial(0) {
+        isFullyCacheable(false), lastMessageSerial(0),
+        enabled(false) {
         numProperties = description->GetProperties();
         if (numProperties > 0) {
             isFullyCacheable = true;
@@ -139,7 +143,8 @@ class CachedProps {
     CachedProps(const CachedProps& other) :
         lock(), values(other.values), description(other.description),
         isFullyCacheable(other.isFullyCacheable),
-        numProperties(other.numProperties), lastMessageSerial(other.lastMessageSerial) { }
+        numProperties(other.numProperties), lastMessageSerial(other.lastMessageSerial),
+        enabled(other.enabled) { }
 
     CachedProps& operator=(const CachedProps& other) {
         if (&other != this) {
@@ -148,6 +153,7 @@ class CachedProps {
             isFullyCacheable = other.isFullyCacheable;
             numProperties = other.numProperties;
             lastMessageSerial = other.lastMessageSerial;
+            enabled = other.enabled;
         }
         return *this;
     }
@@ -159,6 +165,7 @@ class CachedProps {
     void Set(const char* propname, const MsgArg& val, const uint32_t messageSerial);
     void SetAll(const MsgArg& allValues, const uint32_t messageSerial);
     void PropertiesChanged(MsgArg* changed, size_t numChanged, MsgArg* invalidated, size_t numInvalidated, const uint32_t messageSerial);
+    void Enable();
 };
 
 
@@ -174,7 +181,26 @@ struct _SyncReplyContext {
 };
 typedef ManagedObj<_SyncReplyContext> SyncReplyContext;
 
-class ProxyBusObject::Internal : public MessageReceiver {
+class ProxyBusObject::Internal : public MessageReceiver, public BusAttachment::AddMatchAsyncCB {
+    class AddMatchCBInfo {
+      public:
+        String ifaceName;
+        bool& addingRef;
+        AddMatchCBInfo(const String& iface, bool& addingRef) : ifaceName(iface), addingRef(addingRef) { }
+      private:
+        AddMatchCBInfo& operator=(const AddMatchCBInfo& other);
+    };
+
+    class MatchRuleInfo {
+      public:
+        bool adding;
+        uint32_t refCount;
+        MatchRuleInfo() : adding(false), refCount(0) { }
+        MatchRuleInfo(bool adding) : adding(adding), refCount(0) { }
+      private:
+        MatchRuleInfo& operator=(const MatchRuleInfo& other);
+    };
+
   public:
     Internal() :
         bus(nullptr),
@@ -256,6 +282,13 @@ class ProxyBusObject::Internal : public MessageReceiver {
      */
     void PropertiesChangedHandler(const InterfaceDescription::Member* member, const char* srcPath, Message& message);
 
+
+    /**
+     * @internal
+     * Handle property AddMatch reply. (Internal use only)
+     */
+    void AddMatchCB(QStatus status, void* context);
+
     bool operator==(const ProxyBusObject::Internal& other) const
     {
         return ((this == &other) ||
@@ -280,6 +313,7 @@ class ProxyBusObject::Internal : public MessageReceiver {
     mutable Mutex lock;                 /**< Lock that protects access to internal state */
     Condition listenerDone;             /**< Signals that the properties changed listener is done */
     Condition handlerDone;              /**< Signals that the properties changed signal handler is done */
+    Condition addMatchDone;             /**< Signals that AddMatch call has completed */
     bool registeredPropChangedHandler;  /**< true if our PropertiesChangedHandler is registered */
     map<qcc::Thread*, _PropertiesChangedCB*> handlerThreads;   /**< Thread actively calling PropertiesChangedListeners */
 
@@ -296,8 +330,9 @@ class ProxyBusObject::Internal : public MessageReceiver {
     mutable map<const ProxyBusObject* const, set<SyncReplyContext> > syncMethodCalls;
     mutable Condition syncMethodComplete;
 
-    /** Match rule bookkeeping */
-    map<qcc::StringMapKey, int> matchRuleRefcounts;
+    /** Match rule book keeping */
+    typedef map<StringMapKey, MatchRuleInfo> MatchRuleBookKeeping;
+    MatchRuleBookKeeping matchRuleBookKeeping;
 
     /** Property changed handlers */
     multimap<StringMapKey, PropertiesChangedCB> propertiesChangedCBs;
@@ -305,32 +340,36 @@ class ProxyBusObject::Internal : public MessageReceiver {
 
 ProxyBusObject::Internal::~Internal()
 {
-    lock.Lock(MUTEX_CONTEXT);
-    QCC_DbgPrintf(("Destroying PBO internal (%p) for %s on %s (%s)", this, path.c_str(), serviceName.c_str(), uniqueName.c_str()));
-    if (registeredPropChangedHandler && bus) {
-        /*
-         * Unregister the PropertiesChanged signal handler without holding the
-         * PBO lock, because the signal handler itself acquires the lock. The
-         * unregistration procedure busy-waits for a signal handler to finish
-         * before proceeding with the unregistration, so if we hold the lock here,
-         * we can create a deadlock.
-         */
-        const InterfaceDescription* iface = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
-        if (iface) {
-            bus->UnregisterSignalHandler(this,
-                                         static_cast<MessageReceiver::SignalHandler>(&Internal::PropertiesChangedHandler),
-                                         iface->GetMember("PropertiesChanged"),
-                                         path.c_str());
-        }
-    }
-
-    if (bus) {
-        bus->UnregisterAllHandlers(this);
-    }
-
     /* remove match rules added by the property caching & change notification mechanism */
     RemoveAllPropertiesChangedRules();
 
+    lock.Lock(MUTEX_CONTEXT);
+    bool unregHandler = registeredPropChangedHandler;
+    lock.Unlock(MUTEX_CONTEXT);
+
+    QCC_DbgPrintf(("Destroying PBO internal (%p) for %s on %s (%s)", this, path.c_str(), serviceName.c_str(), uniqueName.c_str()));
+    if (bus) {
+        if (unregHandler) {
+            /*
+             * Unregister the PropertiesChanged signal handler without holding
+             * the PBO lock, because the signal handler itself acquires the
+             * lock. The unregistration procedure busy-waits for a signal
+             * handler to finish before proceeding with the unregistration, so
+             * if we hold the lock here, we can create a deadlock.
+             */
+            const InterfaceDescription* iface = bus->GetInterface(org::freedesktop::DBus::Properties::InterfaceName);
+            if (iface) {
+                bus->UnregisterSignalHandler(this,
+                                             static_cast<MessageReceiver::SignalHandler>(&Internal::PropertiesChangedHandler),
+                                             iface->GetMember("PropertiesChanged"),
+                                             path.c_str());
+            }
+        }
+
+        bus->UnregisterAllHandlers(this);
+    }
+
+    lock.Lock(MUTEX_CONTEXT);
     /* Clean up properties changed listeners */
     while (!handlerThreads.empty()) {
         /*
@@ -348,6 +387,50 @@ ProxyBusObject::Internal::~Internal()
     lock.Unlock(MUTEX_CONTEXT);
 }
 
+/**
+ * Figure out the new status code based on the the reply message's error.
+ * If the status code is ER_BUS_REPLY_IS_ERROR_MESSAGE then the error
+ * message is searched to compute the status code.
+ * @param reply the reply message
+ * @param[in,out] status the status code
+ */
+static void GetReplyErrorStatus(Message& reply, QStatus& status)
+{
+    if (ER_BUS_REPLY_IS_ERROR_MESSAGE != status) {
+        return;
+    }
+    if (reply->GetErrorName() == NULL) {
+        return;
+    }
+    if (strcmp(reply->GetErrorName(), PermissionMgmtObj::ERROR_PERMISSION_DENIED) == 0) {
+        status = ER_PERMISSION_DENIED;
+    } else if (strcmp(reply->GetErrorName(), org::alljoyn::Bus::ErrorName) == 0 && reply->GetArg(1)) {
+        status = static_cast<QStatus>(reply->GetArg(1)->v_uint16);
+    }
+}
+
+/**
+ * Figure out whether the reply message is a permission denied error message.
+ * If so, the status code will be replaced with ER_PERMISSION_DENIED.
+ * @param reply the reply message
+ * @param[in,out] status the status code
+ */
+static void AdjustErrorForPermissionDenied(Message& reply, QStatus& status)
+{
+    if (ER_PERMISSION_DENIED == status) {
+        return;
+    }
+    if (ER_BUS_REPLY_IS_ERROR_MESSAGE != status) {
+        return;
+    }
+    QStatus tmpStatus = status;
+    GetReplyErrorStatus(reply, tmpStatus);
+    if (ER_PERMISSION_DENIED == tmpStatus) {
+        status = tmpStatus;
+        return;
+    }
+}
+
 static inline bool SecurityApplies(const ProxyBusObject* obj, const InterfaceDescription* ifc)
 {
     InterfaceSecurityPolicy ifcSec = ifc->GetSecurityPolicy();
@@ -564,6 +647,8 @@ QStatus ProxyBusObject::GetProperty(const char* iface, const char* property, Msg
                     }
                 }
                 internal->lock.Unlock(MUTEX_CONTEXT);
+            } else {
+                GetReplyErrorStatus(reply, status);
             }
         }
     }
@@ -700,15 +785,8 @@ QStatus ProxyBusObject::SetProperty(const char* iface, const char* property, Msg
                                 reply,
                                 timeout,
                                 flags);
-            if ((status == ER_BUS_REPLY_IS_ERROR_MESSAGE) &&
-                (reply->GetErrorName() != NULL) &&
-                (::strcmp(reply->GetErrorName(), org::alljoyn::Bus::ErrorName) == 0)) {
-                const char* err;
-                uint16_t rawStatus;
-                if (reply->GetArgs("sq", &err, &rawStatus) == ER_OK) {
-                    status = static_cast<QStatus>(rawStatus);
-                    QCC_DbgPrintf(("SetProperty call returned %s", err));
-                }
+            if (ER_OK != status) {
+                GetReplyErrorStatus(reply, status);
             }
         }
     }
@@ -763,9 +841,9 @@ QStatus ProxyBusObject::RegisterPropertiesChangedListener(const char* iface,
     multimap<StringMapKey, PropertiesChangedCB>::iterator it = internal->propertiesChangedCBs.lower_bound(iface);
     multimap<StringMapKey, PropertiesChangedCB>::iterator end = internal->propertiesChangedCBs.upper_bound(iface);
     while (it != end) {
-        PropertiesChangedCB ctx = it->second;
-        if (&ctx->listener == &listener) {
-            ctx->isRegistered = false;
+        PropertiesChangedCB propChangedCb = it->second;
+        if (&propChangedCb->listener == &listener) {
+            propChangedCb->isRegistered = false;
             internal->propertiesChangedCBs.erase(it);
             replace = true;
             break;
@@ -850,7 +928,10 @@ void ProxyBusObject::Internal::PropertiesChangedHandler(const InterfaceDescripti
     size_t numInvalidProps;
 
     if (uniqueName != message->GetSender()) {
-        QCC_LogError(ER_FAIL, ("message not for us? (uniqueName = %s   sender = %s)", uniqueName.c_str(), message->GetSender()));
+        /*
+         * Message may not be for us if different senders but same object paths:
+         * RegisterSignalHandler is bound to the path, not the sender.
+         */
         return;
     }
 
@@ -1039,15 +1120,15 @@ const InterfaceDescription* ProxyBusObject::GetInterface(const char* ifaceName)
 QStatus ProxyBusObject::AddInterface(const InterfaceDescription& iface) {
     StringMapKey key(qcc::String(iface.GetName()));
     pair<StringMapKey, const InterfaceDescription*> item(key, &iface);
-    internal->lock.Lock(MUTEX_CONTEXT);
+    bool addRule = false;
 
+    internal->lock.Lock(MUTEX_CONTEXT);
     pair<map<StringMapKey, const InterfaceDescription*>::const_iterator, bool> ret = internal->ifaces.insert(item);
     QStatus status = ret.second ? ER_OK : ER_BUS_IFACE_ALREADY_EXISTS;
 
     if ((status == ER_OK) && internal->cacheProperties && iface.HasCacheableProperties()) {
         internal->caches.insert(std::make_pair(key, CachedProps(&iface)));
-        /* add match rules in case the PropertiesChanged signals are emitted as global broadcast */
-        internal->AddPropertiesChangedRule(iface.GetName(), false);
+        addRule = true;
     }
 
     if ((status == ER_OK) && !internal->hasProperties) {
@@ -1060,6 +1141,12 @@ QStatus ProxyBusObject::AddInterface(const InterfaceDescription& iface) {
         }
     }
     internal->lock.Unlock(MUTEX_CONTEXT);
+
+    if (addRule) {
+        /* add match rules in case the PropertiesChanged signals are emitted as global broadcast */
+        internal->AddPropertiesChangedRule(iface.GetName(), false);
+    }
+
     return status;
 }
 
@@ -1074,12 +1161,53 @@ QStatus ProxyBusObject::AddInterface(const char* ifaceName)
     }
 }
 
+void ProxyBusObject::Internal::AddMatchCB(QStatus status, void* context)
+{
+    QCC_UNUSED(status);
+
+    AddMatchCBInfo* info = reinterpret_cast<AddMatchCBInfo*>(context);
+    lock.Lock(MUTEX_CONTEXT);
+    info->addingRef = false;
+    /* enable property caches */
+    map<StringMapKey, CachedProps>::iterator it = caches.find(info->ifaceName);
+    if (it != caches.end()) {
+        it->second.Enable();
+    }
+    addMatchDone.Broadcast();
+    lock.Unlock(MUTEX_CONTEXT);
+    delete info;
+}
+
 void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool blocking)
 {
     QCC_DbgTrace(("%s(%s)", __FUNCTION__, intf));
+
+    bool registerHandler = false;
+    bool callAddMatch = false;
+
     lock.Lock(MUTEX_CONTEXT);
-    /* make sure we have the signal handler online */
+    MatchRuleBookKeeping::iterator it = matchRuleBookKeeping.find(intf);
+    if (it == matchRuleBookKeeping.end()) {
+        callAddMatch = true;
+        pair<MatchRuleBookKeeping::iterator, bool> r;
+        /*
+         * Setup placeholder.  Other threads that call this function with the
+         * same interface after us will block until our AddMatch call completes.
+         */
+        r = matchRuleBookKeeping.insert(pair<StringMapKey, MatchRuleInfo>(qcc::String(intf), true));
+        it = r.first;
+    }
+
+    assert(it != matchRuleBookKeeping.end());
+    ++it->second.refCount;
+
     if (!registeredPropChangedHandler) {
+        registerHandler = true;
+        registeredPropChangedHandler = true;
+    }
+    lock.Unlock(MUTEX_CONTEXT);
+
+    if (registerHandler) {
         QCC_DbgPrintf(("Registering signal handler"));
         const InterfaceDescription* propIntf = bus->GetInterface(::ajn::org::freedesktop::DBus::Properties::InterfaceName);
         assert(propIntf);
@@ -1087,21 +1215,36 @@ void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool b
                                    static_cast<MessageReceiver::SignalHandler>(&Internal::PropertiesChangedHandler),
                                    propIntf->GetMember("PropertiesChanged"),
                                    path.c_str());
-        registeredPropChangedHandler = true;
     }
 
-    std::map<qcc::StringMapKey, int>::iterator it = matchRuleRefcounts.find(intf);
-    if (it == matchRuleRefcounts.end()) {
-        QCC_DbgPrintf(("Adding match rule"));
+    if (callAddMatch) {
         String rule = String("type='signal',interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',arg0='") + intf + "'";
-        if (blocking) {
-            bus->AddMatch(rule.c_str());
-        } else {
-            bus->AddMatchNonBlocking(rule.c_str());
+        AddMatchCBInfo* cbInfo = new AddMatchCBInfo(intf, it->second.adding);
+        QStatus status = bus->AddMatchAsync(rule.c_str(), this, cbInfo);
+        if (status != ER_OK) {
+            --it->second.refCount;
+            delete cbInfo;
+            return;
+        }
+    }
+
+    lock.Lock(MUTEX_CONTEXT);
+
+    /* If we already have a match rule installed for this interface, enable the property
+     * cache. This is an idempotent operation, so we don't care if we did this before.
+     * If we don't have the match rule yet, PBO::Internal::AddMatchCB will enable the
+     * cache for us. */
+    if (!it->second.adding) {
+        map<StringMapKey, CachedProps>::iterator cit = caches.find(intf);
+        if (cit != caches.end()) {
+            cit->second.Enable();
+        }
+    }
+
+    if (blocking) {
+        while (it->second.adding) {
+            addMatchDone.Wait(lock);
         }
-        matchRuleRefcounts[qcc::String(intf)] = 1;
-    } else {
-        it->second++;
     }
     lock.Unlock(MUTEX_CONTEXT);
 }
@@ -1109,12 +1252,19 @@ void ProxyBusObject::Internal::AddPropertiesChangedRule(const char* intf, bool b
 void ProxyBusObject::Internal::RemovePropertiesChangedRule(const char* intf)
 {
     lock.Lock(MUTEX_CONTEXT);
-    std::map<qcc::StringMapKey, int>::iterator it = matchRuleRefcounts.find(intf);
-    if (it != matchRuleRefcounts.end()) {
-        if (--it->second == 0) {
+    MatchRuleBookKeeping::iterator it = matchRuleBookKeeping.find(intf);
+    if (it != matchRuleBookKeeping.end()) {
+        /*
+         * Check if there is a callback pending that will access this iterator.
+         * If so, need to wait for that callback to complete before removing it.
+         */
+        while (it->second.adding) {
+            addMatchDone.Wait(lock);
+        }
+        if (--it->second.refCount == 0) {
             String rule = String("type='signal',interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',arg0='") + intf + "'";
             bus->RemoveMatchNonBlocking(rule.c_str());
-            matchRuleRefcounts.erase(it);
+            matchRuleBookKeeping.erase(it);
         }
     }
     lock.Unlock(MUTEX_CONTEXT);
@@ -1123,12 +1273,20 @@ void ProxyBusObject::Internal::RemovePropertiesChangedRule(const char* intf)
 void ProxyBusObject::Internal::RemoveAllPropertiesChangedRules()
 {
     lock.Lock(MUTEX_CONTEXT);
-    std::map<qcc::StringMapKey, int>::iterator it = matchRuleRefcounts.begin();
-    for (; it != matchRuleRefcounts.end(); ++it) {
+    MatchRuleBookKeeping::iterator it = matchRuleBookKeeping.begin();
+    for (; it != matchRuleBookKeeping.end(); ++it) {
+        /*
+         * Check if there is a callback pending that will access the same data
+         * referenced by this iterator.  If so, need to wait for that callback
+         * to complete before removing it.
+         */
+        while (it->second.adding) {
+            addMatchDone.Wait(lock);
+        }
         String rule = String("type='signal',interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',arg0='") + it->first.c_str() + "'";
         bus->RemoveMatchNonBlocking(rule.c_str());
     }
-    matchRuleRefcounts.clear();
+    matchRuleBookKeeping.clear();
     lock.Unlock(MUTEX_CONTEXT);
 }
 
@@ -1142,19 +1300,24 @@ bool ProxyBusObject::IsSecure() const {
 
 void ProxyBusObject::EnablePropertyCaching()
 {
+    vector<String> ifcNames;
     internal->lock.Lock(MUTEX_CONTEXT);
+    ifcNames.reserve(internal->ifaces.size());
     if (!internal->cacheProperties) {
         internal->cacheProperties = true;
         map<StringMapKey, const InterfaceDescription*>::const_iterator it = internal->ifaces.begin();
         for (; it != internal->ifaces.end(); ++it) {
             if (it->second->HasCacheableProperties()) {
                 internal->caches.insert(std::make_pair(qcc::String(it->first.c_str()), CachedProps(it->second)));
-                /* add match rules in case the PropertiesChanged signals are emitted as global broadcast */
-                internal->AddPropertiesChangedRule(it->first.c_str(), false);
+                ifcNames.push_back(it->first.c_str());
             }
         }
     }
     internal->lock.Unlock(MUTEX_CONTEXT);
+    for (vector<String>::const_iterator it = ifcNames.begin(); it != ifcNames.end(); ++it) {
+        /* add match rules in case the PropertiesChanged signals are emitted as global broadcast */
+        internal->AddPropertiesChangedRule(it->c_str(), false);
+    }
 }
 
 size_t ProxyBusObject::GetChildren(ProxyBusObject** children, size_t numChildren)
@@ -1428,13 +1591,13 @@ QStatus ProxyBusObject::MethodCallAsync(const char* ifaceName,
     return MethodCallAsync(*member, receiver, replyHandler, args, numArgs, context, timeout, flags);
 }
 
-
 QStatus ProxyBusObject::MethodCall(const InterfaceDescription::Member& method,
                                    const MsgArg* args,
                                    size_t numArgs,
                                    Message& replyMsg,
                                    uint32_t timeout,
-                                   uint8_t flags) const
+                                   uint8_t flags,
+                                   Message* callMsg) const
 {
     QStatus status;
     Message msg(*internal->bus);
@@ -1473,6 +1636,12 @@ QStatus ProxyBusObject::MethodCall(const InterfaceDescription::Member& method,
     if (status != ER_OK) {
         goto MethodCallExit;
     }
+    /*
+     * If caller asked for a copy of the sent message, copy it now that we've successfully created it.
+     */
+    if (NULL != callMsg) {
+        *callMsg = msg;
+    }
     if (flags & ALLJOYN_FLAG_NO_REPLY_EXPECTED) {
         /*
          * Push the message to the router and we are done
@@ -1579,6 +1748,9 @@ MethodCallExit:
     if ((status == ER_OK) && internal->uniqueName.empty()) {
         internal->uniqueName = replyMsg->GetSender();
     }
+    if (ER_OK != status) {
+        AdjustErrorForPermissionDenied(replyMsg, status);
+    }
     return status;
 }
 
@@ -1627,36 +1799,12 @@ void ProxyBusObject::SyncReplyHandler(Message& msg, void* context)
 
 QStatus ProxyBusObject::SecureConnection(bool forceAuth)
 {
-    if (!internal->bus->IsPeerSecurityEnabled()) {
-        return ER_BUS_SECURITY_NOT_ENABLED;
-    }
-    LocalEndpoint localEndpoint = internal->bus->GetInternal().GetLocalEndpoint();
-    if (!localEndpoint->IsValid()) {
-        return ER_BUS_ENDPOINT_CLOSING;
-    } else {
-        AllJoynPeerObj* peerObj = localEndpoint->GetPeerObj();
-        if (forceAuth) {
-            peerObj->ForceAuthentication(internal->serviceName);
-        }
-        return peerObj->AuthenticatePeer(MESSAGE_METHOD_CALL, internal->serviceName);
-    }
+    return internal->bus->SecureConnection(internal->serviceName.c_str(), forceAuth);
 }
 
 QStatus ProxyBusObject::SecureConnectionAsync(bool forceAuth)
 {
-    if (!internal->bus->IsPeerSecurityEnabled()) {
-        return ER_BUS_SECURITY_NOT_ENABLED;
-    }
-    LocalEndpoint localEndpoint = internal->bus->GetInternal().GetLocalEndpoint();
-    if (!localEndpoint->IsValid()) {
-        return ER_BUS_ENDPOINT_CLOSING;
-    } else {
-        AllJoynPeerObj* peerObj =  localEndpoint->GetPeerObj();
-        if (forceAuth) {
-            peerObj->ForceAuthentication(internal->serviceName);
-        }
-        return peerObj->AuthenticatePeerAsync(internal->serviceName);
-    }
+    return internal->bus->SecureConnectionAsync(internal->serviceName.c_str(), forceAuth);
 }
 
 const qcc::String& ProxyBusObject::GetPath(void) const {
@@ -1834,6 +1982,7 @@ ProxyBusObject::ProxyBusObject(const ProxyBusObject& other) : internal(other.int
 ProxyBusObject::ProxyBusObject(ManagedObj<ProxyBusObject::Internal> internal) : internal(internal), isExiting(false) {
 }
 
+
 bool ProxyBusObject::operator==(const ProxyBusObject& other) const {
     return internal == other.internal;
 }
@@ -1923,6 +2072,11 @@ void CachedProps::Set(const char* propname, const MsgArg& val, const uint32_t me
     }
 
     lock.Lock(MUTEX_CONTEXT);
+    if (!enabled) {
+        lock.Unlock(MUTEX_CONTEXT);
+        return;
+    }
+
     if (!IsValidMessageSerial(messageSerial)) {
         values.clear();
     } else {
@@ -1935,6 +2089,10 @@ void CachedProps::Set(const char* propname, const MsgArg& val, const uint32_t me
 void CachedProps::SetAll(const MsgArg& allValues, const uint32_t messageSerial)
 {
     lock.Lock(MUTEX_CONTEXT);
+    if (!enabled) {
+        lock.Unlock(MUTEX_CONTEXT);
+        return;
+    }
 
     size_t nelem;
     MsgArg* elems;
@@ -1977,6 +2135,10 @@ error:
 void CachedProps::PropertiesChanged(MsgArg* changed, size_t numChanged, MsgArg* invalidated, size_t numInvalidated, const uint32_t messageSerial)
 {
     lock.Lock(MUTEX_CONTEXT);
+    if (!enabled) {
+        lock.Unlock(MUTEX_CONTEXT);
+        return;
+    }
 
     QStatus status;
 
@@ -2020,4 +2182,16 @@ error:
     lock.Unlock(MUTEX_CONTEXT);
 }
 
+BusAttachment& ProxyBusObject::GetBusAttachment() const
+{
+    return *(internal->bus);
+}
+
+void CachedProps::Enable()
+{
+    lock.Lock(MUTEX_CONTEXT);
+    enabled = true;
+    lock.Unlock(MUTEX_CONTEXT);
+}
+
 }
