@@ -83,7 +83,6 @@ class _RemoteEndpoint::Internal {
         getNextMsg(true),
         currentWriteMsg(bus),
         stopping(false),
-        sessionId(0),
         pingCallSerial(0),
         sendTimeout(0),
         maxControlMessages(30),
@@ -111,7 +110,7 @@ class _RemoteEndpoint::Internal {
     Features features;                       /**< Requested and negotiated features of this endpoint */
     uint32_t processId;                      /**< Process id of the process at the remote end of this endpoint */
     uint32_t alljoynVersion;                 /**< AllJoyn version of the process at the remote end of this endpoint */
-    int32_t refCount;                        /**< Number of active users of this remote endpoint */
+    volatile int32_t refCount;               /**< Number of active users of this remote endpoint */
     bool isSocket;                           /**< True iff this endpoint contains a SockStream as its 'stream' member */
     bool armRxPause;                         /**< Pause Rx after receiving next METHOD_REPLY message */
 
@@ -132,7 +131,7 @@ class _RemoteEndpoint::Internal {
     bool getNextMsg;                         /**< If true, read the next message from the txQueue */
     Message currentWriteMsg;                 /**< The message currently being read for this endpoint */
     bool stopping;                           /**< Is this EP stopping? */
-    uint32_t sessionId;                      /**< SessionId for BusToBus endpoint. (not used for non-B2B endpoints) */
+    set<SessionId> sessionIdSet;                    /**< Set of session Ids that this endpoint is a part of */
     uint32_t pingCallSerial;                 /**< Serial number of last Heartbeat DBus ping sent */
     uint32_t sendTimeout;                    /**< Send timeout for this endpoint i.e. time after which the Routing node must
                                                   disconnect the remote node if it has not read a message from the link
@@ -455,12 +454,13 @@ QStatus _RemoteEndpoint::Start()
     if (status == ER_OK) {
         /* Register endpoint with router */
         status = router.RegisterEndpoint(bep);
-
-        if (status != ER_OK) {
-            /* Failed to register with iodispatch */
-            internal->bus.GetInternal().GetIODispatch().StopStream(internal->stream);
-            router.UnregisterEndpoint(this->GetUniqueName(), this->GetEndpointType());
-        }
+    } else {
+        /*
+         * Set started to false only if StartStream fails.
+         * If registering with the router or enabling read callback fails,
+         * we still need to Join the endpoint.
+         */
+        internal->started = false;
     }
 
     if (status == ER_OK) {
@@ -469,12 +469,10 @@ QStatus _RemoteEndpoint::Start()
         if (status != ER_OK) {
             /* Failed to start read with iodispatch */
             internal->bus.GetInternal().GetIODispatch().StopStream(internal->stream);
-            router.UnregisterEndpoint(this->GetUniqueName(), this->GetEndpointType());
         }
     }
     if (status != ER_OK) {
         Invalidate();
-        internal->started = false;
     }
 
     return status;
@@ -490,7 +488,6 @@ QStatus _RemoteEndpoint::Start(uint32_t idleTimeout, uint32_t probeTimeout, uint
     internal->maxControlMessages = sendTimeout * MAX_CONTROL_MSGS_PER_SECOND;
     if (status != ER_OK) {
         Invalidate();
-        internal->started = false;
     }
     return status;
 }
@@ -718,7 +715,7 @@ QStatus _RemoteEndpoint::ReadCallback(qcc::Source& source, bool isTimedOut)
                         /* This is a response to the DBus ping sent from RN to LN. Consume the reply quietly. */
                         internal->pingCallSerial = 0;
                     } else if (IsProbeMsg(msg, isAck)) {
-                        QCC_DbgPrintf(("%s: Received %s\n", GetUniqueName().c_str(), isAck ? "ProbeAck" : "ProbeReq"));
+                        QCC_DbgPrintf(("%s: Received %s", GetUniqueName().c_str(), isAck ? "ProbeAck" : "ProbeReq"));
                         if (!isAck) {
                             /* Respond to probe request */
                             Message probeMsg(internal->bus);
@@ -726,7 +723,7 @@ QStatus _RemoteEndpoint::ReadCallback(qcc::Source& source, bool isTimedOut)
                             if (status == ER_OK) {
                                 status = PushMessage(probeMsg);
                             }
-                            QCC_DbgPrintf(("%s: Sent ProbeAck (%s)\n", GetUniqueName().c_str(), QCC_StatusText(status)));
+                            QCC_DbgPrintf(("%s: Sent ProbeAck (%s)", GetUniqueName().c_str(), QCC_StatusText(status)));
                         }
                     } else {
                         BusEndpoint bep  = BusEndpoint::cast(rep);
@@ -806,7 +803,7 @@ QStatus _RemoteEndpoint::ReadCallback(qcc::Source& source, bool isTimedOut)
                 }
 
                 /* Check pause condition. Block until stopped */
-                if (internal->armRxPause && internal->started && (msg->GetType() == MESSAGE_METHOD_RET)) {
+                if (internal->armRxPause && !internal->stopping && (msg->GetType() == MESSAGE_METHOD_RET)) {
                     status = ER_BUS_ENDPOINT_CLOSING;
                     internal->bus.GetInternal().GetIODispatch().DisableReadCallback(internal->stream);
                     return ER_OK;
@@ -846,7 +843,7 @@ QStatus _RemoteEndpoint::ReadCallback(qcc::Source& source, bool isTimedOut)
                 if (status == ER_OK) {
                     PushMessage(probeMsg);
                 }
-                QCC_DbgPrintf(("%s: Sent ProbeReq (%s)\n", GetUniqueName().c_str(), QCC_StatusText(status)));
+                QCC_DbgPrintf(("%s: Sent ProbeReq (%s)", GetUniqueName().c_str(), QCC_StatusText(status)));
 
             } else {
                 Message msg(internal->bus);
@@ -863,7 +860,7 @@ QStatus _RemoteEndpoint::ReadCallback(qcc::Source& source, bool isTimedOut)
                 if (status == ER_OK) {
                     PushMessage(msg);
                 }
-                QCC_DbgPrintf(("%s: Sent DBus ping (%s)\n", GetUniqueName().c_str(), QCC_StatusText(status)));
+                QCC_DbgPrintf(("%s: Sent DBus ping (%s)", GetUniqueName().c_str(), QCC_StatusText(status)));
 
             }
             internal->lock.Lock(MUTEX_CONTEXT);
@@ -939,11 +936,14 @@ QStatus _RemoteEndpoint::WriteCallback(qcc::Sink& sink, bool isTimedOut)
         RemoteEndpoint rep = RemoteEndpoint::wrap(this);
         status = internal->currentWriteMsg->DeliverNonBlocking(rep);
         /* Report authorization failure as a security violation */
-        if (status == ER_BUS_NOT_AUTHORIZED) {
+        if ((status == ER_BUS_NOT_AUTHORIZED) || (status == ER_PERMISSION_DENIED)) {
             internal->bus.GetInternal().GetLocalEndpoint()->GetPeerObj()->HandleSecurityViolation(internal->currentWriteMsg, status);
             /*
              * Clear the error after reporting the security violation otherwise we will exit
              * this thread which will shut down the endpoint.
+             * In the case of ER_PERMISSION_DENIED, we can't deliver this
+             * message to the peer since the peer is not authorized to receive
+             * it.
              */
             status = ER_OK;
         }
@@ -1278,15 +1278,15 @@ QStatus _RemoteEndpoint::PushMessage(Message& msg)
 
 void _RemoteEndpoint::IncrementRef()
 {
-    int refs = IncrementAndFetch(&internal->refCount);
-    QCC_DbgPrintf(("_RemoteEndpoint::IncrementRef(%s) refs=%d\n", GetUniqueName().c_str(), refs));
+    int32_t refs = IncrementAndFetch(&internal->refCount);
+    QCC_DbgPrintf(("_RemoteEndpoint::IncrementRef(%s) refs=%d", GetUniqueName().c_str(), refs));
     QCC_UNUSED(refs); /* avoid unused variable warning in release build */
 }
 
 void _RemoteEndpoint::DecrementRef()
 {
-    int refs = DecrementAndFetch(&internal->refCount);
-    QCC_DbgPrintf(("_RemoteEndpoint::DecrementRef(%s) refs=%d\n", GetUniqueName().c_str(), refs));
+    int32_t refs = DecrementAndFetch(&internal->refCount);
+    QCC_DbgPrintf(("_RemoteEndpoint::DecrementRef(%s) refs=%d", GetUniqueName().c_str(), refs));
     if (refs <= 0) {
         if (minimalEndpoint && refs == 0) {
             Stop();
@@ -1322,24 +1322,47 @@ QStatus _RemoteEndpoint::GenProbeMsg(bool isAck, Message msg)
     return msg->SignalMsg("", NULL, 0, "/", org::alljoyn::Daemon::InterfaceName, isAck ? "ProbeAck" : "ProbeReq", NULL, 0, 0, 0);
 }
 
-void _RemoteEndpoint::SetSessionId(uint32_t sessionId) {
+void _RemoteEndpoint::RegisterSessionId(uint32_t sessionId)
+{
     if (internal) {
-        internal->sessionId = sessionId;
+        QCC_DbgPrintf(("_RemoteEndpoint::RegisterSessionId (%s,%u)", GetUniqueName().c_str(), sessionId));
+        internal->sessionIdSet.insert(sessionId);
+        assert(!internal->features.isBusToBus || (internal->sessionIdSet.size() == 1));
     }
 }
 
-uint32_t _RemoteEndpoint::GetSessionId() const {
+void _RemoteEndpoint::UnregisterSessionId(uint32_t sessionId)
+{
     if (internal) {
-        return internal->sessionId;
-    } else {
-        return 0;
+        QCC_DbgPrintf(("_RemoteEndpoint::UnregisterSessionId (%s,%u)", GetUniqueName().c_str(), sessionId));
+        internal->sessionIdSet.erase(sessionId);
     }
 }
 
+bool _RemoteEndpoint::IsInSession(SessionId sessionId)
+{
+    if (internal) {
+        return internal->sessionIdSet.find(sessionId) != internal->sessionIdSet.end();
+    }
+    return false;
+}
+
+uint32_t _RemoteEndpoint::GetSessionId() const
+{
+    if (internal) {
+        assert(internal->sessionIdSet.size() < 2);
+        if (internal->sessionIdSet.size() == 1) {
+            return *internal->sessionIdSet.begin();
+        }
+    }
+    return 0;
+
+}
+
 bool _RemoteEndpoint::IsSessionRouteSetUp()
 {
     if (internal) {
-        return (internal->sessionId != 0);
+        return (internal->sessionIdSet.size() != 0);
     } else {
         return false;
     }
