@@ -132,8 +132,8 @@ class MyBusListener : public BusListener, public SessionListener {
                        newOwner ? newOwner : "null");
     }
 
-    void SessionLost(SessionId sessionId, SessionLostReason reason) {
-        QCC_SyncPrintf("SessionLost(%u) was called. Reason = %u.\n", sessionId, reason);
+    void SessionLost(SessionId lostSessionId, SessionLostReason reason) {
+        QCC_SyncPrintf("SessionLost(%u) was called. Reason = %u.\n", lostSessionId, reason);
     }
 
     SessionId GetSessionId() const { return sessionId; }
@@ -168,7 +168,7 @@ class MyAboutListener : public AboutListener {
             /* We found a remote bus that is advertising bbservice's well-known name so connect to it */
             SessionOpts opts(SessionOpts::TRAFFIC_MESSAGES, false, SessionOpts::PROXIMITY_ANY, TRANSPORT_ANY);
 
-            QStatus status = g_msgBus->JoinSession(busName, ::org::alljoyn::alljoyn_test::SessionPort, &g_busListener, sessionId, opts);
+            QStatus status = g_msgBus->JoinSession(busName, port, &g_busListener, sessionId, opts);
             if (ER_OK == status) {
                 if (encryption) {
                     ProxyBusObject remotePeerObj(*g_msgBus, busName, "/", 0);
@@ -339,15 +339,15 @@ class LocalTestObject : public BusObject {
         if (args[0].v_uint32 == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
             /* Begin Advertising the well known name to remote busses */
             const ProxyBusObject& alljoynObj = bus->GetAllJoynProxyObj();
-            MsgArg args[2];
-            size_t numArgs = ArraySize(args);
-            MsgArg::Set(args, numArgs, "sq", g_advertiseName.c_str(), g_preferredTransport);
+            MsgArg newArgs[2];
+            size_t newNumArgs = ArraySize(newArgs);
+            MsgArg::Set(newArgs, newNumArgs, "sq", g_advertiseName.c_str(), g_preferredTransport);
             QStatus status = alljoynObj.MethodCallAsync(ajn::org::alljoyn::Bus::InterfaceName,
                                                         "AdvertiseName",
                                                         this,
                                                         static_cast<MessageReceiver::ReplyHandler>(&LocalTestObject::AdvertiseRequestCB),
-                                                        args,
-                                                        numArgs);
+                                                        newArgs,
+                                                        newNumArgs);
             if (ER_OK != status) {
                 QCC_LogError(status, ("Sending org.alljoyn.Bus.Advertise failed"));
             }
@@ -482,15 +482,16 @@ static void usage(void)
     printf("   -a <name>                   = Well-known name to advertise\n");
     printf("   -n <name>                   = Well-known name to find\n");
     printf("   -s                          = Enable stress mode (connect/disconnect w/ server between runs non-stop)\n");
-    printf("   -l                          = Register signal handler for loopback\n");
+    printf("   -rs                         = Register signal handler for loopback\n");
     printf("   -r #                        = Signal rate (delay in ms between signals sent; default = 0)\n");
     printf("   -y #                        = Delay (in ms) between sending last signal and disconnecting (stress mode only)\n");
     printf("   -i #                        = Signal report interval (number of signals tx/rx per update; default = 1000)\n");
     printf("   -c #                        = Max number of signals to send, default = 1000000)\n");
-    printf("   -t #                        = TTL for the signals\n");
-    printf("   --tcp                       = Advertise and discover using the TCP transport\n");
-    printf("   --udp                       = Advertise and discover using the UDP transport\n");
-    printf("   -e[k] [SRP|LOGON]       = Encrypt the test interface using specified auth mechanism, -ek means clear keys\n");
+    printf("   -ttl #                      = TTL for the signals\n");
+    printf("   -t                          = Advertise/Discover over TCP transport\n");
+    printf("   -u                          = Advertise/Discover over UDP transport\n");
+    printf("   -l                          = Advertise/Discover over Local transport\n");
+    printf("   -e[k] [SRP|LOGON]           = Encrypt the test interface using specified auth mechanism, -ek means clear keys\n");
     printf("   -d                          = discover remote bus with test service\n");
     printf("   -b                          = Signal is broadcast rather than multicast\n");
     printf("   --ls                        = Call LeaveSession before tearing down the Bus Attachment\n");
@@ -542,11 +543,14 @@ int CDECL_CALL main(int argc, char** argv)
             exit(0);
         } else if (0 == strcmp("-s", argv[i])) {
             doStress = true;
-        } else if (0 == strcmp("--tcp", argv[i])) {
+        } else if (0 == strcmp("-t", argv[i])) {
+            g_preferredTransport = TRANSPORT_TCP;
+            transportSpecific = true;
+        } else if (0 == strcmp("-u", argv[i])) {
             g_preferredTransport = TRANSPORT_UDP;
             transportSpecific = true;
-        } else if (0 == strcmp("--udp", argv[i])) {
-            g_preferredTransport = TRANSPORT_UDP;
+        } else if (0 == strcmp("-l", argv[i])) {
+            g_preferredTransport = TRANSPORT_LOCAL;
             transportSpecific = true;
         } else if (0 == strcmp("-n", argv[i])) {
             ++i;
@@ -568,7 +572,7 @@ int CDECL_CALL main(int argc, char** argv)
                 g_advertiseName = argv[i];
             }
 
-        } else if (0 == strcmp("-l", argv[i])) {
+        } else if (0 == strcmp("-rs", argv[i])) {
             useSignalHandler = true;
         } else if (0 == strcmp("-d", argv[i])) {
             discoverRemote = true;
@@ -610,7 +614,7 @@ int CDECL_CALL main(int argc, char** argv)
             } else {
                 maxSignals = strtoul(argv[i], NULL, 10);
             }
-        } else if (0 == strcmp("-t", argv[i])) {
+        } else if (0 == strcmp("-ttl", argv[i])) {
             ++i;
             if (i == argc) {
                 printf("option %s requires a parameter\n", argv[i - 1]);
@@ -724,7 +728,7 @@ int CDECL_CALL main(int argc, char** argv)
         if (g_selfjoin) {
             SessionOpts opts(SessionOpts::TRAFFIC_MESSAGES, true, SessionOpts::PROXIMITY_ANY, TRANSPORT_ANY);
             SessionPort sessionPort = ::org::alljoyn::alljoyn_test::SessionPort;
-            QStatus status = g_msgBus->BindSessionPort(sessionPort, opts, g_portListener);
+            status = g_msgBus->BindSessionPort(sessionPort, opts, g_portListener);
             if (ER_OK != status) {
                 QCC_LogError(status, ("Could not bind to session"));
                 break;
@@ -741,15 +745,9 @@ int CDECL_CALL main(int argc, char** argv)
 
             /* Begin discovery on the well-known name of the service to be called */
             Message reply(*g_msgBus);
-            const ProxyBusObject& alljoynObj = g_msgBus->GetAllJoynProxyObj();
 
-            MsgArg serviceName("s", g_wellKnownName.c_str());
-            status = alljoynObj.MethodCall(::ajn::org::alljoyn::Bus::InterfaceName,
-                                           "FindAdvertisedName",
-                                           &serviceName,
-                                           1,
-                                           reply,
-                                           5000);
+            status = g_msgBus->FindAdvertisedNameByTransport(g_wellKnownName.c_str(), g_preferredTransport);
+
             if (ER_OK != status) {
                 QCC_LogError(status, ("%s.FindAdvertisedName failed", ::ajn::org::alljoyn::Bus::InterfaceName));
             }
@@ -815,7 +813,7 @@ int CDECL_CALL main(int argc, char** argv)
                 if (((i + 1) % testObj->reportInterval) == 0) {
                     QCC_SyncPrintf("SendSignal: %u\n", i + 1);
                 }
-                QStatus status = testObj->SendSignal();
+                status = testObj->SendSignal();
                 if (status != ER_OK) {
                     QCC_LogError(status, ("Failed to send signal"));
                     break;
