@@ -54,6 +54,10 @@
 #include <qcc/Thread.h>
 #include <qcc/Util.h>
 
+#if defined(QCC_OS_GROUP_WINDOWS)
+#include <qcc/windows/NamedPipeWrapper.h>
+#endif
+
 #include <alljoyn/BusAttachment.h>
 #include <alljoyn/BusListener.h>
 #include <alljoyn/BusObject.h>
@@ -68,7 +72,7 @@ using namespace qcc;
 
 static volatile sig_atomic_t quit;
 static const SessionPort PORT = 123;
-static const SessionOpts SESSION_OPTS(SessionOpts::TRAFFIC_MESSAGES, false, SessionOpts::PROXIMITY_ANY, TRANSPORT_ANY);
+static SessionOpts SESSION_OPTS(SessionOpts::TRAFFIC_MESSAGES, false, SessionOpts::PROXIMITY_ANY, TRANSPORT_ANY);
 
 static const char* propStressTestInterfaceXML =
     "<node name=\"/org/alljoyn/Testing/PropertyStressTest\">"
@@ -128,11 +132,11 @@ PropTesterObject::~PropTesterObject()
 {
 }
 
-QStatus PropTesterObject::Set(int32_t int32Prop, uint32_t uint32Prop, const char* stringProp)
+QStatus PropTesterObject::Set(int32_t int32Property, uint32_t uint32Property, const char* stringProperty)
 {
-    this->int32Prop = int32Prop;
-    this->uint32Prop = uint32Prop;
-    this->stringProp = stringProp;
+    this->int32Prop = int32Property;
+    this->uint32Prop = uint32Property;
+    this->stringProp = stringProperty;
     QCC_SyncPrintf("Emits properties changed for: \"%s\"\n", this->GetPath());
     return EmitPropChanged(interfaceName, props, ArraySize(props), id, 0);
 }
@@ -508,7 +512,10 @@ void Usage()
            "    -c            Run as client (runs as service by default).\n"
            "    -n <NAME>     Use <NAME> for well known bus name.\n"
            "    -s <SEC>      Run for <SEC> seconds.\n"
-           "    -o <NBR>      Create <NBR> objects.\n");
+           "    -o <NBR>      Create <NBR> objects.\n"
+           "    -t            Advertise/Discover over TCP (enables selective advertising)\n"
+           "    -l            Advertise/Discover locally (enables selective advertising)\n"
+           "    -u            Advertise/Discover over UDP-based ARDP (enables selective advertising)\n");
 }
 
 int CDECL_CALL main(int argc, char** argv)
@@ -561,13 +568,18 @@ int CDECL_CALL main(int argc, char** argv)
         } else if (strcmp(argv[i], "-h") == 0) {
             Usage();
             exit(1);
+        } else if (0 == strcmp("-t", argv[i])) {
+            SESSION_OPTS.transports = TRANSPORT_TCP;
+        } else if (0 == strcmp("-l", argv[i])) {
+            SESSION_OPTS.transports = TRANSPORT_LOCAL;
+        } else if (0 == strcmp("-u", argv[i])) {
+            SESSION_OPTS.transports = TRANSPORT_UDP;
         } else {
             Usage();
             exit(1);
         }
     }
 
-
     signal(SIGINT, SignalHandler);
     signal(SIGTERM, SignalHandler);
 
@@ -581,11 +593,11 @@ int CDECL_CALL main(int argc, char** argv)
 
     if (connSpec.empty()) {
 #if defined(QCC_OS_GROUP_WINDOWS)
-    #if (_WIN32_WINNT > 0x0603)
-        connSpec = env->Find("BUS_ADDRESS", "npipe:");
-    #else
-        connSpec = env->Find("BUS_ADDRESS", "tcp:addr=127.0.0.1,port=9955");
-    #endif
+        if (qcc::NamedPipeWrapper::AreApisAvailable()) {
+            connSpec = env->Find("BUS_ADDRESS", "npipe:");
+        } else {
+            connSpec = env->Find("BUS_ADDRESS", "tcp:addr=127.0.0.1,port=9955");
+        }
 #else
         connSpec = env->Find("BUS_ADDRESS", "unix:abstract=alljoyn");
 #endif
@@ -607,7 +619,7 @@ int CDECL_CALL main(int argc, char** argv)
 
     if (client) {
         app = new Client(*bus, nbrOfObjects);
-        status = bus->FindAdvertisedName(serviceName.c_str());
+        status = bus->FindAdvertisedNameByTransport(serviceName.c_str(), SESSION_OPTS.transports);
         if (status != ER_OK) {
             printf("Failed to find name to \"%s\": %s\n", serviceName.c_str(), QCC_StatusText(status));
             ret = 2;
@@ -635,7 +647,7 @@ int CDECL_CALL main(int argc, char** argv)
 
 exit:
     if (client) {
-        bus->CancelFindAdvertisedName(serviceName.c_str());
+        bus->CancelFindAdvertisedNameByTransport(serviceName.c_str(), SESSION_OPTS.transports);
         bus->Disconnect();
     } else {
         bus->CancelAdvertiseName(serviceName.c_str(), TRANSPORT_ANY);
