@@ -53,6 +53,8 @@ static const char* ifcId = "ifc";
 static const char* ifcDescription[] = { "This is the interface", "<bold>DE:</bold> This is the interface" };
 static const char* propId = "prop";
 static const char* namePropDescription[] = { "This is the actual name", "DE: This is the actual name" };
+static const char* methId = "method";
+static const char* pingMethodDescription[] = { "This is the ping description", "DE: This is the ping description" };
 
 static const char* IntrospectWithDescriptionString1[] = {
     "<!DOCTYPE node PUBLIC \"-//allseen//DTD ALLJOYN Object Introspection 1.0//EN\"\n"
@@ -120,6 +122,11 @@ static const char* IntrospectWithDescriptionString2[] = {
     "  <node name=\"org\"/>\n"
     "  <interface name=\"org.alljoyn.Bus.DescriptionInterface\">\n"
     "    <description>This is the interface</description>\n"
+    "    <method name=\"Ping\">\n"
+    "      <description>This is the ping description</description>\n"
+    "      <arg name=\"inStr\" type=\"s\" direction=\"in\"/>\n"
+    "      <arg name=\"outStr\" type=\"s\" direction=\"out\"/>\n"
+    "    </method>\n"
     "    <property name=\"name\" type=\"s\" access=\"readwrite\">\n"
     "      <description>This is the actual name</description>\n"
     "    </property>\n"
@@ -148,6 +155,11 @@ static const char* IntrospectWithDescriptionString2[] = {
     "  <node name=\"org\"/>\n"
     "  <interface name=\"org.alljoyn.Bus.DescriptionInterface\">\n"
     "    <description>&lt;bold&gt;DE:&lt;/bold&gt; This is the interface</description>\n"
+    "    <method name=\"Ping\">\n"
+    "      <description>DE: This is the ping description</description>\n"
+    "      <arg name=\"inStr\" type=\"s\" direction=\"in\"/>\n"
+    "      <arg name=\"outStr\" type=\"s\" direction=\"out\"/>\n"
+    "    </method>\n"
     "    <property name=\"name\" type=\"s\" access=\"readwrite\">\n"
     "      <description>DE: This is the actual name</description>\n"
     "    </property>\n"
@@ -223,6 +235,54 @@ static const char* IntrospectWithDescriptionString3[] = {
     "</node>\n"
 };
 
+static const char* IntrospectWithDescriptionString4 =
+    "<!DOCTYPE node PUBLIC \"-//allseen//DTD ALLJOYN Object Introspection 1.0//EN\"\n"
+    "\"http://www.allseen.org/alljoyn/introspect-1.0.dtd\">\n"
+    "<node>\n"
+    "  <description>This is the object</description>\n"
+    "  <node name=\"org\"/>\n"
+    "  <interface name=\"org.alljoyn.Bus.DescriptionInterface\">\n"
+    "    <signal name=\"globalBroadcastSignal\" sessionless=\"false\" globalbroadcast=\"true\">\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"legacyNonSessionlessSignal\" sessionless=\"false\">\n"
+    "      <description>legacy non-sessionless signal</description>\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"legacySessionlessSignal\" sessionless=\"true\">\n"
+    "      <description>legacy sessionless signal</description>\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"legacySignal\" sessionless=\"false\">\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"sessioncastSignal\" sessioncast=\"true\" sessionless=\"false\">\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"sessionlessSignal\" sessionless=\"true\">\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "    <signal name=\"unicastSignal\" sessionless=\"false\" unicast=\"true\">\n"
+    "      <arg type=\"s\" direction=\"out\"/>\n"
+    "    </signal>\n"
+    "  </interface>\n"
+    "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n"
+    "    <method name=\"Introspect\">\n"
+    "      <arg name=\"data\" type=\"s\" direction=\"out\"/>\n"
+    "    </method>\n"
+    "  </interface>\n"
+    "  <interface name=\"org.allseen.Introspectable\">\n"
+    "    <method name=\"GetDescriptionLanguages\">\n"
+    "      <arg name=\"languageTags\" type=\"as\" direction=\"out\"/>\n"
+    "    </method>\n"
+    "    <method name=\"IntrospectWithDescription\">\n"
+    "      <arg name=\"languageTag\" type=\"s\" direction=\"in\"/>\n"
+    "      <arg name=\"data\" type=\"s\" direction=\"out\"/>\n"
+    "    </method>\n"
+    "    <annotation name=\"org.alljoyn.Bus.Secure\" value=\"off\"/>\n"
+    "  </interface>\n"
+    "</node>\n";
+
 class MyTranslator : public Translator {
   public:
 
@@ -241,9 +301,9 @@ class MyTranslator : public Translator {
         const char* tag = (*targetLanguage == '\0') ? "en" : targetLanguage;
         size_t i = 0;
 
-        if (0 == strcmp(tag, "en")) {
+        if (0 == strcasecmp(tag, "en")) {
             i = 0;
-        } else if (0 == strcmp(tag, "de")) {
+        } else if (0 == strcasecmp(tag, "de")) {
             i = 1;
         } else {
             return NULL;
@@ -258,6 +318,9 @@ class MyTranslator : public Translator {
         if (0 == strcmp(source, propId)) {
             return namePropDescription[i];
         }
+        if (0 == strcmp(source, methId)) {
+            return pingMethodDescription[i];
+        }
         return NULL;
     }
 
@@ -380,7 +443,9 @@ class DescriptionTest : public::testing::Test {
         Message replyMsg(*s_msgBusClient);
         EXPECT_EQ(ER_OK, remoteObj->MethodCall("org.allseen.Introspectable", "IntrospectWithDescription", &msgArg, 1, replyMsg));
         char* str = NULL;
-        EXPECT_EQ(ER_OK, replyMsg->GetArg(0)->Get("s", &str));
+        const MsgArg* replyArg = replyMsg->GetArg(0);
+        ASSERT_TRUE(replyArg != NULL);
+        EXPECT_EQ(ER_OK, replyArg->Get("s", &str));
         ASSERT_STREQ(IntrospectWithDescriptionStr, str);
     }
 };
@@ -409,6 +474,68 @@ TEST_F(DescriptionTest, IntrospectableDescriptionObject) {
 
     DescriptionLanguages(m_remoteObj);
     IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString1[0]);
+    IntrospectWithDescription(m_remoteObj, "en-US", IntrospectWithDescriptionString1[0]);
+    IntrospectWithDescription(m_remoteObj, "de", IntrospectWithDescriptionString1[1]);
+}
+
+TEST_F(DescriptionTest, IntrospectableDescriptionObjectGlobalTranslator)
+{
+    // Service part
+    InterfaceDescription* intf = NULL;
+    MyTranslator translator;
+    EXPECT_EQ(ER_OK, s_msgBusServer->CreateInterface(INTERFACE_NAME, intf));
+    ASSERT_TRUE(intf != NULL);
+
+    intf->AddProperty("name", "s", PROP_ACCESS_RW);
+    intf->SetDescriptionLanguage("");
+    intf->SetDescription(ifcId);
+    intf->SetPropertyDescription("name", propId);
+    intf->Activate();
+
+    ASSERT_TRUE((m_testObj = new DescriptionObject(*intf, SERVICE_PATH)) != NULL);
+    s_msgBusServer->SetDescriptionTranslator(&translator);
+    ASSERT_EQ(ER_OK, s_msgBusServer->RegisterBusObject(*m_testObj));
+    ASSERT_EQ(ER_OK, s_msgBusServer->Connect());
+
+    ASSERT_TRUE((m_remoteObj = new ProxyBusObject(*s_msgBusClient, s_msgBusServer->GetUniqueName().c_str(), SERVICE_PATH, 0)) != NULL);
+    EXPECT_EQ(ER_OK, m_remoteObj->IntrospectRemoteObject());
+
+    IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString1[0]);
+    IntrospectWithDescription(m_remoteObj, "en-US", IntrospectWithDescriptionString1[0]);
+    IntrospectWithDescription(m_remoteObj, "de", IntrospectWithDescriptionString1[1]);
+}
+
+TEST_F(DescriptionTest, IntrospectableDescriptionObjectDefaultTranslator)
+{
+    // Service part
+    InterfaceDescription* intf = NULL;
+    EXPECT_EQ(ER_OK, s_msgBusServer->CreateInterface(INTERFACE_NAME, intf));
+    ASSERT_TRUE(intf != NULL);
+
+    intf->AddProperty("name", "s", PROP_ACCESS_RW);
+    intf->SetDescriptionLanguage("");
+    intf->SetDescription(ifcId);
+    intf->SetPropertyDescription("name", propId);
+
+    Translator* translator = intf->GetDescriptionTranslator();
+    for (int i = 0; i < 2; i++) {
+        EXPECT_EQ(ER_OK, translator->AddStringTranslation(objId, objDescription[i], tags[i]));
+        EXPECT_EQ(ER_OK, translator->AddStringTranslation(ifcId, ifcDescription[i], tags[i]));
+        EXPECT_EQ(ER_OK, translator->AddStringTranslation(propId, namePropDescription[i], tags[i]));
+    }
+    intf->Activate();
+
+    ASSERT_TRUE((m_testObj = new DescriptionObject(*intf, SERVICE_PATH)) != NULL);
+    m_testObj->SetDescriptionTranslator(translator);
+    ASSERT_EQ(ER_OK, s_msgBusServer->RegisterBusObject(*m_testObj));
+    ASSERT_EQ(ER_OK, s_msgBusServer->Connect());
+
+    ASSERT_TRUE((m_remoteObj = new ProxyBusObject(*s_msgBusClient, s_msgBusServer->GetUniqueName().c_str(), SERVICE_PATH, 0)) != NULL);
+    EXPECT_EQ(ER_OK, m_remoteObj->IntrospectRemoteObject());
+
+    DescriptionLanguages(m_remoteObj);
+    IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString1[0]);
+    IntrospectWithDescription(m_remoteObj, "en-US", IntrospectWithDescriptionString1[0]);
     IntrospectWithDescription(m_remoteObj, "de", IntrospectWithDescriptionString1[1]);
 }
 
@@ -419,10 +546,15 @@ TEST_F(DescriptionTest, IntrospectableDescriptionObjectNoTranslate) {
     EXPECT_EQ(ER_OK, s_msgBusServer->CreateInterface(INTERFACE_NAME, intf));
     ASSERT_TRUE(intf != NULL);
 
-    intf->AddProperty("name", "s", PROP_ACCESS_RW);
     intf->SetDescriptionLanguage("");
     intf->SetDescription(ifcId);
+
+    intf->AddProperty("name", "s", PROP_ACCESS_RW);
     intf->SetPropertyDescription("name", propId);
+
+    intf->AddMethod("Ping", "s", "s", "inStr,outStr", 0);
+    intf->SetMemberDescription("Ping", methId);
+
     intf->SetDescriptionTranslator(&translator);
     intf->Activate();
 
@@ -436,6 +568,7 @@ TEST_F(DescriptionTest, IntrospectableDescriptionObjectNoTranslate) {
 
     DescriptionLanguages(m_remoteObj);
     IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString2[0]);
+    IntrospectWithDescription(m_remoteObj, "en-US", IntrospectWithDescriptionString2[0]);
     IntrospectWithDescription(m_remoteObj, "de", IntrospectWithDescriptionString2[1]);
 }
 
@@ -460,5 +593,55 @@ TEST_F(DescriptionTest, IntrospectableDescriptionObjectNoIntfTranslate) {
     EXPECT_EQ(ER_OK, m_remoteObj->IntrospectRemoteObject());
 
     IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString3[0]);
+    IntrospectWithDescription(m_remoteObj, "en-US", IntrospectWithDescriptionString3[0]);
     IntrospectWithDescription(m_remoteObj, "de", IntrospectWithDescriptionString3[1]);
 }
+
+TEST_F(DescriptionTest, SignalTypes)
+{
+    InterfaceDescription* intf = NULL;
+    EXPECT_EQ(ER_OK, s_msgBusServer->CreateInterface(INTERFACE_NAME, intf));
+    ASSERT_TRUE(intf != NULL);
+
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+#if defined(QCC_OS_GROUP_WINDOWS)
+#pragma warning(push)
+#pragma warning(disable: 4996)
+#endif
+    ASSERT_EQ(ER_OK, intf->AddSignal("legacySignal", "s", NULL));
+
+    intf->SetDescriptionLanguage("en");
+    ASSERT_EQ(ER_OK, intf->AddSignal("legacyNonSessionlessSignal", "s", NULL));
+    ASSERT_EQ(ER_OK, intf->SetMemberDescription("legacyNonSessionlessSignal", "legacy non-sessionless signal", false));
+
+    ASSERT_EQ(ER_OK, intf->AddSignal("legacySessionlessSignal", "s", NULL));
+    ASSERT_EQ(ER_OK, intf->SetMemberDescription("legacySessionlessSignal", "legacy sessionless signal", true));
+#if defined(QCC_OS_GROUP_WINDOWS)
+#pragma warning(pop)
+#endif
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+
+    ASSERT_EQ(ER_OK, intf->AddSignal("sessioncastSignal", "s", NULL, MEMBER_ANNOTATE_SESSIONCAST));
+    ASSERT_EQ(ER_OK, intf->AddSignal("sessionlessSignal", "s", NULL, MEMBER_ANNOTATE_SESSIONLESS));
+    ASSERT_EQ(ER_OK, intf->AddSignal("unicastSignal", "s", NULL, MEMBER_ANNOTATE_UNICAST));
+    ASSERT_EQ(ER_OK, intf->AddSignal("globalBroadcastSignal", "s", NULL, MEMBER_ANNOTATE_GLOBAL_BROADCAST));
+
+    MyTranslator translator;
+    intf->SetDescriptionTranslator(&translator);
+    intf->Activate();
+
+    ASSERT_TRUE((m_testObj = new DescriptionObject(*intf, SERVICE_PATH)) != NULL);
+    m_testObj->SetDescriptionTranslator(&translator);
+    ASSERT_EQ(ER_OK, s_msgBusServer->RegisterBusObject(*m_testObj));
+    ASSERT_EQ(ER_OK, s_msgBusServer->Connect());
+
+    ASSERT_TRUE((m_remoteObj = new ProxyBusObject(*s_msgBusClient, s_msgBusServer->GetUniqueName().c_str(), SERVICE_PATH, 0)) != NULL);
+    EXPECT_EQ(ER_OK, m_remoteObj->IntrospectRemoteObject());
+
+    IntrospectWithDescription(m_remoteObj, "en", IntrospectWithDescriptionString4);
+}
