@@ -1002,12 +1002,8 @@ public class BusAttachment {
     /** Specify if the attachment is connected */
     private boolean isConnected;
 
-    private Method foundAdvertisedName;
-
     private ExecutorService executor;
 
-    private Method lostAdvertisedName;
-
     private DBusProxyObj dbus;
     private ProxyBusObject dbusbo;
 
@@ -1035,16 +1031,6 @@ public class BusAttachment {
         isConnected = false;
         this.allowRemoteMessages = (policy == RemoteMessage.Receive);
         busAuthListener = new AuthListenerInternal();
-        try {
-            foundAdvertisedName = getClass().getDeclaredMethod(
-                    "foundAdvertisedName", String.class, Short.class, String.class);
-            foundAdvertisedName.setAccessible(true);
-            lostAdvertisedName = getClass().getDeclaredMethod(
-                    "lostAdvertisedName", String.class, Short.class, String.class);
-            lostAdvertisedName.setAccessible(true);
-        } catch (NoSuchMethodException ex) {
-            /* This will not happen */
-        }
         create(applicationName, allowRemoteMessages, concurrency);
 
         /*
