@@ -156,17 +156,17 @@ const NSInteger kBusObjectTestsServicePort = 999;
     [client.bus registerBusListener:client];
     
     QStatus status = [self.bus start];
-    STAssertTrue(status == ER_OK, @"Bus failed to start.");    
+    XCTAssertTrue(status == ER_OK, @"Bus failed to start.");    
     status = [client.bus start];
-    STAssertTrue(status == ER_OK, @"Bus for client failed to start.");        
+    XCTAssertTrue(status == ER_OK, @"Bus for client failed to start.");        
     
     status = [self.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
     status = [client.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
     
     status = [self.bus requestWellKnownName:kBusObjectTestsAdvertisedName withFlags:kAJNBusNameFlagDoNotQueue|kAJNBusNameFlagReplaceExisting];
-    STAssertTrue(status == ER_OK, @"Request for well known name failed.");
+    XCTAssertTrue(status == ER_OK, @"Request for well known name failed.");
     
     basicObject = [[BasicObject alloc] initWithBusAttachment:self.bus onPath:kBusObjectTestsObjectPath];
     
@@ -175,45 +175,45 @@ const NSInteger kBusObjectTestsServicePort = 999;
     AJNSessionOptions *sessionOptions = [[AJNSessionOptions alloc] initWithTrafficType:kAJNTrafficMessages supportsMultipoint:YES proximity:kAJNProximityAny transportMask:kAJNTransportMaskAny];
     
     status = [self.bus bindSessionOnPort:kBusObjectTestsServicePort withOptions:sessionOptions withDelegate:self];
-    STAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
+    XCTAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
     
     status = [self.bus advertiseName:kBusObjectTestsAdvertisedName withTransportMask:kAJNTransportMaskAny];
-    STAssertTrue(status == ER_OK, @"Advertise name failed.");
+    XCTAssertTrue(status == ER_OK, @"Advertise name failed.");
     
     status = [client.bus findAdvertisedName:kBusObjectTestsAdvertisedName];
-    STAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
+    XCTAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
     
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried for acceptance of the client joiner.");
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client joined the session.");
-    STAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
-    STAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried for acceptance of the client joiner.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client joined the session.");
+    XCTAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
+    XCTAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
     
     BasicObjectProxy *proxy = [[BasicObjectProxy alloc] initWithBusAttachment:client.bus serviceName:kBusObjectTestsAdvertisedName objectPath:kBusObjectTestsObjectPath sessionId:self.testSessionId];
     
     [proxy introspectRemoteObject];
     
     NSString *resultantString = [proxy concatenateString:@"Hello " withString:@"World!"];
-    STAssertTrue([resultantString compare:@"Hello World!"] == NSOrderedSame, @"Test client call to method via proxy object failed.");
+    XCTAssertTrue([resultantString compare:@"Hello World!"] == NSOrderedSame, @"Test client call to method via proxy object failed.");
     
     status = [client.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
     status = [self.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
     
     status = [client.bus stop];
-    STAssertTrue(status == ER_OK, @"Client bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Client bus failed to stop.");
     status = [self.bus stop];
-    STAssertTrue(status == ER_OK, @"Bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Bus failed to stop.");
     
-    STAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");    
-    STAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");    
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");    
+    XCTAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");    
+    XCTAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");    
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");    
     
     proxy = nil;
     
     [client.bus unregisterBusListener:client];
     [self.bus unregisterBusListener:self];
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
     
     [client tearDown];    
 }
@@ -231,17 +231,17 @@ const NSInteger kBusObjectTestsServicePort = 999;
     [client.bus registerBusListener:client];
 
     QStatus status = [self.bus start];
-    STAssertTrue(status == ER_OK, @"Bus failed to start.");
+    XCTAssertTrue(status == ER_OK, @"Bus failed to start.");
     status = [client.bus start];
-    STAssertTrue(status == ER_OK, @"Bus for client failed to start.");
+    XCTAssertTrue(status == ER_OK, @"Bus for client failed to start.");
 
     status = [self.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
     status = [client.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
 
     status = [self.bus requestWellKnownName:kBusObjectTestsAdvertisedName withFlags:kAJNBusNameFlagDoNotQueue|kAJNBusNameFlagReplaceExisting];
-    STAssertTrue(status == ER_OK, @"Request for well known name failed.");
+    XCTAssertTrue(status == ER_OK, @"Request for well known name failed.");
 
     basicObject = [[BasicObject alloc] initWithBusAttachment:self.bus onPath:kBusObjectTestsObjectPath];
 
@@ -250,18 +250,18 @@ const NSInteger kBusObjectTestsServicePort = 999;
     AJNSessionOptions *sessionOptions = [[AJNSessionOptions alloc] initWithTrafficType:kAJNTrafficMessages supportsMultipoint:YES proximity:kAJNProximityAny transportMask:kAJNTransportMaskAny];
 
     status = [self.bus bindSessionOnPort:kBusObjectTestsServicePort withOptions:sessionOptions withDelegate:self];
-    STAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
+    XCTAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
 
     status = [self.bus advertiseName:kBusObjectTestsAdvertisedName withTransportMask:kAJNTransportMaskAny];
-    STAssertTrue(status == ER_OK, @"Advertise name failed.");
+    XCTAssertTrue(status == ER_OK, @"Advertise name failed.");
 
     status = [client.bus findAdvertisedName:kBusObjectTestsAdvertisedName];
-    STAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
+    XCTAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
 
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried for acceptance of the client joiner.");
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client joined the session.");
-    STAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
-    STAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried for acceptance of the client joiner.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client joined the session.");
+    XCTAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
+    XCTAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
 
     BasicObjectProxy *proxy = [[BasicObjectProxy alloc] initWithBusAttachment:client.bus serviceName:kBusObjectTestsAdvertisedName objectPath:kBusObjectTestsObjectPath sessionId:self.testSessionId];
 
@@ -276,34 +276,34 @@ const NSInteger kBusObjectTestsServicePort = 999;
 
     servicePropertyValue = basicObject.testStringProperty;
 
-    STAssertTrue([proxyPropertyValue compare:@"Hello World!!!"] == NSOrderedSame, @"The value of the property in the client-side object does not match what it was just set to. Actual value=[%@]",proxyPropertyValue);
-    STAssertTrue([servicePropertyValue compare:@"Hello World!!!"] == NSOrderedSame, @"The value of the property in the service-side object does not match what it was just set to. Actual value=[%@]",servicePropertyValue);
+    XCTAssertTrue([proxyPropertyValue compare:@"Hello World!!!"] == NSOrderedSame, @"The value of the property in the client-side object does not match what it was just set to. Actual value=[%@]",proxyPropertyValue);
+    XCTAssertTrue([servicePropertyValue compare:@"Hello World!!!"] == NSOrderedSame, @"The value of the property in the service-side object does not match what it was just set to. Actual value=[%@]",servicePropertyValue);
 
     basicObject.testStringProperty = @"Foo bar???";
     proxyPropertyValue = proxy.testStringProperty;
     servicePropertyValue = basicObject.testStringProperty;
-    STAssertTrue([proxyPropertyValue compare:@"Foo bar???"] == NSOrderedSame, @"The value of the property in the client-side object does not match what it was just set to. Actual value=[%@]",proxyPropertyValue);
-    STAssertTrue([servicePropertyValue compare:@"Foo bar???"] == NSOrderedSame, @"The value of the property in the service-side object does not match what it was just set to. Actual value=[%@]",servicePropertyValue);
+    XCTAssertTrue([proxyPropertyValue compare:@"Foo bar???"] == NSOrderedSame, @"The value of the property in the client-side object does not match what it was just set to. Actual value=[%@]",proxyPropertyValue);
+    XCTAssertTrue([servicePropertyValue compare:@"Foo bar???"] == NSOrderedSame, @"The value of the property in the service-side object does not match what it was just set to. Actual value=[%@]",servicePropertyValue);
 
     status = [client.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
     status = [self.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
 
     status = [client.bus stop];
-    STAssertTrue(status == ER_OK, @"Client bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Client bus failed to stop.");
     status = [self.bus stop];
-    STAssertTrue(status == ER_OK, @"Bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Bus failed to stop.");
 
-    STAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");
-    STAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");
+    XCTAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");
+    XCTAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");
 
     proxy = nil;
 
     [client.bus unregisterBusListener:client];
     [self.bus unregisterBusListener:self];
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
 
     [client tearDown];
 }
@@ -328,76 +328,76 @@ const NSInteger kBusObjectTestsServicePort = 999;
     [client.bus registerBusListener:client];
     
     QStatus status = [self.bus start];
-    STAssertTrue(status == ER_OK, @"Bus failed to start.");    
+    XCTAssertTrue(status == ER_OK, @"Bus failed to start.");    
     status = [client.bus start];
-    STAssertTrue(status == ER_OK, @"Bus for client failed to start.");        
+    XCTAssertTrue(status == ER_OK, @"Bus for client failed to start.");        
     
     status = [self.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Connection to bus via null transport failed.");
     status = [client.bus connectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client connection to bus via null transport failed.");
     
     status = [self.bus requestWellKnownName:kBusObjectTestsAdvertisedName withFlags:kAJNBusNameFlagDoNotQueue|kAJNBusNameFlagReplaceExisting];
-    STAssertTrue(status == ER_OK, @"Request for well known name failed.");
+    XCTAssertTrue(status == ER_OK, @"Request for well known name failed.");
     
     
     AJNSessionOptions *sessionOptions = [[AJNSessionOptions alloc] initWithTrafficType:kAJNTrafficMessages supportsMultipoint:YES proximity:kAJNProximityAny transportMask:kAJNTransportMaskAny];
     
     status = [self.bus bindSessionOnPort:kBusObjectTestsServicePort withOptions:sessionOptions withDelegate:self];
-    STAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
+    XCTAssertTrue(status == ER_OK, @"Bind session on port %u failed.", kBusObjectTestsServicePort);
     
     status = [self.bus advertiseName:kBusObjectTestsAdvertisedName withTransportMask:kAJNTransportMaskAny];
-    STAssertTrue(status == ER_OK, @"Advertise name failed.");
+    XCTAssertTrue(status == ER_OK, @"Advertise name failed.");
     
     status = [client.bus findAdvertisedName:kBusObjectTestsAdvertisedName];
-    STAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
+    XCTAssertTrue(status == ER_OK, @"Client attempt to find advertised name %@ failed.", kBusObjectTestsAdvertisedName);
     
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried //for acceptance of the client joiner.");
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client //joined the session.");
-    STAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
-    STAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_shouldAcceptSessionJoinerNamed], @"The service did not report that it was queried //for acceptance of the client joiner.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_didJoinInSession], @"The service did not receive a notification that the client //joined the session.");
+    XCTAssertTrue(client.clientConnectionCompleted, @"The client did not report that it connected.");
+    XCTAssertTrue(client.testSessionId == self.testSessionId, @"The client session id does not match the service session id.");
     
     [basicObject sendTestStringPropertyChangedFrom:@"Hello World" to:@"Foo Bar" inSession:self.testSessionId toDestination:nil];
-    STAssertTrue([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
-    STAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
+    XCTAssertTrue([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
+    XCTAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
     self.didReceiveSignal = NO;
     client.didReceiveSignal = NO;
     
     [clientBasicObject sendTestStringPropertyChangedFrom:@"Hello World" to:@"Foo Bar" inSession:self.testSessionId toDestination:nil];
-    STAssertFalse([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
-    STAssertTrue([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
+    XCTAssertFalse([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
+    XCTAssertTrue([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
     self.didReceiveSignal = NO;
     client.didReceiveSignal = NO;
     
     [basicObject sendTestStringPropertyChangedFrom:@"Foo Bar" to:@"Bar Baz" inSession:kAJNSessionIdAllHosted toDestination:nil];
-    STAssertTrue([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
-    STAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
+    XCTAssertTrue([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was not called.");
+    XCTAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
     self.didReceiveSignal = NO;
     client.didReceiveSignal = NO;
     
     [clientBasicObject sendTestStringPropertyChangedFrom:@"Foo Bar" to:@"Bar Baz" inSession:kAJNSessionIdAllHosted toDestination:nil];
-    STAssertFalse([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
-    STAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
+    XCTAssertFalse([client waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
+    XCTAssertFalse([self waitForSignalToBeReceived:kBusObjectTestsWaitTimeBeforeFailure], @"The signal handler was called.");
     self.didReceiveSignal = NO;
     client.didReceiveSignal = NO;
     
     status = [client.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Client disconnect from bus via null transport failed.");
     status = [self.bus disconnectWithArguments:@"null:"];
-    STAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
+    XCTAssertTrue(status == ER_OK, @"Disconnect from bus via null transport failed.");
     
     status = [client.bus stop];
-    STAssertTrue(status == ER_OK, @"Client bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Client bus failed to stop.");
     status = [self.bus stop];
-    STAssertTrue(status == ER_OK, @"Bus failed to stop.");
+    XCTAssertTrue(status == ER_OK, @"Bus failed to stop.");
     
-    STAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");    
-    STAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");    
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");
+    XCTAssertTrue([self waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The bus listener should have been notified that the bus is stopping.");    
+    XCTAssertTrue([client waitForBusToStop:kBusObjectTestsWaitTimeBeforeFailure], @"The client bus listener should have been notified that the bus is stopping.");    
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_busDidDisconnectCompleted], @"The bus listener should have been notified that the bus was disconnected.");
     
     [client.bus unregisterBusListener:client];
     [self.bus unregisterBusListener:self];
-    STAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
+    XCTAssertTrue([self waitForCompletion:kBusObjectTestsWaitTimeBeforeFailure onFlag:&_listenerDidUnregisterWithBusCompleted], @"The bus listener should have been notified that a listener was unregistered.");
     
     [client tearDown];    
 }
@@ -454,7 +454,7 @@ const NSInteger kBusObjectTestsServicePort = 999;
             AJNSessionOptions *sessionOptions = [[AJNSessionOptions alloc] initWithTrafficType:kAJNTrafficMessages supportsMultipoint:NO proximity:kAJNProximityAny transportMask:kAJNTransportMaskAny];
             
             self.testSessionId = [self.bus joinSessionWithName:name onPort:kBusObjectTestsServicePort withDelegate:self options:sessionOptions];
-            STAssertTrue(self.testSessionId != -1, @"Test client failed to connect to the service %@ on port %u", name, kBusObjectTestsServicePort);
+            XCTAssertTrue(self.testSessionId != -1, @"Test client failed to connect to the service %@ on port %u", name, kBusObjectTestsServicePort);
             
             self.clientConnectionCompleted = YES;
         }
