@@ -32,6 +32,8 @@
 
 #include <qcc/CryptoECCMath.h>
 
+#include <qcc/CryptoECCp256.h>
+
 #include <Status.h>
 
 using namespace std;
@@ -45,7 +47,7 @@ namespace qcc {
 /* These values describe why the verify failed.  This simplifies testing. */
 typedef enum {
     V_SUCCESS = 0, V_R_ZERO, V_R_BIG, V_S_ZERO, V_S_BIG,
-    V_INFINITY, V_UNEQUAL
+    V_INFINITY, V_UNEQUAL, V_INTERNAL
 } verify_res_t;
 
 struct ECDSASig {
@@ -74,33 +76,38 @@ struct ECCSecret::ECCSecretState {
     ECCSecretState() {
         memset(z, 0, ECC_COORDINATE_SZ);
     }
+    ~ECCSecretState()
+    {
+        ClearMemory(z, sizeof(z));
+    }
     void operator=(const ECCSecretState& k)
     {
         memcpy(z, k.z, ECC_COORDINATE_SZ);
     }
     bool operator==(const ECCSecretState& k) const
     {
-        return memcmp(z, k.z, ECC_COORDINATE_SZ) == 0;
+        return Crypto_Compare(z, k.z, ECC_COORDINATE_SZ) == 0;
     }
 };
 
 #define ECDSA
 
 /*
- * The exported ECDH derive function.  Differs from ECHD_derive_pt
+ * The exported ECDH derive function.  Differs from ECDH_derive_pt
  * only in that it returns just the X coordinate of the derived point.
  * The ECDH_derive functionality is split in two so that the test
  * program can get the entire point.
  */
-boolean_t ECDH_derive(bigval_t* tgt, bigval_t const* k, affine_point_t const* Q)
+bool ECDH_derive(bigval_t* tgt, bigval_t const* k, affine_point_t const* Q)
 {
     affine_point_t Q2;
-    boolean_t rv;
+    bool rv;
 
     rv = ECDH_derive_pt(&Q2, k, Q);
     if (rv) {
         *tgt = Q2.x;
     }
+    ClearMemory(&Q2, sizeof(affine_point_t));
     return (rv);
 }
 
@@ -138,6 +145,9 @@ startpoint:
         goto startpoint;
     }
 
+    ClearMemory(k.data, ECC_BIGVAL_SZ);
+    ClearMemory(t.data, ECC_BIGVAL_SZ);
+
     return (0);
 }
 
@@ -158,27 +168,55 @@ static verify_res_t ECDSA_verify_inner(bigval_t const* msgdgst,
        unions between the affine and jacobian versions of points. But
        check that out before doing it. */
 
+    verify_res_t res;
     bigval_t v;
     bigval_t w;
     bigval_t u1;
     bigval_t u2;
-    affine_point_t P1;
-    affine_point_t P2;
-    affine_point_t X;
-    jacobian_point_t P2Jacobian;
-    jacobian_point_t XJacobian;
+    digit256_t digU1;
+    digit256_t digU2;
+    ecpoint_t Q;
+    ecpoint_t P1;
+    ecpoint_t P2;
+    ecpoint_t G;
+    ecpoint_t X;
+    ec_t curve;
+
+    bool status;
+    QStatus ajstatus;
+
+    ajstatus = ec_getcurve(&curve, NISTP256r1);
+    if (ajstatus != ER_OK) {
+        /* curve has already been free'd */
+        res = V_INTERNAL;
+        goto Exit;
+    }
+
+    ec_get_generator(&G, &curve);
+
+    status = bigval_to_digit256(&(pubkey->x), Q.x);
+    status = status && bigval_to_digit256(&(pubkey->y), Q.y);
+    status = status && ecpoint_validation(&Q, &curve);
+    if (!status) {
+        res = V_INTERNAL;
+        goto Exit;
+    }
 
     if (big_cmp(&sig->r, &big_one) < 0) {
-        return (V_R_ZERO);
+        res = V_R_ZERO;
+        goto Exit;
     }
     if (big_cmp(&sig->r, &orderP) >= 0) {
-        return(V_R_BIG);
+        res = V_R_BIG;
+        goto Exit;
     }
     if (big_cmp(&sig->s, &big_one) < 0) {
-        return (V_S_ZERO);
+        res = V_S_ZERO;
+        goto Exit;
     }
     if (big_cmp(&sig->s, &orderP) >= 0) {
-        return(V_S_BIG);
+        res = V_S_BIG;
+        goto Exit;
     }
 
     big_divide(&w, &big_one, &sig->s, &orderP);
@@ -186,19 +224,39 @@ static verify_res_t ECDSA_verify_inner(bigval_t const* msgdgst,
     big_precise_reduce(&u1, &u1, &orderP);
     big_mpyP(&u2, &sig->r, &w, MOD_ORDER);
     big_precise_reduce(&u2, &u2, &orderP);
-    pointMpyP(&P1, &u1, &base_point);
-    pointMpyP(&P2, &u2, pubkey);
-    toJacobian(&P2Jacobian, &P2);
-    pointAdd(&XJacobian, &P2Jacobian, &P1);
-    toAffine(&X, &XJacobian);
-    if (X.infinity) {
-        return (V_INFINITY);
+
+    status = bigval_to_digit256(&u1, digU1);
+    status = status && bigval_to_digit256(&u2, digU2);
+    if (!status) {
+        res = V_INTERNAL;
+        goto Exit;
     }
-    big_precise_reduce(&v, &X.x, &orderP);
+
+    ec_scalarmul(&(curve.generator), digU1, &P1, &curve);
+    ec_scalarmul(&Q, digU2, &P2, &curve);
+
+    // copy P1 to X
+    fpcopy_p256(P1.x, X.x);
+    fpcopy_p256(P1.y, X.y);
+
+    ec_add(&X, &P2, &curve);
+
+    if (ec_is_infinity(&X, &curve)) {
+        res = V_INFINITY;
+        goto Exit;
+    }
+
+    digit256_to_bigval(X.x, &v);
     if (big_cmp(&v, &sig->r) != 0) {
-        return (V_UNEQUAL);
+        res = V_UNEQUAL;
+        goto Exit;
     }
-    return (V_SUCCESS);
+
+    res = V_SUCCESS;
+
+Exit:
+    ec_freecurve(&curve);
+    return res;
 }
 
 bool ECDSA_verify(bigval_t const* msgdgst,
@@ -279,7 +337,6 @@ Exit:
 
 ECCSecret::~ECCSecret()
 {
-    memset(eccSecretState, 0, sizeof(ECCSecretState));
     delete eccSecretState;
 }
 
@@ -299,10 +356,24 @@ static QStatus Crypto_ECC_GenerateKeyPair(ECCPublicKey* publicKey, ECCPrivateKey
     if (ECDH_generate(&ap, &k) != 0) {
         return ER_FAIL;
     }
-    bigval_to_binary(&ap.x, publicKey->x, sizeof(publicKey->x));
-    bigval_to_binary(&ap.y, publicKey->y, sizeof(publicKey->y));
-    bigval_to_binary(&k, privateKey->d, sizeof(privateKey->d));
-    return ER_OK;
+    const size_t coordinateSize = publicKey->GetCoordinateSize();
+    const size_t privateKeySize = privateKey->GetSize();
+    uint8_t* X = new uint8_t[coordinateSize];
+    uint8_t* Y = new uint8_t[coordinateSize];
+    uint8_t* D = new uint8_t[privateKeySize];
+    bigval_to_binary(&ap.x, X, coordinateSize);
+    bigval_to_binary(&ap.y, Y, coordinateSize);
+    bigval_to_binary(&k, D, privateKeySize);
+
+    QStatus status = publicKey->Import(X, coordinateSize, Y, coordinateSize);
+    if (ER_OK == status) {
+        status = privateKey->Import(D, privateKeySize);
+    }
+    delete[] X;
+    delete[] Y;
+    qcc::ClearMemory(D, privateKeySize);
+    delete[] D;
+    return status;
 }
 
 QStatus Crypto_ECC::GenerateDHKeyPair() {
@@ -320,9 +391,9 @@ QStatus Crypto_ECC::GenerateSharedSecret(const ECCPublicKey* peerPublicKey, ECCS
     QStatus status = ER_FAIL;
 
     pub.infinity = 0;
-    binary_to_bigval(peerPublicKey->x, &pub.x, sizeof(peerPublicKey->x));
-    binary_to_bigval(peerPublicKey->y, &pub.y, sizeof(peerPublicKey->y));
-    binary_to_bigval(eccState->dhPrivateKey.d, &prv, sizeof(eccState->dhPrivateKey.d));
+    binary_to_bigval(peerPublicKey->GetX(), &pub.x, peerPublicKey->GetCoordinateSize());
+    binary_to_bigval(peerPublicKey->GetY(), &pub.y, peerPublicKey->GetCoordinateSize());
+    binary_to_bigval(eccState->dhPrivateKey.GetD(), &prv, eccState->dhPrivateKey.GetDSize());
     if (!ECDH_derive(&sec, &prv, &pub)) {
         goto Exit;
     }
@@ -337,10 +408,9 @@ QStatus Crypto_ECC::GenerateSharedSecret(const ECCPublicKey* peerPublicKey, ECCS
 
 Exit:
 
-    memset((void*)&eccSecretState, 0, sizeof(ECCSecret::ECCSecretState));   // could be optimized out need a secure zero memory
-
-    memset(sec.data, 0, sizeof(sec.data));
-    memset(prv.data, 0, sizeof(prv.data));
+    ClearMemory(&eccSecretState, sizeof(ECCSecret::ECCSecretState));
+    ClearMemory(sec.data, sizeof(sec.data));
+    ClearMemory(prv.data, sizeof(prv.data));
 
     return status;
 }
@@ -370,15 +440,25 @@ static QStatus Crypto_ECC_DSASignDigest(const uint8_t* digest, uint32_t len, con
     ECC_hash_to_bigval(&source, digest, len);
     bigval_t privKey;
     ECDSA_sig_t localSig;
+    QStatus Status = ER_FAIL;
 
-    binary_to_bigval(signingPrivateKey->d, &privKey, sizeof(signingPrivateKey->d));
+
+    binary_to_bigval(signingPrivateKey->GetD(), &privKey, signingPrivateKey->GetDSize());
     if (ECDSA_sign(&source, &privKey, &localSig) != 0) {
-        return ER_FAIL;
+        Status = ER_FAIL;
+        goto Exit;
     }
     bigval_to_binary(&localSig.r, sig->r, sizeof(sig->r));
     bigval_to_binary(&localSig.s, sig->s, sizeof(sig->s));
 
-    return ER_OK;
+
+    Status = ER_OK;
+
+Exit:
+
+    ClearMemory(privKey.data, ECC_BIGVAL_SZ);
+
+    return Status;
 }
 
 /*
@@ -433,8 +513,8 @@ static QStatus Crypto_ECC_DSAVerifyDigest(const uint8_t* digest, uint32_t len, c
     ECDSA_sig_t localSig;
 
     pub.infinity = 0;
-    binary_to_bigval(signingPubKey->x, &pub.x, sizeof(signingPubKey->x));
-    binary_to_bigval(signingPubKey->y, &pub.y, sizeof(signingPubKey->y));
+    binary_to_bigval(signingPubKey->GetX(), &pub.x, signingPubKey->GetCoordinateSize());
+    binary_to_bigval(signingPubKey->GetY(), &pub.y, signingPubKey->GetCoordinateSize());
     binary_to_bigval(sig->r, &localSig.r, sizeof(sig->r));
     binary_to_bigval(sig->s, &localSig.s, sizeof(sig->s));
 
@@ -527,38 +607,114 @@ Crypto_ECC::~Crypto_ECC()
     delete eccState;
 }
 
+/** ECCPublicKey **/
+
+/**
+ * Empty ECC coordinate
+ */
+static const uint8_t ECC_COORDINATE_EMPTY[ECC_COORDINATE_SZ] = { 0 };
+
+bool ECCPublicKey::empty() const
+{
+    return (memcmp(x, ECC_COORDINATE_EMPTY, GetCoordinateSize()) == 0) &&
+           (memcmp(y, ECC_COORDINATE_EMPTY, GetCoordinateSize()) == 0);
+}
+
 const qcc::String ECCPublicKey::ToString() const
 {
     qcc::String s = "x=[";
-    s.append(BytesToHexString(x, ECC_COORDINATE_SZ));
+    s.append(BytesToHexString(x, GetCoordinateSize()));
     s.append("], y=[");
-    s.append(BytesToHexString(y, ECC_COORDINATE_SZ));
+    s.append(BytesToHexString(y, GetCoordinateSize()));
     s.append("]");
     return s;
 }
 
 QStatus ECCPublicKey::Export(uint8_t* data, size_t* size) const
 {
-    if (data == NULL || size == NULL || *size < (ECC_COORDINATE_SZ + ECC_COORDINATE_SZ)) {
+    const size_t coordinateSize = GetCoordinateSize();
+
+    if (data == NULL || size == NULL || *size < (coordinateSize + coordinateSize)) {
         return ER_FAIL;
     }
 
-    memcpy(data, x, ECC_COORDINATE_SZ);
-    memcpy(data + ECC_COORDINATE_SZ, y, ECC_COORDINATE_SZ);
-    *size = ECC_COORDINATE_SZ + ECC_COORDINATE_SZ;
+    memcpy(data, x, coordinateSize);
+    memcpy(data + coordinateSize, y, coordinateSize);
+    *size = coordinateSize + coordinateSize;
     return ER_OK;
 }
 
 QStatus ECCPublicKey::Import(const uint8_t* data, size_t size)
 {
-    if ((size != (ECC_COORDINATE_SZ + ECC_COORDINATE_SZ)) || (!data)) {
-        return ER_FAIL;
+    if (NULL == data) {
+        return ER_BAD_ARG_1;
+    }
+    if (size != GetSize()) {
+        return ER_BAD_ARG_2;
     }
 
-    memcpy(x, data, ECC_COORDINATE_SZ);
-    memcpy(y, data + ECC_COORDINATE_SZ, ECC_COORDINATE_SZ);
+    const size_t coordinateSize = GetCoordinateSize();
+
+    memcpy(x, data, coordinateSize);
+    memcpy(y, data + coordinateSize, coordinateSize);
 
     return ER_OK;
 }
 
+QStatus ECCPublicKey::Import(const uint8_t* xData, const size_t xSize, const uint8_t* yData, const size_t ySize)
+{
+    if (NULL == xData) {
+        return ER_BAD_ARG_1;
+    }
+    if (this->GetCoordinateSize() != xSize) {
+        return ER_BAD_ARG_2;
+    }
+    if (NULL == yData) {
+        return ER_BAD_ARG_3;
+    }
+    if (this->GetCoordinateSize() != ySize) {
+        return ER_BAD_ARG_4;
+    }
+
+    memcpy(x, xData, xSize);
+    memcpy(y, yData, ySize);
+
+    return ER_OK;
+}
+
+/** ECCPrivateKey **/
+
+ECCPrivateKey::~ECCPrivateKey()
+{
+    ClearMemory(d, ECC_COORDINATE_SZ);
+}
+
+QStatus ECCPrivateKey::Export(uint8_t* data, size_t* size) const
+{
+    if (NULL == data) {
+        return ER_BAD_ARG_1;
+    }
+    if (NULL == size) {
+        return ER_BAD_ARG_2;
+    }
+    if (this->GetSize() > *size) {
+        *size = this->GetSize();
+        return ER_BUFFER_TOO_SMALL;
+    }
+
+    *size = this->GetSize();
+    memcpy(data, d, *size);
+
+    return ER_OK;
+}
+
+const String ECCPrivateKey::ToString() const
+{
+    qcc::String s = "d=[";
+    s.append(BytesToHexString(d, GetSize()));
+    s.append("]");
+    return s;
+}
+
+
 } /* namespace qcc */
