@@ -31,7 +31,6 @@
 #include <qcc/KeyBlob.h>
 #include <qcc/Stream.h>
 #include <qcc/String.h>
-#include <qcc/StringUtil.h>
 
 
 namespace qcc {
@@ -309,13 +308,11 @@ class Crypto_Hash {
 
     /// Typedef for abstracting the hash algorithm specifier.
     typedef enum {
-        SHA1,    ///< SHA1 algorithm specifier
-        MD5,     ///< MD5 algorithm specifier
-        SHA256   ///< SHA256 algorithm specifier
+        SHA1,          ///< SHA1 algorithm specifier
+        SHA256         ///< SHA256 algorithm specifier
     } Algorithm;
 
     static const size_t SHA1_SIZE = 20;   ///< SHA1 digest size - 20 bytes == 160 bits
-    static const size_t MD5_SIZE = 16;    ///< MD5 digest size - 16 bytes == 128 bits
     static const size_t SHA256_SIZE = 32; ///< SHA256 digest size - 32 bytes == 256 bits
 
     /**
@@ -364,19 +361,6 @@ class Crypto_SHA1 : public Crypto_Hash {
     static const size_t DIGEST_SIZE = Crypto_Hash::SHA1_SIZE;
 };
 
-/**
- * MD5 hash calculation interface abstraction class.
- */
-class Crypto_MD5 : public Crypto_Hash {
-  public:
-    virtual QStatus Init(const uint8_t* hmacKey = NULL, size_t keyLen = 0)
-    {
-        return Crypto_Hash::Init(MD5, hmacKey, keyLen);
-    }
-
-    static const size_t DIGEST_SIZE = Crypto_Hash::MD5_SIZE;
-};
-
 /**
  * SHA256 hash calculation interface abstraction class.
  */
@@ -753,7 +737,9 @@ class Crypto_ASN1 {
     static const uint8_t ASN_ASCII     = 0x16;
     static const uint8_t ASN_UTC_TIME  = 0x17;
     static const uint8_t ASN_GEN_TIME  = 0x18;
-    static const uint8_t ASN_CONTEXT_SPECIFIC = 0xA0;
+    static const uint8_t ASN_CONTEXT_SPECIFIC = 0x80;
+    static const uint8_t ASN_CONTEXT_SPECIFIC_CONSTRUCTED = 0xA0;
+    static const uint8_t ASN_CONSTRUCTED_ENCODING = 0x20;
 
     static QStatus DecodeV(const char*& syntax, const uint8_t* asn, size_t asnLen, va_list* argpIn);
 
@@ -794,13 +780,14 @@ class Crypto_DRBG : public Crypto_Rand {
     static const size_t KEYLEN = Crypto_AES::AES128_SIZE;
     static const size_t OUTLEN = sizeof (Crypto_AES::Block);
     static const size_t SEEDLEN = KEYLEN + OUTLEN;
+    static const uint32_t RESEED_COUNT = 0x80000000;
     Crypto_DRBG();
     virtual QStatus Seed(uint8_t* seed, size_t size);
     virtual QStatus Generate(uint8_t* rand, size_t size);
     virtual ~Crypto_DRBG();
 
   private:
-    void Update(uint8_t* data, size_t size);
+    void Update(uint8_t data[SEEDLEN]);
 
     /**
      * Opaque type for the internal context
@@ -813,6 +800,19 @@ class Crypto_DRBG : public Crypto_Rand {
     Context* ctx;
 };
 
+/**
+ * Compare two buffers in constant time. For any two inputs buf1 and buf2, and
+ * fixed count, the function will use the same number of cycles.
+ *
+ * @param buf1  The first buffer to compare.
+ * @param buf2  The second buffer to compare.
+ * @param count The number of bytes to compare.
+ *
+ * @return 0  if the first count bytes of buf1 and buf2 are equal, nonzero otherwise.
+ *
+ */
+int Crypto_Compare(const void* buf1, const void* buf2, size_t count);
+
 }
 
 #endif
