@@ -26,6 +26,7 @@
 #include <qcc/Debug.h>
 #include <qcc/Crypto.h>
 #include <qcc/StringUtil.h>
+#include <qcc/Util.h>
 
 #include <Status.h>
 
@@ -239,7 +240,7 @@ QStatus Crypto_ASN1::EncodeV(const char*& syntax, qcc::String& asn, va_list* arg
                     //Calculate the unused bits. bitLen 3 results in 5 unused bits
                     size_t unusedBits = (8 - bitLen) & 7;
                     //assure we have enough space to store the bits, if bitLen is not a multitude of 8.
-                    size_t len = (bitLen + 7) / 8;
+                    len = BitlenToBytelen(bitLen);
                     asn.push_back((char)ASN_BITS);
                     //we write 1 extra byte to indicate how many unused bits there are in the string
                     EncodeLen(asn, len + 1);
@@ -335,7 +336,7 @@ QStatus Crypto_ASN1::EncodeV(const char*& syntax, qcc::String& asn, va_list* arg
                     if (*syntax++ != ')') {
                         status = ER_FAIL;
                     } else if (status == ER_OK) {
-                        asn.push_back((char) (ASN_CONTEXT_SPECIFIC | v));
+                        asn.push_back((char) (ASN_CONTEXT_SPECIFIC_CONSTRUCTED | v));
                         EncodeLen(asn, seq.size());
                         asn += seq;
                     }
@@ -534,9 +535,10 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
                     status = ER_FAIL;
                     continue;
                 }
-                if ((tag != (ASN_CONTEXT_SPECIFIC | v)) || !DecodeLen(asn, eod, len)) {
+                if ((ASN_CONTEXT_SPECIFIC != (tag & ASN_CONTEXT_SPECIFIC)) ||
+                    ((uint8_t) (tag & 0x1F) != v) || !DecodeLen(asn, eod, len)) {
                     status = ER_FAIL;
-                } else {
+                } else if (ASN_CONSTRUCTED_ENCODING == (tag & ASN_CONSTRUCTED_ENCODING)) {
                     qcc::String seq;
                     status = DecodeV(syntax, asn, len, &argp);
                     if (*syntax++ != ')') {
@@ -544,6 +546,19 @@ QStatus Crypto_ASN1::DecodeV(const char*& syntax, const uint8_t* asn, size_t asn
                     } else if (status == ER_OK) {
                         asn += len;
                     }
+                } else {
+                    /* primitive content */
+                    while (*syntax && (*syntax != ')')) {
+                        syntax++;
+                    }
+                    if (*syntax == ')') {
+                        /* skip to the end of the c(...) block */
+                        status = ER_OK;
+                        syntax++;
+                        break; /* no more parsing */
+                    } else {
+                        status = ER_FAIL;
+                    }
                 }
             }
             continue;
