@@ -51,6 +51,15 @@ const qcc::String OID_DN_OU = "2.5.4.11";
 const qcc::String OID_DN_CN = "2.5.4.3";
 const qcc::String OID_BASIC_CONSTRAINTS = "2.5.29.19";
 const qcc::String OID_DIG_SHA256 = "2.16.840.1.101.3.4.2.1";
+const qcc::String OID_SUB_ALTNAME = "2.5.29.17";
+const qcc::String OID_EKU = "2.5.29.37";
+const qcc::String OID_CUSTOM_EKU_IDENTITY = "1.3.6.1.4.1.44924.1.1";
+const qcc::String OID_CUSTOM_EKU_MEMBERSHIP = "1.3.6.1.4.1.44924.1.5";
+const qcc::String OID_CUSTOM_DIGEST = "1.3.6.1.4.1.44924.1.2";
+const qcc::String OID_CUSTOM_SECURITY_GROUP_ID = "1.3.6.1.4.1.44924.1.3";
+const qcc::String OID_CUSTOM_IDENTITY_ALIAS = "1.3.6.1.4.1.44924.1.4";
+const qcc::String OID_AUTHORITY_KEY_IDENTIFIER = "2.5.29.35";
+
 
 static const char* EC_PRIVATE_KEY_PEM_BEGIN_TAG = "-----BEGIN EC PRIVATE KEY-----";
 static const char* EC_PRIVATE_KEY_PEM_END_TAG = "-----END EC PRIVATE KEY-----";
@@ -100,31 +109,48 @@ static QStatus StripTags(String& pem, const char* beg, const char* end)
     return ER_OK;
 }
 
-QStatus AJ_CALL CertificateX509::EncodePrivateKeyPEM(const uint8_t* privateKey, size_t len, String& encoded)
+QStatus AJ_CALL CertificateX509::EncodePrivateKeyPEM(const ECCPrivateKey* privateKey, String& encoded)
 {
     QStatus status;
     qcc::String beg = EC_PRIVATE_KEY_PEM_BEGIN_TAG;
     qcc::String end = EC_PRIVATE_KEY_PEM_END_TAG;
     qcc::String der;
-    qcc::String prv((const char*) privateKey, len);
+    qcc::String prv;
     qcc::String oid = OID_CRV_PRIME256V1;
     qcc::String pem;
 
+    size_t privateKeyLength = privateKey->GetSize();
+    uint8_t* privateKeyBytes = new uint8_t[privateKeyLength];
+
+    status = privateKey->Export(privateKeyBytes, &privateKeyLength);
+    if (ER_OK != status) {
+        QCC_LogError(status, ("Error exporting private key"));
+        goto Exit;
+    }
+
+    prv.assign((const char*)privateKeyBytes, privateKeyLength);
     status = Crypto_ASN1::Encode(der, "(ixc(o))", 1, &prv, 0, &oid);
     if (ER_OK != status) {
         QCC_LogError(status, ("Error encoding private key in PEM format"));
-        return status;
+        goto Exit;
     }
     status = Crypto_ASN1::EncodeBase64(der, pem);
     if (ER_OK != status) {
-        return status;
+        goto Exit;
     }
     encoded = beg + "\n" + pem + end;
 
-    return ER_OK;
+Exit:
+
+    if (NULL != privateKeyBytes) {
+        qcc::ClearMemory(privateKeyBytes, privateKeyLength);
+        delete[] privateKeyBytes;
+    }
+
+    return status;
 }
 
-QStatus AJ_CALL CertificateX509::DecodePrivateKeyPEM(const String& encoded, uint8_t* privateKey, size_t len)
+QStatus AJ_CALL CertificateX509::DecodePrivateKeyPEM(const String& encoded, ECCPrivateKey* privateKey)
 {
     QStatus status;
     qcc::String pem = encoded;
@@ -165,15 +191,11 @@ QStatus AJ_CALL CertificateX509::DecodePrivateKeyPEM(const String& encoded, uint
             return ER_FAIL;
         }
     }
-    if (len != prv.size()) {
-        return ER_FAIL;
-    }
-    memcpy(privateKey, prv.data(), len);
 
-    return ER_OK;
+    return privateKey->Import((const uint8_t*) prv.data(), prv.size());
 }
 
-QStatus AJ_CALL CertificateX509::EncodePublicKeyPEM(const uint8_t* publicKey, size_t len, String& encoded)
+QStatus AJ_CALL CertificateX509::EncodePublicKeyPEM(const ECCPublicKey* publicKey, String& encoded)
 {
     QStatus status;
     qcc::String beg = PUBLIC_KEY_PEM_BEGIN_TAG;
@@ -183,9 +205,20 @@ QStatus AJ_CALL CertificateX509::EncodePublicKeyPEM(const uint8_t* publicKey, si
     qcc::String oid2 = OID_CRV_PRIME256V1;
     qcc::String pem;
 
+    size_t publicKeySize = publicKey->GetSize();
+    uint8_t* publicKeyBytes = new uint8_t[publicKeySize];
+
+    status = publicKey->Export(publicKeyBytes, &publicKeySize);
+    if (ER_OK != status) {
+        QCC_LogError(status, ("Failed to export public key bytes"));
+        delete[] publicKeyBytes;
+        return status;
+    }
+
     // Uncompressed points only
     qcc::String key(0x4);
-    key += qcc::String((const char*) publicKey, len);
+    key += qcc::String((const char*)publicKeyBytes, publicKeySize);
+    delete[] publicKeyBytes;
     status = Crypto_ASN1::Encode(der, "((oo)b)", &oid1, &oid2, &key, 8 * key.size());
     if (ER_OK != status) {
         return status;
@@ -199,7 +232,7 @@ QStatus AJ_CALL CertificateX509::EncodePublicKeyPEM(const uint8_t* publicKey, si
     return ER_OK;
 }
 
-QStatus AJ_CALL CertificateX509::DecodePublicKeyPEM(const String& encoded, uint8_t* publicKey, size_t len)
+QStatus AJ_CALL CertificateX509::DecodePublicKeyPEM(const String& encoded, ECCPublicKey* publicKey)
 {
     QStatus status;
     qcc::String pem = encoded;
@@ -224,23 +257,73 @@ QStatus AJ_CALL CertificateX509::DecodePublicKeyPEM(const String& encoded, uint8
         return status;
     }
     if (OID_KEY_ECC != oid1) {
+        QCC_LogError(ER_FAIL, ("First OID did not match ECC oid"));
         return ER_FAIL;
     }
     if (OID_CRV_PRIME256V1 != oid2) {
-        return ER_FAIL;
-    }
-    if (1 + len != key.size()) {
+        QCC_LogError(ER_FAIL, ("Second OID did not match P-256 OID"));
         return ER_FAIL;
     }
     // Uncompressed points only
     if (0x4 != *key.data()) {
+        QCC_LogError(ER_FAIL, ("Key data is not in uncompressed format; other formats are not supported"));
         return ER_FAIL;
     }
-    memcpy(publicKey, key.data() + 1, len);
 
-    return ER_OK;
+    return publicKey->Import((const uint8_t*)(key.data() + 1), key.size() - 1);
 }
 
+/* To be deprecated versions of {Encode|Decode}{Private|Public}KeyPEM */
+QStatus AJ_CALL CertificateX509::EncodePrivateKeyPEM(const uint8_t* privateKey, size_t len, String& encoded)
+{
+    ECCPrivateKey keyObject;
+
+    QStatus status = keyObject.Import(privateKey, len);
+    if (ER_OK != status) {
+        return status;
+    }
+    return CertificateX509::EncodePrivateKeyPEM(&keyObject, encoded);
+}
+
+QStatus AJ_CALL CertificateX509::DecodePrivateKeyPEM(const String& encoded, uint8_t* privateKey, size_t len)
+{
+    ECCPrivateKey keyObject;
+
+    QStatus status = CertificateX509::DecodePrivateKeyPEM(encoded, &keyObject);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return keyObject.Export(privateKey, &len);
+}
+
+QStatus AJ_CALL EncodePublicKeyPEM(const uint8_t* publicKey, size_t len, String& encoded)
+{
+    ECCPublicKey keyObject;
+
+    QStatus status = keyObject.Import(publicKey, len);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return CertificateX509::EncodePublicKeyPEM(&keyObject, encoded);
+}
+
+QStatus AJ_CALL DecodePublicKeyPEM(const String& encoded, uint8_t* publicKey, size_t len)
+{
+    ECCPublicKey keyObject;
+
+    QStatus status = CertificateX509::DecodePublicKeyPEM(encoded, &keyObject);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    return keyObject.Export(publicKey, &len);
+}
+/* End of to-be-deprecated functions */
+
+
+
 QStatus CertificateX509::DecodeCertificateName(const qcc::String& dn, CertificateX509::DistinguishedName& name)
 {
     QStatus status = ER_OK;
@@ -279,7 +362,7 @@ QStatus CertificateX509::DecodeCertificateName(const qcc::String& dn, Certificat
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificateName(qcc::String& dn, CertificateX509::DistinguishedName& name)
+QStatus CertificateX509::EncodeCertificateName(qcc::String& dn, const CertificateX509::DistinguishedName& name) const
 {
     qcc::String ouOID;
     qcc::String cnOID;
@@ -333,6 +416,7 @@ static QStatus DecodeTime(uint64_t& epoch, const qcc::String& t)
      *  savings time is in effect
      */
     int originalTmHour = tm.tm_hour;
+    int originalTmDay = tm.tm_mday;
 
     /* Compute the GMT time from struct tm.
         Can't use timegm since it is not available in some platforms like Android and Windows */
@@ -341,18 +425,23 @@ static QStatus DecodeTime(uint64_t& epoch, const qcc::String& t)
     if (localTime < 0) {
         return ER_FAIL;
     }
-    struct tm* gtm = ConvertTimeToStructure(&localTime);
-    if (!gtm) {
+    struct tm gtm;
+    if (ER_OK != ConvertTimeToStructure(&localTime, &gtm)) {
         return ER_FAIL;
     }
     /* figure the time zone offset */
-    int32_t tzDiff = gtm->tm_hour - originalTmHour;
+    int32_t tzDiff = gtm.tm_hour - originalTmHour;
     /* some time zones are at 30 minute or 45 minute boundary */
-    int32_t minuteDiff = gtm->tm_min - tm.tm_min;
+    int32_t minuteDiff = gtm.tm_min - tm.tm_min;
     if (tzDiff < -12) {
         tzDiff += 24;
     } else if (tzDiff > 12) {
-        tzDiff = 24 - tzDiff;
+        if (gtm.tm_mday == originalTmDay) {
+            /* same day */
+            tzDiff = 24 - tzDiff;
+        } else {
+            tzDiff = tzDiff - 24;
+        }
     }
     epoch = localTime - (tzDiff * 3600) - (minuteDiff * 60);
     return ER_OK;
@@ -389,6 +478,12 @@ QStatus CertificateX509::DecodeCertificateTime(const qcc::String& time)
         return ER_FAIL;
     }
 
+    char buf1[16];
+    char buf2[16];
+    memset(buf1, 0, 16);
+    memset(buf2, 0, 16);
+    memcpy(buf1, time1.data(), time1.size());
+    memcpy(buf2, time2.data(), time2.size());
     status = DecodeTime(validity.validFrom, time1);
     if (ER_OK != status) {
         return status;
@@ -403,8 +498,8 @@ QStatus CertificateX509::DecodeCertificateTime(const qcc::String& time)
 
 static QStatus EncodeTime(uint64_t epoch, qcc::String& t)
 {
-    struct tm* ptm = ConvertTimeToStructure((int64_t*)&epoch);
-    if (!ptm) {
+    struct tm ptm;
+    if (ER_OK != ConvertTimeToStructure((int64_t*)&epoch, &ptm)) {
         return ER_FAIL;
     }
     /**
@@ -413,10 +508,10 @@ static QStatus EncodeTime(uint64_t epoch, qcc::String& t)
      *      validity date in the year 2050 or later as UTC time YYYYMMDDHHMMSSZ
      * The value 150 means 2050 - 1900 where tm_year is based.
      */
-    const char* format = (ptm->tm_year < 150) ? "%y%m%d%H%M%SZ" : "%Y%m%d%H%M%SZ";
+    const char* format = (ptm.tm_year < 150) ? "%y%m%d%H%M%SZ" : "%Y%m%d%H%M%SZ";
     char buf[16];
     size_t ret;
-    ret = FormatTime(buf, sizeof(buf), format, ptm);
+    ret = FormatTime(buf, sizeof(buf), format, &ptm);
     if (!ret) {
         return ER_FAIL;
     }
@@ -424,7 +519,7 @@ static QStatus EncodeTime(uint64_t epoch, qcc::String& t)
     return ER_OK;
 }
 
-QStatus CertificateX509::EncodeCertificateTime(qcc::String& time)
+QStatus CertificateX509::EncodeCertificateTime(qcc::String& time) const
 {
     QStatus status;
     qcc::String time1;
@@ -462,32 +557,40 @@ QStatus CertificateX509::DecodeCertificatePub(const qcc::String& pub)
     if (OID_CRV_PRIME256V1 != oid2) {
         return ER_FAIL;
     }
-    if (1 + sizeof (publickey) != key.size()) {
+    if (1 + publickey.GetSize() != key.size()) {
         return ER_FAIL;
     }
     // Uncompressed points only
     if (0x4 != *key.data()) {
         return ER_FAIL;
     }
-    memcpy((uint8_t*) &publickey, key.data() + 1, key.size() - 1);
+    status = publickey.Import((const uint8_t*)(key.data() + 1), key.size() - 1);
 
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificatePub(qcc::String& pub)
+QStatus CertificateX509::EncodeCertificatePub(qcc::String& pub) const
 {
     QStatus status = ER_OK;
     qcc::String oid1 = OID_KEY_ECC;
     qcc::String oid2 = OID_CRV_PRIME256V1;
 
-    // Uncompressed points only
-    qcc::String key(0x4);
-    key += qcc::String((const char*) &publickey, sizeof (publickey));
-    status = Crypto_ASN1::Encode(pub, "(oo)b", &oid1, &oid2, &key, 8 * key.size());
+    size_t publicKeySize = publickey.GetSize();
+    uint8_t* publicKeyBytes = new uint8_t[publicKeySize];
+
+    status = publickey.Export(publicKeyBytes, &publicKeySize);
     if (ER_OK != status) {
+        QCC_LogError(status, ("Failed to export public key bytes"));
+        delete[] publicKeyBytes;
         return status;
     }
 
+    // Uncompressed points only
+    qcc::String key(0x4);
+    key += qcc::String((const char*) publicKeyBytes, publicKeySize);
+    status = Crypto_ASN1::Encode(pub, "(oo)b", &oid1, &oid2, &key, 8 * key.size());
+
+    delete[] publicKeyBytes;
     return status;
 }
 
@@ -525,13 +628,94 @@ QStatus CertificateX509::DecodeCertificateExt(const qcc::String& ext)
                     return status;
                 }
             }
+        } else if (OID_EKU == oid) {
+            bool identityEKUPresent = false;
+            bool membershipEKUPresent = false;
+
+            status = Crypto_ASN1::Decode(str, "(.)", &tmp);
+            if (ER_OK != status) {
+                return status;
+            }
+            /* There has to be at least one EKU listed per the RFC. If tmp is empty,
+             * the cert is malformed. */
+            if (0 == tmp.size()) {
+                return ER_FAIL;
+            }
+            while ((ER_OK == status) && (tmp.size() > 0)) {
+                qcc::String eku;
+                qcc::String rem;
+                status = Crypto_ASN1::Decode(tmp, "o.", &eku, &rem);
+                if (ER_OK == status) {
+                    if (OID_CUSTOM_EKU_IDENTITY == eku) {
+                        if (identityEKUPresent) {
+                            QCC_DbgPrintf(("Identity EKU seen multiple times"));
+                        }
+                        identityEKUPresent = true;
+                    } else if (OID_CUSTOM_EKU_MEMBERSHIP == eku) {
+                        if (membershipEKUPresent) {
+                            QCC_DbgPrintf(("Membership EKU seen multiple times"));
+                        }
+                        membershipEKUPresent = true;
+                    } else {
+                        /* An unrelated EKU. Unexpected but not fatal. Log and move on. */
+                        QCC_DbgPrintf(("Informational: Saw unexpected EKU %s", eku.c_str()));
+                    }
+
+                    tmp = rem;
+                }
+            }
+
+            if (ER_OK != status) {
+                QCC_LogError(status, ("Failed while decoding EKU extension: %s", QCC_StatusText(status)));
+                return status;
+            }
+
+            if (identityEKUPresent && membershipEKUPresent) {
+                type = UNRESTRICTED_CERTIFICATE;
+            } else if (identityEKUPresent) {
+                type = IDENTITY_CERTIFICATE;
+            } else if (membershipEKUPresent) {
+                type = MEMBERSHIP_CERTIFICATE;
+            } else {
+                /* EKUs are present but none belonging to known AllJoyn EKUs, and so certificate is not valid
+                 * for any AllJoyn purpose. */
+                type = INVALID_CERTIFICATE;
+            }
+        } else if (OID_SUB_ALTNAME == oid) {
+            qcc::String otherNameOid;
+            qcc::String otherName;
+            /* only interested in the otherName field [0] */
+            if (ER_OK == Crypto_ASN1::Decode(str, "(c(o.))", 0, &otherNameOid, &otherName)) {
+                if ((OID_CUSTOM_SECURITY_GROUP_ID == otherNameOid) || (OID_CUSTOM_IDENTITY_ALIAS == otherNameOid)) {
+                    status = Crypto_ASN1::Decode(otherName, "c(x)", 0, &subjectAltName);
+                    if (ER_OK != status) {
+                        return status;
+                    }
+                }
+            }
+        } else if (OID_CUSTOM_DIGEST == oid) {
+            qcc::String dig;
+            oid.clear();
+            status = Crypto_ASN1::Decode(str, "(ox)", &oid, &dig);
+            if (ER_OK != status) {
+                return status;
+            }
+            if (OID_DIG_SHA256 != oid) {
+                return ER_INVALID_DATA;
+            }
+            digest = dig;
+        } else if (OID_AUTHORITY_KEY_IDENTIFIER == oid) {
+            status = Crypto_ASN1::Decode(str, "(c(.))", 0, &aki);
+            if (ER_OK != status) {
+                return status;
+            }
         }
         tmp = rem;
     }
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificateExt(qcc::String& ext)
+QStatus CertificateX509::EncodeCertificateExt(qcc::String& ext) const
 {
     QStatus status = ER_OK;
     qcc::String oid;
@@ -551,9 +735,97 @@ QStatus CertificateX509::EncodeCertificateExt(qcc::String& ext)
     oid = OID_BASIC_CONSTRAINTS;
     status = Crypto_ASN1::Encode(raw, "(ox)", &oid, &tmp);
     if (ER_OK != status) {
-        QCC_LogError(status, ("Error decoding certificate basic constraint"));
+        QCC_LogError(status, ("Error encoding certificate basic constraint"));
         return status;
     }
+
+    tmp.clear();
+    switch (type) {
+    case UNRESTRICTED_CERTIFICATE:
+        status = Crypto_ASN1::Encode(tmp, "(oo)", &OID_CUSTOM_EKU_IDENTITY, &OID_CUSTOM_EKU_MEMBERSHIP);
+        if (ER_OK != status) {
+            return status;
+        }
+        break;
+
+    case IDENTITY_CERTIFICATE:
+        status = Crypto_ASN1::Encode(tmp, "(o)", &OID_CUSTOM_EKU_IDENTITY);
+        if (ER_OK != status) {
+            return status;
+        }
+        break;
+
+    case MEMBERSHIP_CERTIFICATE:
+        status = Crypto_ASN1::Encode(tmp, "(o)", &OID_CUSTOM_EKU_MEMBERSHIP);
+        if (ER_OK != status) {
+            return status;
+        }
+        break;
+
+    default: /* Shouldn't ever happen. */
+        assert(false);
+        QCC_LogError(ER_FAIL, ("Encountered unknown type %u", type));
+        return ER_FAIL;
+    }
+
+    oid = OID_EKU;
+    status = Crypto_ASN1::Encode(raw, "(ox)", &oid, &tmp);
+    if (ER_OK != status) {
+        return status;
+    }
+
+    if (subjectAltName.size()) {
+        qcc::String octet;
+        if ((type == IDENTITY_CERTIFICATE) || (type == MEMBERSHIP_CERTIFICATE)) {
+            qcc::String otherNameOid;
+            if (type == IDENTITY_CERTIFICATE) {
+                otherNameOid = OID_CUSTOM_IDENTITY_ALIAS;
+            } else {
+                otherNameOid = OID_CUSTOM_SECURITY_GROUP_ID;
+            }
+            status = Crypto_ASN1::Encode(octet, "(c(oc(x)))", 0, &otherNameOid, 0, &subjectAltName);
+            if (ER_OK != status) {
+                return status;
+            }
+        } else {
+            octet = subjectAltName;
+        }
+        oid = OID_SUB_ALTNAME;
+        status = Crypto_ASN1::Encode(raw, "(ox)", &oid, &octet);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
+    if (aki.size()) {
+        String tmp;
+        status = Crypto_ASN1::Encode(tmp, "c(R)", 0, &aki);
+        if (ER_OK != status) {
+            return status;
+        }
+        String akiOctet;
+        status = Crypto_ASN1::Encode(akiOctet, "(R)", &tmp);
+        if (ER_OK != status) {
+            return status;
+        }
+        oid = OID_AUTHORITY_KEY_IDENTIFIER;
+        status = Crypto_ASN1::Encode(raw, "(ox)", &oid, &akiOctet);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
+    if (digest.size()) {
+        qcc::String dig;
+        oid = OID_DIG_SHA256;
+        status = Crypto_ASN1::Encode(dig, "(ox)", &oid, &digest);
+        if (ER_OK != status) {
+            return status;
+        }
+        oid = OID_CUSTOM_DIGEST;
+        status = Crypto_ASN1::Encode(raw, "(ox)", &oid, &dig);
+        if (ER_OK != status) {
+            return status;
+        }
+    }
     status = Crypto_ASN1::Encode(ext, "c((R))", 3, &raw);
 
     return status;
@@ -569,9 +841,10 @@ QStatus CertificateX509::DecodeCertificateTBS()
     qcc::String time;
     qcc::String pub;
     qcc::String ext;
+    qcc::String serialStr;
 
     status = Crypto_ASN1::Decode(tbs, "(c(i)l(o)(.)(.)(.)(.).)",
-                                 0, &x509Version, &serial, &oid, &iss, &time, &sub, &pub, &ext);
+                                 0, &x509Version, &serialStr, &oid, &iss, &time, &sub, &pub, &ext);
     if (ER_OK != status) {
         QCC_LogError(status, ("Error decoding certificate"));
         return status;
@@ -580,6 +853,7 @@ QStatus CertificateX509::DecodeCertificateTBS()
         QCC_LogError(status, ("Certificate not X.509v3"));
         return ER_FAIL;
     }
+    this->SetSerial((const uint8_t*)serialStr.data(), serialStr.size());
     if (OID_SIG_ECDSA_SHA256 != oid) {
         QCC_LogError(status, ("Certificate signature must be SHA-256"));
         return ER_FAIL;
@@ -622,6 +896,7 @@ QStatus CertificateX509::EncodeCertificateTBS()
     qcc::String time;
     qcc::String pub;
     qcc::String ext;
+    qcc::String serialStr;
 
     status = EncodeCertificateName(iss, issuer);
     if (ER_OK != status) {
@@ -643,8 +918,10 @@ QStatus CertificateX509::EncodeCertificateTBS()
     if (ER_OK != status) {
         return status;
     }
+    serialStr.assign((const char*)serial, serialLen);
+    tbs.clear(tbs.size()); //empty the tbs string.
     status = Crypto_ASN1::Encode(tbs, "(c(i)l(o)(R)(R)(R)(R)R)",
-                                 0, x509Version, &serial, &oid, &iss, &time, &sub, &pub, &ext);
+                                 0, x509Version, &serialStr, &oid, &iss, &time, &sub, &pub, &ext);
 
     return status;
 }
@@ -679,7 +956,7 @@ QStatus CertificateX509::DecodeCertificateSig(const qcc::String& sig)
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificateSig(qcc::String& sig)
+QStatus CertificateX509::EncodeCertificateSig(qcc::String& sig) const
 {
     QStatus status = ER_OK;
     qcc::String r((const char*) signature.r, sizeof (signature.r));
@@ -706,6 +983,7 @@ QStatus CertificateX509::DecodeCertificateDER(const qcc::String& der)
         return status;
     }
     // Put the sequence back on the TBS
+    tbs.clear(tbs.size());
     status = Crypto_ASN1::Encode(tbs, "(R)", &tmp);
     if (ER_OK != status) {
         return status;
@@ -725,7 +1003,7 @@ QStatus CertificateX509::DecodeCertificateDER(const qcc::String& der)
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificateDER(qcc::String& der)
+QStatus CertificateX509::EncodeCertificateDER(qcc::String& der) const
 {
     QStatus status;
     qcc::String oid = OID_SIG_ECDSA_SHA256;
@@ -791,7 +1069,7 @@ QStatus AJ_CALL CertificateX509::EncodeCertificatePEM(qcc::String& der, qcc::Str
     return status;
 }
 
-QStatus CertificateX509::EncodeCertificatePEM(qcc::String& pem)
+QStatus CertificateX509::EncodeCertificatePEM(qcc::String& pem) const
 {
     qcc::String der;
     QStatus status = EncodeCertificateDER(der);
@@ -801,7 +1079,7 @@ QStatus CertificateX509::EncodeCertificatePEM(qcc::String& pem)
     return EncodeCertificatePEM(der, pem);
 }
 
-QStatus CertificateX509::VerifyValidity()
+QStatus CertificateX509::VerifyValidity() const
 {
     uint64_t currentTime = GetEpochTimestamp() / 1000;
 
@@ -812,18 +1090,31 @@ QStatus CertificateX509::VerifyValidity()
     return ER_OK;
 }
 
-QStatus CertificateX509::Verify()
+QStatus CertificateX509::Verify() const
 {
     return Verify(&publickey);
 }
 
-QStatus CertificateX509::Verify(const ECCPublicKey* key)
+QStatus CertificateX509::Verify(const ECCPublicKey* key) const
 {
+    if (key->empty()) {
+        return ER_FAIL;
+    }
     Crypto_ECC ecc;
     ecc.SetDSAPublicKey(key);
     return ecc.DSAVerify((const uint8_t*) tbs.data(), tbs.size(), &signature);
 }
 
+QStatus CertificateX509::Verify(const KeyInfoNISTP256& ta) const
+{
+    QStatus status;
+    status = VerifyValidity();
+    if (ER_OK != status) {
+        QCC_DbgPrintf(("Invalid validity period"));
+        return status;
+    }
+    return Verify(ta.GetPublicKey());
+}
 
 QStatus CertificateX509::Sign(const ECCPrivateKey* key)
 {
@@ -837,10 +1128,44 @@ QStatus CertificateX509::Sign(const ECCPrivateKey* key)
     return ecc.DSASign((const uint8_t*) tbs.data(), tbs.size(), &signature);
 }
 
+QStatus AJ_CALL CertificateX509::GenerateAuthorityKeyId(const ECCPublicKey* issuerPubKey, String& authorityKeyId)
+{
+    Crypto_SHA256 sha;
+    uint8_t digest[Crypto_SHA256::DIGEST_SIZE];
+    sha.Init();
+    sha.Update((const uint8_t*) issuerPubKey, sizeof(ECCPublicKey));
+    sha.GetDigest(digest);
+    uint8_t keyId[AUTHORITY_KEY_ID_SZ];
+    /* the format of the authority key identifier is 64 bits:
+     * first 4 bits is 0100
+     * the next 60 bits is the least significant 60 bits of the digest
+     */
+    memcpy(keyId, &digest[Crypto_SHA256::DIGEST_SIZE - AUTHORITY_KEY_ID_SZ], AUTHORITY_KEY_ID_SZ);
+    keyId[0] = 0x40 | (keyId[0] & 0xF);
+    authorityKeyId.assign((const char*) keyId, sizeof(keyId));
+    return ER_OK;
+}
+
+QStatus CertificateX509::GenerateAuthorityKeyId(const ECCPublicKey* issuerPubKey)
+{
+    return GenerateAuthorityKeyId(issuerPubKey, aki);
+}
+
+QStatus CertificateX509::SignAndGenerateAuthorityKeyId(const ECCPrivateKey* privateKey, const ECCPublicKey* publicKey)
+{
+    QStatus status = GenerateAuthorityKeyId(publicKey);
+    if (ER_OK != status) {
+        return status;
+    }
+    return Sign(privateKey);
+}
+
 String CertificateX509::ToString() const
 {
     qcc::String str("Certificate:\n");
-    str += "serial:    " + serial + " (0x" + BytesToHexString((const uint8_t*) serial.data(), serial.length()) + ")\n";
+    /* Printing out the serial number as a string can produce control characters that cause output issues
+     * on Windows. Sometimes the rest of the output will not appear. Instead, we only print it as a hex string. */
+    str += "serial:     0x" + BytesToHexString(serial, serialLen) + "\n";
     if ((GetIssuerOULength() > 0) || (GetIssuerCNLength() > 0)) {
         str += "issuer: ";
         bool addComma = false;
@@ -876,7 +1201,7 @@ String CertificateX509::ToString() const
         }
         str += "\n";
     }
-    str += "publickey: " + BytesToHexString((const uint8_t*) &publickey, sizeof(publickey)) + "\n";
+    str += "publickey: " + publickey.ToString() + "\n";
     str += "ca:        " + BytesToHexString((const uint8_t*) &ca, sizeof(uint8_t)) + "\n";
     str += "validity: not-before ";
     str += U64ToString(GetValidity()->validFrom);
@@ -888,51 +1213,26 @@ String CertificateX509::ToString() const
     EncodeTime(GetValidity()->validTo, tmpTime);
     str += " (" + tmpTime + ") ";
     str += "\n";
+    if (digest.size()) {
+        str += "digest:    " + BytesToHexString((const uint8_t*) digest.data(), digest.size()) + "\n";
+    }
+    if (subjectAltName.size()) {
+        if (GetType() == IDENTITY_CERTIFICATE) {
+            str += "alias:     " + subjectAltName;
+        } else if (GetType() == MEMBERSHIP_CERTIFICATE) {
+            str += "security group Id:     0x" + BytesToHexString((const uint8_t*) subjectAltName.data(), subjectAltName.size());
+        } else {
+            str += "subject alt name:     " + subjectAltName;
+        }
+        str += "\n";
+    }
+    if (aki.size()) {
+        str += "aki:     " + BytesToHexString((const uint8_t*) aki.data(), aki.size()) + "\n";
+    }
     str += "signature: " + BytesToHexString((const uint8_t*) &signature, sizeof(signature)) + "\n";
     return str;
 }
 
-QStatus CertificateX509::GenEncoded()
-{
-    delete [] encoded;
-    encoded = NULL;
-    encodedLen = 0;
-    String der;
-    QStatus status = EncodeCertificateDER(der);
-    if (ER_OK != status) {
-        return status;
-    }
-    encodedLen = der.length();
-    encoded = new uint8_t[encodedLen];
-    if (!encoded) {
-        return ER_OUT_OF_MEMORY;
-    }
-    memcpy(encoded, der.data(), encodedLen);
-    return ER_OK;
-}
-
-const uint8_t* CertificateX509::GetEncoded()
-{
-    if (encodedLen == 0) {
-        GenEncoded();
-    }
-    return encoded;
-}
-
-size_t CertificateX509::GetEncodedLen()
-{
-    if (encodedLen == 0) {
-        GenEncoded();
-    }
-    return encodedLen;
-}
-
-QStatus CertificateX509::LoadEncoded(const uint8_t* encodedBytes, size_t len)
-{
-    String der((const char*) encodedBytes, len);
-    return DecodeCertificateDER(der);
-}
-
 String CertificateX509::GetPEM()
 {
     String pem;
@@ -966,4 +1266,81 @@ QStatus AJ_CALL CertificateX509::DecodeCertChainPEM(const String& encoded, Certi
     return status;
 }
 
+bool CertificateX509::IsIssuerOf(const CertificateX509& issuedCertificate) const
+{
+    return ((this->IsDNEqual(issuedCertificate.GetIssuerCN(), issuedCertificate.GetIssuerCNLength(),
+                             issuedCertificate.GetIssuerOU(), issuedCertificate.GetIssuerOULength())) &&
+            (ER_OK == issuedCertificate.Verify(this->GetSubjectPublicKey())));
+}
+
+bool CertificateX509::IsDNEqual(const CertificateX509& other) const
+{
+    return (this->IsDNEqual(other.GetSubjectCN(), other.GetSubjectCNLength(), other.GetSubjectOU(), other.GetSubjectOULength()));
+}
+
+bool CertificateX509::IsDNEqual(const uint8_t* cn, const size_t cnLength, const uint8_t* ou, const size_t ouLength) const
+{
+    /* Check lengths are equal. */
+    if ((this->GetSubjectCNLength() != cnLength) || (this->GetSubjectOULength() != ouLength)) {
+        return false;
+    }
+
+    /* For CN and OU, if either is NULL, they must be equal (both NULL). */
+    if ((NULL == this->GetSubjectCN()) || (NULL == cn)) {
+        if (this->GetSubjectCN() != cn) {
+            return false;
+        }
+    }
+
+    if ((NULL == this->GetSubjectOU()) || (NULL == ou)) {
+        if (this->GetSubjectOU() != ou) {
+            return false;
+        }
+    }
+
+    /* At this point either both are NULL or neither is, for each pair of CNs and OUs. */
+    if ((NULL != this->GetSubjectCN()) &&
+        (0 != memcmp(this->GetSubjectCN(), cn, cnLength))) {
+        return false;
+    }
+
+    if ((NULL != this->GetSubjectOU()) &&
+        (0 != memcmp(this->GetSubjectOU(), ou, ouLength))) {
+        return false;
+    }
+
+    return true;
+}
+
+bool CertificateX509::IsSubjectPublicKeyEqual(const ECCPublicKey* publicKey) const
+{
+    return ((*(this->GetSubjectPublicKey())) == (*publicKey));
+}
+
+bool CertificateX509::ValidateCertificateTypeInCertChain(const CertificateX509* certChain, size_t count)
+{
+    if ((count == 0) || !certChain) {
+        return false;
+    }
+    /* leaf cert must have a type */
+    if (certChain[0].GetType() == CertificateX509::UNRESTRICTED_CERTIFICATE) {
+        return false;
+    }
+    if (count == 1) {
+        return true;
+    }
+
+    /**
+     * Any signing cert in the chain must have the same type as the end entity certificate
+     * or be unrestricted in order to sign the next cert in the chain.
+     */
+    for (size_t cnt = 1; cnt < count; cnt++) {
+        if ((certChain[cnt].GetType() != CertificateX509::UNRESTRICTED_CERTIFICATE) &&
+            (certChain[cnt].GetType() != certChain[0].GetType())) {
+            return false;
+        }
+    }
+    return true;
+}
+
 }
